@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,9 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
+#include "core-ignite-cpu.h"
+#include "core-killpid.h"
 
 #define SETTING_SCALING_FREQ		(0x01)
 #define SETTING_CPUINFO_FREQ		(0x02)
@@ -66,7 +69,11 @@ static stress_settings_t settings[] = {
 	SETTING(NULL, NULL)
 };
 
-
+/*
+ *  stress_ignite_cpu_set()
+ *	attempt to apply settings, disable settings in settings_flag
+ *	if we cannot apply them.
+ */
 static void stress_ignite_cpu_set(
 	bool maximize_freq,
 	const int32_t cpu,
@@ -80,14 +87,14 @@ static void stress_ignite_cpu_set(
 	char buffer[128];
 
 	if (((*setting_flag & SETTING_FREQ) == SETTING_FREQ) && (max_freq > 0) && (min_freq <= max_freq)) {
-		uint64_t freq_delta = (max_freq - min_freq) / 10;
+		const uint64_t freq_delta = (max_freq - min_freq) / 10;
 		uint64_t freq;
 
 		(void)snprintf(path, sizeof(path),
 			"/sys/devices/system/cpu/cpu%" PRId32
 			"/cpufreq/scaling_max_freq", cpu);
 		(void)snprintf(buffer, sizeof(buffer), "%" PRIu64 "\n", max_freq);
-		if (system_write(path, buffer, strlen(buffer)) < 0)
+		if (stress_system_write(path, buffer, strlen(buffer)) < 0)
 			*setting_flag &= ~SETTING_SCALING_FREQ;
 
 		/* Try to set min to be 100% of max down to lowest, which ever works first*/
@@ -97,7 +104,7 @@ static void stress_ignite_cpu_set(
 		freq = (maximize_freq) ? max_freq : min_freq;
 		while ((freq_delta > 0) && (freq >= min_freq)) {
 			(void)snprintf(buffer, sizeof(buffer), "%" PRIu64 "\n", freq);
-			if (system_write(path, buffer, strlen(buffer)) >= 0)
+			if (stress_system_write(path, buffer, strlen(buffer)) >= 0)
 				break;
 			freq -= freq_delta;
 		}
@@ -108,7 +115,7 @@ static void stress_ignite_cpu_set(
 			"/sys/devices/system/cpu/cpu%" PRIu32
 			"/power/energy_perf_bias", cpu);
 		(void)snprintf(buffer, sizeof(buffer), "%" PRIu8 "\n", energy_perf_bias);
-		if (system_write(path, buffer, strlen(buffer)) < 0)
+		if (stress_system_write(path, buffer, strlen(buffer)) < 0)
 			*setting_flag &= ~SETTING_ENERGY_PERF_BIAS;
 	}
 
@@ -116,7 +123,7 @@ static void stress_ignite_cpu_set(
 		(void)snprintf(path, sizeof(path),
 			"/sys/devices/system/cpu/cpu%" PRIu32
 			"/cpufreq/scaling_governor", cpu);
-		if (system_write(path, governor, strlen(governor)) < 0)
+		if (stress_system_write(path, governor, strlen(governor)) < 0)
 			*setting_flag &= ~SETTING_GOVERNOR;
 	}
 }
@@ -164,22 +171,22 @@ void stress_ignite_cpu_start(void)
 			cpu_settings[cpu].setting_flag = 0;
 			cpu_settings[cpu].energy_perf_bias = -1;
 
-			(void)memset(buffer, 0, sizeof(buffer));
+			(void)shim_memset(buffer, 0, sizeof(buffer));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/cpufreq/scaling_max_freq", cpu);
-			ret = system_read(path, buffer, sizeof(buffer));
+			ret = stress_system_read(path, buffer, sizeof(buffer));
 			if (ret > 0) {
 				ret = sscanf(buffer, "%" SCNu64,
 					&cpu_settings[cpu].scaling_max_freq);
 				if (ret == 1)
 					cpu_settings[cpu].setting_flag |= SETTING_SCALING_FREQ;
 			}
-			(void)memset(buffer, 0, sizeof(buffer));
+			(void)shim_memset(buffer, 0, sizeof(buffer));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/cpufreq/scaling_min_freq", cpu);
-			ret = system_read(path, buffer, sizeof(buffer));
+			ret = stress_system_read(path, buffer, sizeof(buffer));
 			if (ret > 0) {
 				ret = sscanf(buffer, "%" SCNu64,
 					&cpu_settings[cpu].scaling_min_freq);
@@ -187,22 +194,22 @@ void stress_ignite_cpu_start(void)
 					cpu_settings[cpu].setting_flag &= ~SETTING_SCALING_FREQ;
 			}
 
-			(void)memset(buffer, 0, sizeof(buffer));
+			(void)shim_memset(buffer, 0, sizeof(buffer));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/cpufreq/cpuinfo_max_freq", cpu);
-			ret = system_read(path, buffer, sizeof(buffer));
+			ret = stress_system_read(path, buffer, sizeof(buffer));
 			if (ret > 0) {
 				ret = sscanf(buffer, "%" SCNu64,
 					&cpu_settings[cpu].cpuinfo_max_freq);
 				if (ret == 1)
 					cpu_settings[cpu].setting_flag |= SETTING_CPUINFO_FREQ;
 			}
-			(void)memset(buffer, 0, sizeof(buffer));
+			(void)shim_memset(buffer, 0, sizeof(buffer));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/cpufreq/cpuinfo_min_freq", cpu);
-			ret = system_read(path, buffer, sizeof(buffer));
+			ret = stress_system_read(path, buffer, sizeof(buffer));
 			if (ret > 0) {
 				ret = sscanf(buffer, "%" SCNu64,
 					&cpu_settings[cpu].cpuinfo_min_freq);
@@ -210,21 +217,21 @@ void stress_ignite_cpu_start(void)
 					cpu_settings[cpu].setting_flag &= ~SETTING_CPUINFO_FREQ;
 			}
 
-			(void)memset(cpu_settings[cpu].cur_governor, 0,
+			(void)shim_memset(cpu_settings[cpu].cur_governor, 0,
 				sizeof(cpu_settings[cpu].cur_governor));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/cpufreq/scaling_governor", cpu);
-			ret = system_read(path, cpu_settings[cpu].cur_governor,
+			ret = stress_system_read(path, cpu_settings[cpu].cur_governor,
 					  sizeof(cpu_settings[cpu].cur_governor));
 			if (ret > 0)
 				cpu_settings[cpu].setting_flag |= SETTING_GOVERNOR;
 
-			(void)memset(buffer, 0, sizeof(buffer));
+			(void)shim_memset(buffer, 0, sizeof(buffer));
 			(void)snprintf(path, sizeof(path),
 				"/sys/devices/system/cpu/cpu%" PRIu32
 				"/power/energy_perf_bias", cpu);
-			ret = system_read(path, buffer, sizeof(buffer));
+			ret = stress_system_read(path, buffer, sizeof(buffer));
 			if (ret > 0) {
 				int8_t bias;
 
@@ -244,7 +251,7 @@ void stress_ignite_cpu_start(void)
 		size_t len;
 
 		settings[i].ignore = true;
-		ret = system_read(settings[i].path, buf, sizeof(buf) - 1);
+		ret = stress_system_read(settings[i].path, buf, sizeof(buf) - 1);
 		if (ret < 0)
 			continue;
 		buf[ret] = '\0';
@@ -255,7 +262,7 @@ void stress_ignite_cpu_start(void)
 		settings[i].default_setting_len =
 			strlen(settings[i].default_setting);
 		/* If we can't update the setting, skip it */
-		ret = system_write(settings[i].path,
+		ret = stress_system_write(settings[i].path,
 			settings[i].default_setting,
 			settings[i].default_setting_len);
 		if (ret < 0) {
@@ -289,26 +296,27 @@ void stress_ignite_cpu_start(void)
 	} else if (pid == 0) {
 		/* Child */
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
-		stress_set_proc_name("stress-ng-ignite");
+		stress_set_proc_state_str("ignite","periodic");
 
-		while (keep_stressing_flag()) {
+		while (stress_continue_flag()) {
 			for (i = 0; settings[i].path; i++) {
 				if (settings[i].ignore)
 					continue;
-				(void)system_write(settings[i].path,
+				(void)stress_system_write(settings[i].path,
 					settings[i].default_setting,
 					settings[i].default_setting_len);
 			}
 
 			if (cpu_settings) {
 				int32_t cpu;
+				const char *governor = (stress_mwc8() < 240) ? "performance" : "powersave";
 
 				/*
 				 *  Attempt to crank CPUs up to max freq
 				 */
 				for (cpu = 0; cpu < max_cpus; cpu++) {
+
 					if (cpu_settings[cpu].setting_flag == 0)
 						continue;
 
@@ -316,16 +324,13 @@ void stress_ignite_cpu_start(void)
 						cpu_settings[cpu].cpuinfo_max_freq,
 						cpu_settings[cpu].cpuinfo_min_freq,
 						0,
-						"performance",
+						governor,
 						&cpu_settings[cpu].setting_flag);
 				}
 			}
 			(void)sleep(1);
 		}
 		_exit(0);
-	} else {
-		/* Parent */
-		(void)setpgid(pid, g_pgrp);
 	}
 }
 
@@ -336,17 +341,13 @@ void stress_ignite_cpu_start(void)
 void stress_ignite_cpu_stop(void)
 {
 	size_t i;
-	int status;
 
 	if (latency_fd != -1) {
 		(void)close(latency_fd);
 		latency_fd = -1;
 	}
-	if (pid > -1) {
-		(void)kill(pid, SIGTERM);
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
-	}
+	if (pid > -1)
+		(void)stress_kill_pid_wait(pid, NULL);
 
 	if (cpu_settings) {
 		int32_t cpu;
@@ -366,7 +367,7 @@ void stress_ignite_cpu_stop(void)
 		if (settings[i].ignore)
 			continue;
 
-		(void)system_write(settings[i].path, settings[i].setting,
+		(void)stress_system_write(settings[i].path, settings[i].setting,
 			settings[i].setting_len);
 		free(settings[i].setting);
 		settings[i].setting = NULL;
