@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King.
+ * Copyright (C) 2021-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-lock.h"
 #include "core-perf.h"
 #include "core-perf-event.h"
 
@@ -78,17 +80,19 @@ typedef struct {
 #define PERF_INFO_SW(config, label)	\
 	{ PERF_TYPE_SOFTWARE, PERF_COUNT_ ## config, NULL, label }
 
+#define PERF_INFO_HW_CACHE_CONFIG(cache_id, op_id, result_id)	\
+	  (PERF_COUNT_HW_CACHE_ ## cache_id) |			\
+	  ((PERF_COUNT_HW_CACHE_OP_ ## op_id) << 8) |		\
+	  ((PERF_COUNT_HW_CACHE_RESULT_ ## result_id) << 16)	\
+
 /* Hardware Cache */
 #define PERF_INFO_HW_C(cache_id, op_id, result_id, label)	\
 	{ PERF_TYPE_HW_CACHE, 					\
-	  (PERF_COUNT_HW_CACHE_ ## cache_id) |			\
-	  ((PERF_COUNT_HW_CACHE_OP_ ## op_id) << 8) |		\
-	  ((PERF_COUNT_HW_CACHE_RESULT_ ## result_id) << 16),	\
+	  PERF_INFO_HW_CACHE_CONFIG(cache_id, op_id, result_id),\
 	  NULL, label }
 
 #define STRESS_PERF_DEFINED(x) STRESS_PERF_COUNT_ ## x
 
-
 /*
  *  Perf scaling factors
  */
@@ -108,7 +112,7 @@ static const stress_perf_scale_t perf_scale[] = {
 /* perf counters to be read */
 static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	/*
-	 *  Hardware conters
+	 *  Hardware counters
 	 */
 #if STRESS_PERF_DEFINED(HW_CPU_CYCLES)
 	PERF_INFO_HW(HW_CPU_CYCLES,		"CPU Cycles"),
@@ -126,7 +130,7 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_HW(HW_STALLED_CYCLES_FRONTEND,"Stalled Cycles Frontend"),
 #endif
 #if STRESS_PERF_DEFINED(HW_STALLED_CYCLES_BACKEND)
-	PERF_INFO_HW(HW_STALLED_CYCLES_BACKEND,"Stalled Cycles Backend"),
+	PERF_INFO_HW(HW_STALLED_CYCLES_BACKEND,	"Stalled Cycles Backend"),
 #endif
 #if STRESS_PERF_DEFINED(HW_BUS_CYCLES)
 	PERF_INFO_HW(HW_BUS_CYCLES,		"Bus Cycles"),
@@ -134,7 +138,6 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 #if STRESS_PERF_DEFINED(HW_REF_CPU_CYCLES)
 	PERF_INFO_HW(HW_REF_CPU_CYCLES,		"Total Cycles"),
 #endif
-
 #if STRESS_PERF_DEFINED(HW_CACHE_REFERENCES)
 	PERF_INFO_HW(HW_CACHE_REFERENCES,	"Cache References"),
 #endif
@@ -187,7 +190,7 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_HW_C(ITLB, WRITE, ACCESS,	"Cache ITLB Write"),
 	PERF_INFO_HW_C(ITLB, WRITE, MISS,	"Cache ITLB Write Miss"),
 	PERF_INFO_HW_C(ITLB, PREFETCH, ACCESS,	"Cache ITLB Prefetch"),
-	PERF_INFO_HW_C(ITLB, PREFETCH, MISS,	"Cache DILB Prefetch Miss"),
+	PERF_INFO_HW_C(ITLB, PREFETCH, MISS,	"Cache IILB Prefetch Miss"),
 #endif
 
 #if STRESS_PERF_DEFINED(HW_CACHE_BPU)
@@ -241,7 +244,9 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 #if STRESS_PERF_DEFINED(SW_EMULATION_FAULTS)
 	PERF_INFO_SW(SW_EMULATION_FAULTS,	"Emulation Faults"),
 #endif
-
+#if STRESS_PERF_DEFINED(SW_CGROUP_SWITCHES)
+	PERF_INFO_SW(SW_CGROUP_SWITCHES,	"Cgroup Switches"),
+#endif
 	/*
 	 *  Tracepoint counters
  	 */
@@ -251,7 +256,9 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("raw_syscalls/sys_exit",		"System Call Exit"),
 
 	/* This perf metric causes 5.4+ kernel hangs, disable it for now */
-	/* PERF_INFO_TP("tlb/tlb_flush",		"TLB Flushes"), */
+#if 1
+	PERF_INFO_TP("tlb/tlb_flush",			"TLB Flushes"),
+#endif
 	PERF_INFO_TP("kmem/kmalloc",			"Kmalloc"),
 	PERF_INFO_TP("kmem/kmalloc_node",		"Kmalloc Node"),
 	PERF_INFO_TP("kmem/kfree",			"Kfree"),
@@ -261,10 +268,15 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("kmem/mm_page_alloc",		"MM Page Alloc"),
 	PERF_INFO_TP("kmem/mm_page_free",		"MM Page Free"),
 
-	PERF_INFO_TP("mmap_lock/mmap_lock_start_locking",	"MMAP lock start"),
+	PERF_INFO_TP("mmap_lock/mmap_lock_start_locking","MMAP lock start"),
 	PERF_INFO_TP("mmap_lock/mmap_lock_released",	"MMAP lock release"),
+#if 1
+	PERF_INFO_TP("mmap_lock/mmap_lock_acquire_returned","MMAP lock acquire"),
+#endif
 
 	PERF_INFO_TP("rcu/rcu_utilization",		"RCU Utilization"),
+	PERF_INFO_TP("rcu/rcu_stall_warning",		"RCU Stall Warning"),
+	PERF_INFO_TP("rcu/rcu_preempt_task",		"RCU Preempt Task"),
 
 	PERF_INFO_TP("sched/sched_migrate_task",	"Sched Migrate Task"),
 	PERF_INFO_TP("sched/sched_move_numa",		"Sched Move NUMA"),
@@ -277,6 +289,10 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("sched/sched_process_wait",	"Sched Proc Wait"),
 	PERF_INFO_TP("sched/sched_switch",		"Sched Switch"),
 
+	PERF_INFO_TP("task/task_newtask",		"New Task"),
+	PERF_INFO_TP("context_tracking/user_enter",	"Context User Enter"),
+	PERF_INFO_TP("context_tracking/user_exit",	"Context User Exit"),
+
 	PERF_INFO_TP("signal/signal_generate",		"Signal Generate"),
 	PERF_INFO_TP("signal/signal_deliver",		"Signal Deliver"),
 
@@ -286,13 +302,22 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("irq/softirq_exit",		"Soft IRQ Exit"),
 	PERF_INFO_TP("nmi/nmi_handler",			"NMI handler"),
 
-	PERF_INFO_TP("block/block_bio_complete",	"Block BIO complete"),
+	PERF_INFO_TP("ipi/ipi_entry",			"IPI Entry"),
+	PERF_INFO_TP("ipi/ipi_send_cpu",		"IPI Send CPU"),
+	PERF_INFO_TP("ipi/ipi_exit",			"IPI Exit"),
+
+	PERF_INFO_TP("block/block_bio_complete",	"Block BIO Complete"),
+#if 1
+	PERF_INFO_TP("iomap/iomap_readpage",		"IOMAP Read Page"),
+	PERF_INFO_TP("iomap/iomap_writepage",		"IOMAP Write Page"),
+#endif
 
 	PERF_INFO_TP("io_uring/io_uring_submit_sqe",	"IO uring submit"),
 	PERF_INFO_TP("io_uring/io_uring_complete",	"IO uring complete"),
 
 	PERF_INFO_TP("writeback/writeback_dirty_inode",	"Writeback Dirty Inode"),
 	PERF_INFO_TP("writeback/writeback_dirty_page",	"Writeback Dirty Page"),
+	PERF_INFO_TP("writeback/writeback_dirty_folio",	"Writeback Dirty Folio"),
 
 	PERF_INFO_TP("migrate/mm_migrate_pages",	"Migrate MM Pages"),
 
@@ -302,6 +327,12 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("lock/contention_begin",		"Lock Contention Begin"),
 	PERF_INFO_TP("lock/contention_end",		"Lock Contention End"),
 
+	PERF_INFO_TP("maple_tree/ma_read",		"Maple Tree Read"),
+	PERF_INFO_TP("maple_tree/ma_write",		"Maple Tree Write"),
+
+	PERF_INFO_TP("msr/read_msr",			"MSR read"),
+	PERF_INFO_TP("msr/write_msr",			"MSR write"),
+
 	PERF_INFO_TP("iommu/io_page_fault",		"IOMMU IO Page Fault"),
 	PERF_INFO_TP("iommu/map",			"IOMMU Map"),
 	PERF_INFO_TP("iommu/unmap",			"IOMMU Unmap"),
@@ -311,12 +342,24 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 
 	PERF_INFO_TP("oom/compact_retry",		"OOM Compact Retry"),
 	PERF_INFO_TP("oom/wake_reaper",			"OOM Wake Reaper"),
+	PERF_INFO_TP("oom/oom_score_adj_update",	"OOM Score Adjust Update"),
 
 	PERF_INFO_TP("thermal/thermal_zone_trip",	"Thermal Zone Trip"),
 
 	{ 0, 0, NULL, NULL }
 };
 
+static inline size_t stress_perf_info_find(const unsigned int type, const unsigned long config)
+{
+	size_t i;
+
+	for (i = 0; (i < STRESS_PERF_MAX) && perf_info[i].label; i++) {
+		if ((perf_info[i].type == type) && (perf_info[i].config == config))
+			return i;
+	}
+	return STRESS_PERF_MAX;
+}
+
 /*
  *  stress_perf_type_tracepoint_resolve_config()
  *	resolve tracing event config value
@@ -415,7 +458,7 @@ int stress_perf_open(stress_perf_t *sp)
 	if (g_shared->perf.no_perf)
 		return -1;
 
-	(void)memset(sp, 0, sizeof(*sp));
+	(void)shim_memset(sp, 0, sizeof(*sp));
 	sp->perf_opened = 0;
 
 	for (i = 0; i < STRESS_PERF_MAX; i++) {
@@ -423,11 +466,11 @@ int stress_perf_open(stress_perf_t *sp)
 		sp->perf_stat[i].counter = 0;
 	}
 
-	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
+	for (i = 0; (i < STRESS_PERF_MAX) && perf_info[i].label; i++) {
 		if (perf_info[i].config != UNRESOLVED) {
 			struct perf_event_attr attr;
 
-			(void)memset(&attr, 0, sizeof(attr));
+			(void)shim_memset(&attr, 0, sizeof(attr));
 			attr.type = perf_info[i].type;
 			attr.config = perf_info[i].config;
 			attr.disabled = 1;
@@ -478,7 +521,7 @@ int stress_perf_enable(stress_perf_t *sp)
 	if (!sp->perf_opened)
 		return 0;
 
-	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
+	for (i = 0; (i < STRESS_PERF_MAX) && perf_info[i].label; i++) {
 		const int fd = sp->perf_stat[i].fd;
 
 		if (fd > -1) {
@@ -511,7 +554,7 @@ int stress_perf_disable(stress_perf_t *sp)
 	if (!sp->perf_opened)
 		return 0;
 
-	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
+	for (i = 0; (i < STRESS_PERF_MAX) && perf_info[i].label; i++) {
 		const int fd = sp->perf_stat[i].fd;
 
 		if (fd > -1) {
@@ -541,7 +584,7 @@ int stress_perf_close(stress_perf_t *sp)
 	if (!sp->perf_opened)
 		goto out_ok;
 
-	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
+	for (i = 0; (i < STRESS_PERF_MAX) && perf_info[i].label; i++) {
 		const int fd = sp->perf_stat[i].fd;
 
 		if (fd < 0 ) {
@@ -549,7 +592,7 @@ int stress_perf_close(stress_perf_t *sp)
 			continue;
 		}
 
-		(void)memset(&data, 0, sizeof(data));
+		(void)shim_memset(&data, 0, sizeof(data));
 		ret = read(fd, &data, sizeof(data));
 		if (ret != sizeof(data))
 			sp->perf_stat[i].counter = STRESS_PERF_INVALID;
@@ -608,12 +651,63 @@ static const char *stress_perf_stat_scale(const uint64_t counter, const double d
 	}
 	scaled /= scale;
 
-	(void)snprintf(buffer, sizeof(buffer), "%11.2f %-5s",
+	(void)snprintf(buffer, sizeof(buffer), "%11.3f %-5s",
 		scaled, suffix);
 
 	return buffer;
 }
 
+/*
+ *  Compare type + config relative to another reference type and config
+ */
+typedef struct {
+	const unsigned int	type;
+	const unsigned long	config;
+	const unsigned int	ref_type;
+	const unsigned long	ref_config;
+	const bool		percent;	/* scale by 100.0 for percentages? */
+	const char 		*fmt;		/* snprintf format */
+} perf_relative_t;
+
+static const perf_relative_t perf_relatives[] = {
+	{ PERF_TYPE_HARDWARE,	PERF_COUNT_HW_INSTRUCTIONS,
+	  PERF_TYPE_HARDWARE,	PERF_COUNT_HW_CPU_CYCLES,
+	  false, " (%.3f instr. per cycle)" },
+	{ PERF_TYPE_HARDWARE,	PERF_COUNT_HW_CACHE_MISSES,
+	  PERF_TYPE_HARDWARE,	PERF_COUNT_HW_CACHE_REFERENCES,
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HARDWARE,	PERF_COUNT_HW_BRANCH_MISSES,
+	  PERF_TYPE_HARDWARE,	PERF_COUNT_HW_BRANCH_INSTRUCTIONS,
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(L1D, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(L1D, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(LL, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(LL, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(LL, WRITE, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(LL, WRITE, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(DTLB, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(DTLB, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(DTLB, WRITE, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(DTLB, WRITE, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(ITLB, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(ITLB, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(BPU, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(BPU, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(NODE, READ, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(NODE, READ, ACCESS),
+	  true, " (%6.3f%%)" },
+	{ PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(NODE, WRITE, MISS),
+	  PERF_TYPE_HW_CACHE,	PERF_INFO_HW_CACHE_CONFIG(NODE, WRITE, ACCESS),
+	  true, " (%6.3f%%)" },
+};
+
 /*
  *  stress_perf_stat_dump()
  *	emit perf statistics
@@ -632,23 +726,25 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 	for (ss = stressors_list; ss; ss = ss->next) {
 		int p;
 		uint64_t counter_totals[STRESS_PERF_MAX];
-		uint64_t total_cpu_cycles = 0;
-		uint64_t total_cache_refs = 0;
-		uint64_t total_branches = 0;
 		bool got_data = false;
-		char *munged;
+		char munged[64];
+		stress_perf_t *sp;
 
-		(void)memset(counter_totals, 0, sizeof(counter_totals));
+		if (ss->ignore.run)
+			continue;
+		if (!ss->stats)
+			continue;
+		sp = &ss->stats[0]->sp;
+		if (!stress_perf_stat_succeeded(sp))
+			continue;
+
+		(void)shim_memset(counter_totals, 0, sizeof(counter_totals));
 
 		/* Sum totals across all instances of the stressor */
-		for (p = 0; p < STRESS_PERF_MAX && perf_info[p].label; p++) {
+		for (p = 0; (p < STRESS_PERF_MAX) && perf_info[p].label; p++) {
 			int32_t j;
-			stress_perf_t *sp = &ss->stats[0]->sp;
 
-			if (!stress_perf_stat_succeeded(sp))
-				continue;
-
-			for (j = 0; j < ss->started_instances; j++) {
+			for (j = 0; j < ss->num_instances; j++) {
 				const uint64_t counter = sp->perf_stat[p].counter;
 
 				if (counter == STRESS_PERF_INVALID) {
@@ -658,27 +754,17 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 				counter_totals[p] += counter;
 				got_data |= (counter > 0);
 			}
-			if (perf_info[p].type == PERF_TYPE_HARDWARE) {
-				unsigned long config = perf_info[p].config;
-
-				if (config == PERF_COUNT_HW_CPU_CYCLES)
-					total_cpu_cycles = counter_totals[p];
-				else if (config == PERF_COUNT_HW_CACHE_REFERENCES)
-					total_cache_refs = counter_totals[p];
-				else if (config == PERF_COUNT_HW_BRANCH_INSTRUCTIONS)
-					total_branches = counter_totals[p];
-			}
 		}
 
 		if (!got_data)
 			continue;
 
-		munged = stress_munge_underscore(ss->stressor->name);
+		(void)stress_munge_underscore(munged, ss->stressor->name, sizeof(munged));
 		pr_inf("%s:\n", munged);
 		pr_yaml(yaml, "    - stressor: %s\n", munged);
 		pr_yaml(yaml, "      duration: %f\n", duration);
 
-		for (p = 0; p < STRESS_PERF_MAX && perf_info[p].label; p++) {
+		for (p = 0; (p < STRESS_PERF_MAX) && perf_info[p].label; p++) {
 			const char *l = perf_info[p].label;
 			const uint64_t ct = counter_totals[p];
 
@@ -686,26 +772,24 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 				char extra[32];
 				char yaml_label[128];
 				*extra = '\0';
+				size_t i;
 
 				no_perf_stats = false;
 
-				if (perf_info[p].type == PERF_TYPE_HARDWARE) {
-					unsigned long config = perf_info[p].config;
-					if ((config == PERF_COUNT_HW_INSTRUCTIONS) &&
-					    (total_cpu_cycles > 0))
-						(void)snprintf(extra, sizeof(extra),
-							" (%.3f instr. per cycle)",
-							(double)ct / (double)total_cpu_cycles);
-					else if ((config == PERF_COUNT_HW_CACHE_MISSES) &&
-					     (total_cache_refs > 0))
-						(void)snprintf(extra, sizeof(extra),
-							" (%5.2f%%)",
-							100.0 * (double)ct / (double)total_cache_refs);
-					else if ((config == PERF_COUNT_HW_BRANCH_MISSES) &&
-					    (total_branches > 0))
-						(void)snprintf(extra, sizeof(extra),
-							" (%5.2f%%)",
-							100.0 * (double)ct / (double)total_branches);
+				for (i = 0; i < SIZEOF_ARRAY(perf_relatives); i++) {
+					if ((perf_info[p].type == perf_relatives[i].type) &&
+					    (perf_info[p].config == perf_relatives[i].config)) {
+						const size_t idx = stress_perf_info_find(
+									perf_relatives[i].ref_type,
+									perf_relatives[i].ref_config);
+
+						if ((idx < STRESS_PERF_MAX) &&
+						    (counter_totals[idx] > 0))
+							(void)snprintf(extra, sizeof(extra),
+									perf_relatives[i].fmt,
+									(perf_relatives[i].percent ? 100.0 : 1.0) *
+									(double)ct / (double)counter_totals[idx]);
+					}
 				}
 
 				pr_inf("%'26" PRIu64 " %-24s %s%s\n",
@@ -730,7 +814,7 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 			int level = 0;
 			static const char *path = "/proc/sys/kernel/perf_event_paranoid";
 
-			ret = system_read(path, buffer, sizeof(buffer) - 1);
+			ret = stress_system_read(path, buffer, sizeof(buffer) - 1);
 			if (ret > 0) {
 				if (sscanf(buffer, "%5d", &level) == 1)
 					paranoid = true;
