@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-pthread.h"
 #include "core-resources.h"
 
 #if defined(HAVE_LIB_PTHREAD)
@@ -30,13 +34,107 @@ static void *stress_resources_pthread_func(void *ctxt)
 }
 #endif
 
+/*
+ *  stress_resources_init()
+ *	helper function to set resources to initial 'unallocated' values
+ */
+static void stress_resources_init(stress_resources_t *resources, const size_t num_resources)
+{
+	size_t i;
+
+	for (i = 0; i < num_resources; i++) {
+		resources[i].m_malloc = NULL;
+		resources[i].m_mmap = MAP_FAILED;
+		resources[i].pipe_ret = -1;
+		resources[i].fd_open = -1;
+#if defined(HAVE_EVENTFD)
+		resources[i].fd_ev = -1;
+#endif
+#if defined(HAVE_MEMFD_CREATE)
+		resources[i].fd_memfd = -1;
+		resources[i].ptr_memfd = NULL;
+#endif
+#if defined(__NR_memfd_secret)
+		resources[i].fd_memfd_secret = -1;
+		resources[i].ptr_memfd_secret = NULL;
+#endif
+		resources[i].fd_sock = -1;
+		resources[i].fd_socketpair[0] = -1;
+		resources[i].fd_socketpair[1] = -1;
+#if defined(HAVE_USERFAULTFD)
+		resources[i].fd_uf = -1;
+#endif
+#if defined(O_TMPFILE)
+		resources[i].fd_tmp = -1;
+#endif
+#if defined(HAVE_LIB_PTHREAD)
+		resources[i].pthread_ret = -1;
+		resources[i].pthread = (pthread_t)0;
+#if defined(HAVE_PTHREAD_MUTEX_T) &&	\
+    defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
+    defined(HAVE_PTHREAD_MUTEX_DESTROY)
+		(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
+		resources[i].mutex_ret = -1;
+#endif
+#endif
+
+#if defined(HAVE_LIB_RT) &&		\
+    defined(HAVE_TIMER_CREATE) &&	\
+    defined(HAVE_TIMER_DELETE) &&	\
+    defined(SIGUNUSED)
+		resources[i].timerok = false;
+#endif
+#if defined(HAVE_SYS_TIMERFD_H) &&	\
+    defined(HAVE_TIMERFD_CREATE) &&	\
+    defined(HAVE_TIMERFD_GETTIME) &&	\
+    defined(HAVE_TIMERFD_SETTIME) &&	\
+    defined(CLOCK_REALTIME)
+		resources[i].timer_fd = -1;
+#endif
+#if defined(HAVE_SYS_INOTIFY)
+		resources[i].wd_inotify = -1;
+		resources[i].fd_inotify = -1;
+#endif
+#if defined(HAVE_PTSNAME)
+		resources[i].pty = -1;
+		resources[i].pty_mtx = -1;
+#endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_SEM_POSIX)
+		resources[i].semok = false;
+#endif
+#if defined(HAVE_SEM_SYSV)
+		resources[i].sem_id = -1;
+#endif
+#if defined(HAVE_MQ_SYSV) &&	\
+    defined(HAVE_SYS_IPC_H) &&	\
+    defined(HAVE_SYS_MSG_H)
+		resources[i].msgq_id = -1;
+#endif
+#if defined(HAVE_LIB_RT) &&	\
+    defined(HAVE_MQ_POSIX) &&	\
+    defined(HAVE_MQUEUE_H)
+		resources[i].mq = -1;
+		resources[i].mq_name[0] = '\0';
+#endif
+#if defined(HAVE_PKEY_ALLOC) &&	\
+    defined(HAVE_PKEY_FREE)
+		resources[i].pkey = -1;
+#endif
+#if defined(HAVE_PIDFD_OPEN)
+		resources[i].pid_fd = -1;
+#endif
+		resources[i].pid = 0;
+	}
+}
+
 /*
  *  stress_resources_allocate()
  *	allocate a wide range of resources, perform num_resources
  *	resource allocations
  */
 size_t stress_resources_allocate(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_resources_t *resources,
         const size_t num_resources,
 	const size_t pipe_size,
@@ -54,6 +152,9 @@ size_t stress_resources_allocate(
 	static const int domains[] = { AF_INET, AF_INET6 };
 	static const int types[] = { SOCK_STREAM, SOCK_DGRAM };
 
+	stress_resources_init(resources, num_resources);
+	stress_ksm_memory_merge(1);
+
 	(void)pid;
 	(void)page_size;
 
@@ -82,7 +183,7 @@ size_t stress_resources_allocate(
 	if ((freemem > 0) && (freemem < min_mem_free))
 		return 0;
 
-	(void)memset(resources, 0, sizeof(*resources) * num_resources);
+	(void)shim_memset(resources, 0, sizeof(*resources) * num_resources);
 
 	for (i = 0; i < num_resources; i++) {
 #if defined(HAVE_MEMFD_CREATE)
@@ -114,11 +215,11 @@ size_t stress_resources_allocate(
 #endif
 #if defined(HAVE_LIB_PTHREAD)
 		resources[i].pthread_ret = -1;
-		resources[i].pthread = 0;
+		resources[i].pthread = (pthread_t)0;
 #if defined(HAVE_PTHREAD_MUTEX_T) &&	\
     defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
     defined(HAVE_PTHREAD_MUTEX_DESTROY)
-		(void)memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
+		(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
 		resources[i].mutex_ret = -1;
 #endif
 #endif
@@ -177,7 +278,7 @@ size_t stress_resources_allocate(
 		 */
 		n = i + 1;
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
@@ -186,19 +287,19 @@ size_t stress_resources_allocate(
 
 		if ((stress_mwc8() & 0xf) == 0) {
 			resources[i].m_malloc = calloc(1, page_size);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 		if ((stress_mwc8() & 0xf) == 0) {
 			resources[i].m_sbrk = shim_sbrk((intptr_t)page_size);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 		if ((stress_mwc8() & 0xf) == 0) {
 			resources[i].m_mmap_size = page_size * 2;
 			resources[i].m_mmap = mmap(NULL, resources[i].m_mmap_size,
 				PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			if (resources[i].m_mmap != MAP_FAILED) {
 				const size_t locked = STRESS_MINIMUM(mlock_size, resources[i].m_mmap_size);
@@ -209,6 +310,7 @@ size_t stress_resources_allocate(
 					shim_mlock(resources[i].m_mmap, locked);
 					mlock_size -= locked;
 				}
+				(void)stress_madvise_mergeable(resources[i].m_mmap, resources[i].m_mmap_size);
 			}
 		}
 		if (resources[i].m_mmap)
@@ -223,22 +325,30 @@ size_t stress_resources_allocate(
 		UNEXPECTED
 		(void)pipe_size;
 #endif
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		resources[i].fd_open = open("/dev/null", O_RDONLY);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #if defined(HAVE_EVENTFD)
 		resources[i].fd_ev = eventfd(0, 0);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #else
 		UNEXPECTED
 #endif
 #if defined(HAVE_MEMFD_CREATE)
+
+#if !defined(MFD_NOEXEC_SEAL)
+#define MFD_NOEXEC_SEAL		0x0008U
+#endif
 		(void)snprintf(name, sizeof(name), "memfd-%" PRIdMAX "-%zu",
 			(intmax_t)pid, i);
-		resources[i].fd_memfd = shim_memfd_create(name, 0);
+		/* Try with MFD_NOEXEC_SEAL */
+		resources[i].fd_memfd = shim_memfd_create(name, MFD_NOEXEC_SEAL);
+		/* ..and if failed, retry with no flags */
+		if (resources[i].fd_memfd == -1)
+			resources[i].fd_memfd = shim_memfd_create(name, 0);
 		if (resources[i].fd_memfd != -1) {
 			if (ftruncate(resources[i].fd_memfd, (off_t)page_size) == 0) {
 				resources[i].ptr_memfd = mmap(NULL, page_size,
@@ -246,10 +356,12 @@ size_t stress_resources_allocate(
 					resources[i].fd_memfd, 0);
 				if (resources[i].ptr_memfd == MAP_FAILED)
 					resources[i].ptr_memfd = NULL;
+				else
+					(void)stress_madvise_mergeable(resources[i].ptr_memfd, page_size);
 			}
 			shim_fallocate(resources[i].fd_memfd, 0, 0, (off_t)stress_mwc16());
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #else
 		UNEXPECTED
@@ -264,15 +376,17 @@ size_t stress_resources_allocate(
 					resources[i].fd_memfd_secret, 0);
 				if (resources[i].ptr_memfd_secret == MAP_FAILED)
 					resources[i].ptr_memfd_secret = NULL;
+				else
+					(void)stress_madvise_mergeable(resources[i].ptr_memfd_secret, page_size);
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #endif
 		resources[i].fd_sock = socket(
 			domains[stress_mwc32modn((uint32_t)SIZEOF_ARRAY(domains))],
 			types[stress_mwc32modn((uint32_t)SIZEOF_ARRAY(types))], 0);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		if (socketpair(AF_UNIX, SOCK_STREAM, 0,
@@ -283,7 +397,7 @@ size_t stress_resources_allocate(
 
 #if defined(HAVE_USERFAULTFD)
 		resources[i].fd_uf = shim_userfaultfd(0);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #else
 		UNEXPECTED
@@ -291,12 +405,10 @@ size_t stress_resources_allocate(
 #if defined(O_TMPFILE)
 		resources[i].fd_tmp = open("/tmp", O_TMPFILE | O_RDWR,
 				      S_IRUSR | S_IWUSR);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		if (resources[i].fd_tmp != -1) {
-			const size_t sz = page_size * stress_mwc32();
-
-			(void)shim_fallocate(resources[i].fd_tmp, 0, 0, (off_t)sz);
+			(void)shim_fallocate(resources[i].fd_tmp, 0, 0, (off_t)page_size);
 #if defined(F_GETLK) &&		\
     defined(F_SETLK) &&		\
     defined(F_SETLKW) &&	\
@@ -308,15 +420,15 @@ size_t stress_resources_allocate(
 				f.l_type = F_WRLCK;
 				f.l_whence = SEEK_SET;
 				f.l_start = 0;
-				f.l_len = (off_t)sz;
+				f.l_len = (off_t)page_size;
 				f.l_pid = pid;
 
 				(void)fcntl(resources[i].fd_tmp, F_SETLK, &f);
 			}
-		}
 #else
 		UNEXPECTED
 #endif
+		}
 #else
 		UNEXPECTED
 #endif
@@ -364,7 +476,7 @@ size_t stress_resources_allocate(
 			resources[i].fd_inotify = -1;
 			resources[i].wd_inotify = -1;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #endif
 #if defined(HAVE_PTSNAME)
@@ -376,7 +488,7 @@ size_t stress_resources_allocate(
 			if (ptyname)
 				resources[i].pty = open(ptyname, O_RDWR);
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #endif
 
@@ -390,7 +502,7 @@ size_t stress_resources_allocate(
     defined(HAVE_PTHREAD_MUTEX_DESTROY)
 			resources[i].mutex_ret = pthread_mutex_init(&resources[i].mutex, NULL);
 #endif
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #endif
@@ -407,7 +519,7 @@ size_t stress_resources_allocate(
 			sevp.sigev_value.sival_ptr = &resources[i].timerid;
 			resources[i].timerok =
 				(timer_create(CLOCK_REALTIME, &sevp, &resources[i].timerid) == 0);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #endif
@@ -419,7 +531,7 @@ size_t stress_resources_allocate(
     defined(CLOCK_REALTIME)
 		if (!i) {
 			resources[i].timer_fd = timerfd_create(CLOCK_REALTIME, 0);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #endif
@@ -427,18 +539,19 @@ size_t stress_resources_allocate(
 #if defined(HAVE_LIB_PTHREAD) &&	\
     defined(HAVE_SEM_POSIX)
 		resources[i].semok = (sem_init(&resources[i].sem, 1, 1) >= 0);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #endif
 
 #if defined(HAVE_SEM_SYSV) &&	\
     defined(HAVE_KEY_T)
 		{
-			const key_t sem_key = (key_t)stress_mwc32();
+			/* Use even key so it won't clash with odd global sem key */
+			const key_t sem_key = (key_t)stress_mwc32() & ~(key_t)1;
 
 			resources[i].sem_id = semget(sem_key, 1,
 				IPC_CREAT | S_IRUSR | S_IWUSR);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #endif
@@ -448,7 +561,7 @@ size_t stress_resources_allocate(
     defined(HAVE_SYS_MSG_H)
 		resources[i].msgq_id = msgget(IPC_PRIVATE,
 				S_IRUSR | S_IWUSR | IPC_CREAT | IPC_EXCL);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #endif
 
@@ -468,7 +581,7 @@ size_t stress_resources_allocate(
 
 			resources[i].mq = mq_open(resources[i].mq_name,
 				O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, &attr);
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #endif
@@ -476,7 +589,7 @@ size_t stress_resources_allocate(
     defined(HAVE_PKEY_FREE)
 		resources[i].pkey = shim_pkey_alloc(0, 0);
 #endif
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 #if defined(HAVE_PIDFD_OPEN)
@@ -509,7 +622,6 @@ size_t stress_resources_allocate(
 		}
 	}
 
-
 	return STRESS_MINIMUM(num_resources, n);
 }
 
@@ -519,7 +631,7 @@ size_t stress_resources_allocate(
  *	resource frees
  */
 void stress_resources_free(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_resources_t *resources,
         const size_t num_resources)
 {
@@ -529,67 +641,99 @@ void stress_resources_free(
 	(void)page_size;
 
 	for (i = 0; i < num_resources; i++) {
-		if (resources[i].m_malloc)
+		if (resources[i].m_malloc) {
 			free(resources[i].m_malloc);
+			resources[i].m_malloc = NULL;
+		}
 		if (resources[i].m_mmap && (resources[i].m_mmap != MAP_FAILED)) {
 			(void)shim_munlock(resources[i].m_mmap, resources[i].m_mmap_size);
-			(void)munmap(resources[i].m_mmap, resources[i].m_mmap_size);
+			(void)stress_munmap_retry_enomem(resources[i].m_mmap, resources[i].m_mmap_size);
+			resources[i].m_mmap = MAP_FAILED;
 		}
 		if (resources[i].pipe_ret != -1) {
 			(void)close(resources[i].fd_pipe[0]);
 			(void)close(resources[i].fd_pipe[1]);
+			resources[i].fd_pipe[0] = -1;
+			resources[i].fd_pipe[1] = -1;
 		}
-		if (resources[i].fd_open != -1)
+		if (resources[i].fd_open != -1) {
 			(void)close(resources[i].fd_open);
+			resources[i].fd_open = -1;
+		}
 #if defined(HAVE_EVENTFD)
-		if (resources[i].fd_ev != -1)
+		if (resources[i].fd_ev != -1) {
 			(void)close(resources[i].fd_ev);
+			resources[i].fd_ev = -1;
+		}
 #endif
 #if defined(HAVE_MEMFD_CREATE)
-		if (resources[i].fd_memfd != -1)
+		if (resources[i].fd_memfd != -1) {
 			(void)close(resources[i].fd_memfd);
-		if (resources[i].ptr_memfd)
-			(void)munmap(resources[i].ptr_memfd, page_size);
+			resources[i].fd_memfd = -1;
+		}
+		if (resources[i].ptr_memfd) {
+			(void)stress_munmap_retry_enomem(resources[i].ptr_memfd, page_size);
+			resources[i].ptr_memfd = MAP_FAILED;
+		}
 #endif
 #if defined(__NR_memfd_secret)
-		if (resources[i].fd_memfd_secret != -1)
+		if (resources[i].fd_memfd_secret != -1) {
 			(void)close(resources[i].fd_memfd_secret);
-		if (resources[i].ptr_memfd_secret)
-			(void)munmap(resources[i].ptr_memfd_secret, page_size);
+			resources[i].fd_memfd_secret = -1;
+		}
+		if (resources[i].ptr_memfd_secret) {
+			(void)stress_munmap_retry_enomem(resources[i].ptr_memfd_secret, page_size);
+			resources[i].ptr_memfd_secret = NULL;
+		}
 #endif
-		if (resources[i].fd_sock != -1)
+		if (resources[i].fd_sock != -1) {
 			(void)close(resources[i].fd_sock);
-		if (resources[i].fd_socketpair[0] != -1)
+			resources[i].fd_sock = -1;
+		}
+		if (resources[i].fd_socketpair[0] != -1) {
 			(void)close(resources[i].fd_socketpair[0]);
-		if (resources[i].fd_socketpair[1] != -1)
+			resources[i].fd_socketpair[0] = -1;
+		}
+		if (resources[i].fd_socketpair[1] != -1) {
 			(void)close(resources[i].fd_socketpair[1]);
+			resources[i].fd_socketpair[1] = -1;
+		}
 
 #if defined(HAVE_USERFAULTFD)
-		if (resources[i].fd_uf != -1)
+		if (resources[i].fd_uf != -1) {
 			(void)close(resources[i].fd_uf);
+			resources[i].fd_uf = -1;
+		}
 #endif
 
 #if defined(O_TMPFILE)
-		if (resources[i].fd_tmp != -1)
+		if (resources[i].fd_tmp != -1) {
 			(void)close(resources[i].fd_tmp);
+			resources[i].fd_tmp = -1;
+		}
 #endif
 
 #if defined(HAVE_LIB_PTHREAD)
-		if ((!i) && (!resources[i].pthread_ret) && (resources[i].pthread))
+		if ((!i) && (!resources[i].pthread_ret) && (resources[i].pthread)) {
 			(void)pthread_join(resources[i].pthread, NULL);
+			resources[i].pthread = (pthread_t)0;
 #if defined(HAVE_PTHREAD_MUTEX_T) &&	\
     defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
     defined(HAVE_PTHREAD_MUTEX_DESTROY)
 			(void)pthread_mutex_destroy(&resources[i].mutex);
+			(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
 #endif
+		}
 #endif
 
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
     defined(SIGUNUSED)
-		if ((!i) && (resources[i].timerok))
+		if ((!i) && (resources[i].timerok)) {
 			(void)timer_delete(resources[i].timerid);
+			resources[i].timerok = false;
+		}
 #endif
 
 #if defined(HAVE_SYS_TIMERFD_H) &&	\
@@ -597,65 +741,87 @@ void stress_resources_free(
     defined(HAVE_TIMERFD_GETTIME) &&	\
     defined(HAVE_TIMERFD_SETTIME) &&	\
     defined(CLOCK_REALTIME)
-		if ((!i) && (resources[i].timer_fd != -1))
+		if ((!i) && (resources[i].timer_fd != -1)) {
 			(void)close(resources[i].timer_fd);
+			resources[i].timer_fd = -1;
+		}
 #endif
 
 #if defined(HAVE_SYS_INOTIFY)
-		if (resources[i].wd_inotify != -1)
+		if (resources[i].wd_inotify != -1) {
 			(void)inotify_rm_watch(resources[i].fd_inotify, resources[i].wd_inotify);
-		if (resources[i].fd_inotify != -1)
+			resources[i].wd_inotify = -1;
+		}
+		if (resources[i].fd_inotify != -1) {
 			(void)close(resources[i].fd_inotify);
+			resources[i].fd_inotify = -1;
+		}
 #endif
 
 #if defined(HAVE_PTSNAME)
-		if (resources[i].pty != -1)
+		if (resources[i].pty != -1) {
 			(void)close(resources[i].pty);
-		if (resources[i].pty_mtx != -1)
+			resources[i].pty = -1;
+		}
+		if (resources[i].pty_mtx != -1) {
 			(void)close(resources[i].pty_mtx);
+			resources[i].pty_mtx = -1;
+		}
 #endif
 
 #if defined(HAVE_LIB_PTHREAD) &&	\
     defined(HAVE_SEM_POSIX)
-		if (resources[i].semok)
+		if (resources[i].semok) {
 			(void)sem_destroy(&resources[i].sem);
+			resources[i].semok = false;
+		}
 #endif
 
 #if defined(HAVE_SEM_SYSV)
-		if (resources[i].sem_id >= 0)
+		if (resources[i].sem_id >= 0) {
 			(void)semctl(resources[i].sem_id, 0, IPC_RMID);
+			resources[i].sem_id = -1;
+		}
 #endif
 
 #if defined(HAVE_MQ_SYSV) &&	\
     defined(HAVE_SYS_IPC_H) &&	\
     defined(HAVE_SYS_MSG_H)
-		if (resources[i].msgq_id >= 0)
+		if (resources[i].msgq_id >= 0) {
 			(void)msgctl(resources[i].msgq_id, IPC_RMID, NULL);
+			resources[i].msgq_id = -1;
+		}
 #endif
 
 #if defined(HAVE_LIB_RT) &&	\
     defined(HAVE_MQ_POSIX) &&	\
     defined(HAVE_MQUEUE_H)
-		if (resources[i].mq >= 0)
+		if (resources[i].mq >= 0) {
 			(void)mq_close(resources[i].mq);
-		if (resources[i].mq_name[0])
+			resources[i].mq = -1;
+		}
+		if (resources[i].mq_name[0]) {
 			(void)mq_unlink(resources[i].mq_name);
+			resources[i].mq_name[0] = '\0';
+		}
 #endif
 #if defined(HAVE_PKEY_ALLOC) &&	\
     defined(HAVE_PKEY_FREE)
-		if (resources[i].pkey > -1)
+		if (resources[i].pkey > -1) {
 			 (void)shim_pkey_free(resources[i].pkey);
+			 resources[i].pkey = -1;
+		}
 #endif
 
 #if defined(HAVE_PIDFD_OPEN)
-		if (resources[i].pid_fd > -1)
+		if (resources[i].pid_fd > -1) {
 			(void)close(resources[i].pid_fd);
+			resources[i].pid_fd = -1;
+		}
 #endif
 		if (resources[i].pid > 0) {
-			int status;
-
-			(void)stress_killpid(resources[i].pid);
-			(void)shim_waitpid(resources[i].pid, &status, 0);
+			(void)stress_kill_pid_wait(resources[i].pid, NULL);
+			resources[i].pid = 0;
 		}
 	}
 }
