@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King
+ * Copyright (C) 2021-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,8 +20,17 @@
 #define STRESS_CORE_SHIM
 
 #include "stress-ng.h"
-#include "core-pragma.h"
 #include "core-arch.h"
+#include "core-attribute.h"
+#include "core-asm-riscv.h"
+#include "core-builtin.h"
+#include "core-cpu-cache.h"
+#include "core-killpid.h"
+#include "core-pragma.h"
+#include "core-shim.h"
+
+#include <sched.h>
+#include <pwd.h>
 
 #if defined(__NR_pkey_get)
 #define HAVE_PKEY_GET
@@ -66,6 +75,10 @@ UNEXPECTED
 #include <sys/prctl.h>
 #endif
 
+#if defined(HAVE_SYS_TIMEX_H)
+#include <sys/timex.h>
+#endif
+
 #if defined(HAVE_SYS_RANDOM_H)
 #include <sys/random.h>
 #endif
@@ -84,9 +97,13 @@ UNEXPECTED
 #endif
 
 #if defined(HAVE_RUSAGE_WHO_T)
-#define shim_rusage_who_t	__rusage_who_t
+typedef __rusage_who_t	shim_rusage_who_t;
 #else
-#define shim_rusage_who_t	int
+typedef int		shim_rusage_who_t;
+#endif
+
+#if defined(HAVE_LINUX_MODULE_H)
+#include <linux/module.h>
 #endif
 
 #if defined(__sun__)
@@ -149,6 +166,28 @@ int shim_cacheflush(char *addr, int nbytes, int cache)
 	extern int cacheflush(void *addr, int nbytes, int cache);
 
 	return cacheflush((void *)addr, nbytes, cache);
+#elif defined(STRESS_ARCH_RISCV)
+#if defined(__NR_riscv_flush_icache)
+	if (cache == SHIM_ICACHE) {
+		if (syscall(__NR_riscv_flush_icache, (uintptr_t)addr,
+			    ((uintptr_t)addr) + nbytes, 0) == 0) {
+			return 0;
+		}
+	}
+#endif
+#if defined(HAVE_ASM_RISCV_FENCE_I)
+	if (cache == SHIM_ICACHE)  {
+		stress_asm_riscv_fence_i();
+		return 0;
+	}
+#endif
+#if defined(HAVE_ASM_RISCV_FENCE)
+	if (cache == SHIM_ICACHE)  {
+		stress_asm_riscv_fence();
+		return 0;
+	}
+#endif
+	return -1;
 #elif defined(HAVE_BUILTIN___CLEAR_CACHE)
 	/* More portable builtin */
 	(void)cache;
@@ -171,9 +210,9 @@ int shim_cacheflush(char *addr, int nbytes, int cache)
  */
 ssize_t shim_copy_file_range(
 	int fd_in,
-	shim_loff_t *off_in,
+	shim_off64_t *off_in,
 	int fd_out,
-	shim_loff_t *off_out,
+	shim_off64_t *off_out,
 	size_t len,
 	unsigned int flags)
 {
@@ -201,10 +240,10 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 	if (n == (off_t)-1)
 		return -1;
 
-	(void)memset(buffer, 0, FALLOCATE_BUF_SIZE);
+	(void)shim_memset(buffer, 0, FALLOCATE_BUF_SIZE);
 	n = len;
 
-	while (keep_stressing_flag() && (n > 0)) {
+	while (stress_continue_flag() && (n > 0)) {
 		ssize_t ret;
 		const size_t count = (size_t)STRESS_MINIMUM(n, FALLOCATE_BUF_SIZE);
 
@@ -218,6 +257,47 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 	return 0;
 }
 
+/*
+ *  shim_posix_fallocate()
+ *	emulation of posix_fallocate using chunks of allocations
+ *	and add EINTR support (which is not POSIX, but this is
+ *	an emulation wrapper so too bad).
+ */
+int shim_posix_fallocate(int fd, off_t offset, off_t len)
+{
+#if defined(HAVE_POSIX_FALLOCATE)
+	const off_t chunk_len = (off_t)(1 * MB);
+
+	do {
+		const off_t sz = (len > chunk_len) ? chunk_len : len;
+		int ret;
+
+		errno = 0;
+		ret = posix_fallocate(fd, offset, sz);
+		if (ret != 0) {
+			errno = ret;
+			return -1;
+		}
+
+		/*
+		 *  stressor has been signaled to finish, fake
+		 *  an EINTR return
+		 */
+		if (!stress_continue_flag()) {
+			errno = EINTR;
+			return -1;
+		}
+		offset += sz;
+		len -= sz;
+	} while (len > 0);
+
+	return 0;
+#else
+	return shim_emulate_fallocate(fd, offset, len);
+#endif
+}
+
+
 /*
  * shim_fallocate()
  *	shim wrapper for fallocate system call
@@ -296,7 +376,7 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 	/*
 	 *  posix_fallocate returns 0 for success, > 0 as errno
 	 */
-	ret = posix_fallocate(fd, offset, len);
+	ret = shim_posix_fallocate(fd, offset, len);
 	errno = 0;
 	if (ret != 0) {
 		/* failed, so retry with slower emulated fallocate */
@@ -418,7 +498,7 @@ int shim_getrandom(void *buff, size_t buflen, unsigned int flags)
  */
 void shim_flush_icache(void *begin, void *end)
 {
-#if defined(__GNUC__) &&	\
+#if defined(HAVE_COMPILER_GCC_OR_MUSL) &&	\
     defined(STRESS_ARCH_ARM)
 	__clear_cache(begin, end);
 #elif defined(STRESS_ARCH_RISCV) &&		\
@@ -466,7 +546,9 @@ int shim_klogctl(int type, char *bufp, int len)
  */
 int shim_membarrier(int cmd, int flags, int cpu_id)
 {
-#if defined(__NR_membarrier) &&	\
+#if defined(HAVE_MEMBARRIER)
+	return membarrier(cmd, flags, cpu_id);
+#elif defined(__NR_membarrier) &&	\
     defined(HAVE_SYSCALL)
 	return (int)syscall(__NR_membarrier, cmd, flags, cpu_id);
 #else
@@ -768,7 +850,7 @@ int shim_nanosleep_uint64(uint64_t nsec)
 		if (nanosleep(&t, &trem) < 0) {
 			if (errno == EINTR) {
 				t = trem;
-				if (keep_stressing_flag())
+				if (stress_continue_flag())
 					continue;
 			} else {
 				return -1;
@@ -791,7 +873,7 @@ int shim_nanosleep_uint64(uint64_t nsec)
 				usec = (useconds_t)(t_left * STRESS_DBL_MICROSECOND);
 				if (usec == 0)
 					return 0;
-				if (keep_stressing_flag())
+				if (stress_continue_flag())
 					continue;
 			} else {
 				return -1;
@@ -1012,7 +1094,7 @@ int shim_statx(
 	unsigned int mask,
 	shim_statx_t *buffer)
 {
-	(void)memset(buffer, 0, sizeof(*buffer));
+	(void)shim_memset(buffer, 0, sizeof(*buffer));
 #if defined(HAVE_STATX)
 	return statx(dfd, filename, flags, mask, (struct statx *)buffer);
 #elif defined(__NR_statx) &&	\
@@ -1422,7 +1504,14 @@ pid_t shim_waitpid(pid_t pid, int *wstatus, int options)
 
 	for (;;) {
 		errno = 0;
+#if defined(HAVE_WAITPID)
 		ret = waitpid(pid, wstatus, options);
+#elif defined(__NR_waitpid) &&	\
+    defined(HAVE_SYSCALL)
+		ret = (pid_t)syscall(__NR_waitpid, pid, wstatus, options);
+#else
+		ret = (pid_t)shim_enosys(0, pid, wstatus, options);
+#endif
 		if ((ret >= 0) || (errno != EINTR))
 			break;
 
@@ -1431,10 +1520,19 @@ pid_t shim_waitpid(pid_t pid, int *wstatus, int options)
 		 *  Retry if EINTR unless we've have 2 mins
 		 *  consecutive EINTRs then give up.
 		 */
-		if (!keep_stressing_flag()) {
-			(void)kill(pid, SIGALRM);
+		if (!stress_continue_flag()) {
+			(void)shim_kill(pid, SIGALRM);
 			if (count > 120)
-				(void)kill(pid, SIGKILL);
+				(void)stress_kill_pid(pid);
+		}
+		/*
+		 *  Process seems unkillable, report and bail out after 10 mins
+		 */
+		if (count > 600) {
+			pr_dbg("waitpid: SIGALRM on PID %jd has not resulted in "
+				"process termination after 10 minutes, giving up\n",
+				(intmax_t)pid);
+			break;
 		}
 		if (count > 10)
 			(void)sleep(1);
@@ -1448,7 +1546,9 @@ pid_t shim_waitpid(pid_t pid, int *wstatus, int options)
  */
 pid_t shim_wait(int *wstatus)
 {
-#if defined(__NR_waitpid) &&	\
+#if defined(HAVE_WAITPID)
+	return waitpid(-1, wstatus, 0);
+#elif defined(__NR_waitpid) &&	\
     defined(HAVE_SYSCALL)
 	return (pid_t)syscall(__NR_waitpid, -1, wstatus, 0);
 #else
@@ -1991,8 +2091,8 @@ int shim_clock_settime(clockid_t clk_id, struct timespec *tp)
  *  shim_nice
  *	wrapper for nice.  Some C libraries may use setpriority
  *	and hence the nice system call is not being used. Directly
- *	call the nice system call if it's available, else use the
- *	libc version.
+ *	call the nice system call if it's available, else use
+ *	setpriority or fall back onto the libc version.
  *
  *	Some operating systems like Hiaku don't even support nice,
  *	so handle these cases too.
@@ -2009,6 +2109,33 @@ int shim_nice(int inc)
 		return nice(inc);
 	}
 	return ret;
+#elif defined(HAVE_GETPRIORITY) &&	\
+      defined(HAVE_SETPRIORITY) &&	\
+      defined(PRIO_PROCESS)
+	int prio, ret, saved_err;
+
+	prio = getpriority(PRIO_PROCESS, 0);
+	if (prio == -1) {
+		if (errno != 0) {
+#if defined(HAVE_NICE)
+			/* fall back to nice */
+			return nice(inc);
+#else
+			/* ok, give up */
+			return -1;
+#endif
+		}
+	}
+	ret = setpriority(PRIO_PROCESS, 0, prio + inc);
+	if (ret == -1) {
+		if (errno == EACCES)
+			errno = EPERM;
+		return -1;
+	}
+	saved_err = errno;
+	ret = getpriority(PRIO_PROCESS, 0);
+	errno = saved_err;
+	return ret;
 #elif defined(HAVE_NICE)
 	return nice(inc);
 #else
@@ -2025,12 +2152,16 @@ int shim_nice(int inc)
  *	wrapper for time system call to
  *	avoid libc calling gettimeofday via
  *	the VDSO and force the time system call
+ *	for linux, otherwise use time()
  */
 time_t shim_time(time_t *tloc)
 {
 #if defined(__NR_time) &&	\
-    defined(HAVE_SYSCALL)
+    defined(HAVE_SYSCALL) &&	\
+    defined(__linux__)
 	return (time_t)syscall(__NR_time, tloc);
+#elif defined(HAVE_TIME)
+	return time(tloc);
 #else
 	return (time_t)shim_enosys(0, tloc);
 #endif
@@ -2045,8 +2176,11 @@ time_t shim_time(time_t *tloc)
 int shim_gettimeofday(struct timeval *tv, struct timezone *tz)
 {
 #if defined(__NR_gettimeofday) &&	\
-    defined(HAVE_SYSCALL)
+    defined(HAVE_SYSCALL) &&		\
+    defined(__linux__)
 	return (int)syscall(__NR_gettimeofday, tv, tz);
+#elif defined(HAVE_GETTIMEOFDAY)
+	return gettimeofday(tv, tz);
 #else
 	return (int)shim_enosys(0, tv, tz);
 #endif
@@ -2136,12 +2270,12 @@ int shim_stime(const time_t *t)
 	 *  Minix does not provide the prototype for stime
 	 *  and does not use a const time_t * argument
 	 */
-	int stime(time_t *t);
+	extern int stime(time_t *t);
 	time_t *ut = (time_t *)shim_unconstify_ptr(t);
 
 	return stime(ut);
 #else
-	int stime(const time_t *t);
+	extern int stime(const time_t *t);
 
 	return stime(t);
 #endif
@@ -2288,9 +2422,9 @@ int shim_modify_ldt(int func, void *ptr, unsigned long bytecount)
  */
 int shim_process_mrelease(int pidfd, unsigned int flags)
 {
-#if defined(__NR_process_release) &&	\
+#if defined(__NR_process_mrelease) &&	\
     defined(HAVE_SYSCALL)
-	return (int)syscall(__NR_process_release, pidfd, flags);
+	return (int)syscall(__NR_process_mrelease, pidfd, flags);
 #else
 	return (int)shim_enosys(0, pidfd, flags);
 #endif
@@ -2315,6 +2449,22 @@ int shim_futex_waitv(
 #endif
 }
 
+/*
+ *  shim_force_unlink(const char *pathname)
+ *	unlink always
+ */
+int shim_force_unlink(const char *pathname)
+{
+	int ret;
+
+	ret = unlink(pathname);
+	if (ret < 0) {
+		stress_unset_chattr_flags(pathname);
+		ret = unlink(pathname);
+	}
+	return ret;
+}
+
 /*
  *  shim_unlink()
  *	unlink, skip operation if --keep-files is enabled
@@ -2323,16 +2473,7 @@ int shim_unlink(const char *pathname)
 {
 	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
 		return 0;
-	return unlink(pathname);
-}
-
-/*
- *  shim_force_unlink(const char *pathname)
- *	unlink always
- */
-int shim_force_unlink(const char *pathname)
-{
-	return unlink(pathname);
+	return shim_force_unlink(pathname);
 }
 
 /*
@@ -2353,6 +2494,22 @@ int shim_unlinkat(int dirfd, const char *pathname, int flags)
 #endif
 }
 
+/*
+ *  shim_force_rmdir()
+ *	rmdir always
+ */
+int shim_force_rmdir(const char *pathname)
+{
+	int ret;
+
+	ret = rmdir(pathname);
+	if (ret < 0) {
+		stress_unset_chattr_flags(pathname);
+		ret = rmdir(pathname);
+	}
+	return ret;
+}
+
 /*
  *  shim_rmdir()
  *	rmdir, skip operation if --keep-files is enabled
@@ -2361,16 +2518,7 @@ int shim_rmdir(const char *pathname)
 {
 	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
 		return 0;
-	return rmdir(pathname);
-}
-
-/*
- *  shim_force_rmdir()
- *	rmdir always
- */
-int shim_force_rmdir(const char *pathname)
-{
-	return rmdir(pathname);
+	return shim_force_rmdir(pathname);
 }
 
 /*
@@ -2395,7 +2543,7 @@ int shim_getdomainname(char *name, size_t len)
  */
 int shim_setdomainname(const char *name, size_t len)
 {
-#if defined(HAVE_GETDOMAINNAME)
+#if defined(HAVE_SETDOMAINNAME)
 	return setdomainname(name, len);
 #elif defined(__NR_setdomainname) &&	\
       defined(HAVE_SYSCALL)
@@ -2423,3 +2571,165 @@ int shim_setgroups(int size, const gid_t *list)
 	return (int)shim_enosys(size, list);
 #endif
 }
+
+/*
+ *  shim_finit_module()
+ *      shim for linux finit_module
+ */
+int shim_finit_module(int fd, const char *uargs, int flags)
+{
+#if defined(HAVE_FINIT_MODULE)
+	extern finit_module(int fd, const char *param_values, int flags);
+
+	return finit_module(fd, uargs, flags);
+#elif defined(__NR_finit_module)
+	return syscall(__NR_finit_module, fd, uargs, flags);
+#else
+	return shim_enosys(0, fd, uargs, flags);
+#endif
+}
+
+/*
+ *  shim_delete_module()
+ *      shim for linux delete_module
+ */
+int shim_delete_module(const char *name, unsigned int flags)
+{
+#if defined(HAVE_DELETE_MODULE)
+	extern int delete_module(const char *name, int flags);
+
+	return delete_module(name, flags);
+#elif defined(__NR_delete_module)
+	return syscall(__NR_delete_module, name, flags);
+#else
+	return shim_enosys(0, name, flags);
+#endif
+}
+
+/*
+ *  shim_raise
+ *	shim for raise(), workaround SH4 raise not working
+ *	with glibc 2.34-7
+ */
+int shim_raise(int sig)
+{
+#if defined(STRESS_ARCH_SH4)
+	return shim_kill(getpid(), sig);
+#else
+	return raise(sig);
+#endif
+}
+
+/*
+ *  shim_kill
+ *	shim for kill() and ignore kill on pid 0, 1
+ *	with glibc 2.34-7
+ */
+int shim_kill(pid_t pid, int sig)
+{
+	if (sig == 0)
+		return kill(pid, sig);
+	if (pid == 1) {
+		errno = EPERM;
+		return -1;
+	}
+	if ((pid == -1) && (sig == SIGKILL)) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (geteuid() != 0)
+		return kill(pid, sig);
+	if (pid <= 0) {
+		errno = EPERM;
+		return -1;
+	}
+	return kill(pid, sig);
+}
+
+/*
+ *  shim_set_mempolicy_home_node()
+ *	wrapper for NUMA system call set_mempolicy_home_node()
+ */
+int shim_set_mempolicy_home_node(
+        unsigned long start,
+        unsigned long len,
+        unsigned long home_node,
+        unsigned long flags)
+{
+#if defined(__NR_set_mempolicy_home_node)
+        return (int)syscall(__NR_set_mempolicy_home_node, start, len, home_node, flags);
+#else
+	return shim_enosys(0, start, len, home_node, flags);
+#endif
+}
+
+/*
+ *  shim_fchmodat
+ *	shim for Linux fchmodat
+ */
+int shim_fchmodat(int dfd, const char *filename, mode_t mode, unsigned int flags)
+{
+#if defined(HAVE_FCHMODAT)
+	return fchmodat(dfd, filename, mode, flags);
+#elif defined(__NR_fchmodat)
+	/* No flags field in system call */
+	(void)flags;
+	return (int)syscall(__NR_fchmodat, dfd, filename, mode);
+#else
+	return shim_enosys(0, dfd, filename, mode, flags);
+#endif
+}
+
+/*
+ *  shim_fchmodat2
+ *	shim for Linux 6.6 fchmodat2
+ */
+int shim_fchmodat2(int dfd, const char *filename, mode_t mode, unsigned int flags)
+{
+#if defined(HAVE_FCHMODAT2)
+	return fchmodat2(dfd, filename, mode, flags);
+#elif defined(__NR_fchmodat2)
+	return (int)syscall(__NR_fchmodat2, dfd, filename, mode, flags);
+#else
+	return shim_enosys(0, dfd, filename, mode, flags);
+#endif
+}
+
+/*
+ *  shim_fstat()
+ *	shim for fstat
+ */
+int shim_fstat(int fd, struct stat *statbuf)
+{
+#if defined(HAVE_FSTAT)
+	return fstat(fd, statbuf);
+#else
+	return shim_enosys(0, fd, statbuf);
+#endif
+}
+
+/*
+ *  shim_lstat()
+ *	shim for lstat
+ */
+int shim_lstat(const char *pathname, struct stat *statbuf)
+{
+#if defined(HAVE_LSTAT)
+	return lstat(pathname, statbuf);
+#else
+	return shim_enosys(0, pathname, statbuf);
+#endif
+}
+
+/*
+ *  shim_stat()
+ *	shim for stat
+ */
+int shim_stat(const char *pathname, struct stat *statbuf)
+{
+#if defined(HAVE_STAT)
+	return stat(pathname, statbuf);
+#else
+	return shim_enosys(0, pathname, statbuf);
+#endif
+}
