@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King
+ * Copyright (C) 2022-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 
 #define STRESSORS(MACRO)	\
 	MACRO(access) 		\
+	MACRO(acl)		\
 	MACRO(af_alg) 		\
 	MACRO(affinity) 	\
 	MACRO(aio) 		\
@@ -44,6 +45,7 @@
 	MACRO(cache)		\
 	MACRO(cacheline)	\
 	MACRO(cap)		\
+	MACRO(cgroup)		\
 	MACRO(chattr)		\
 	MACRO(chdir)		\
 	MACRO(chmod)		\
@@ -70,6 +72,7 @@
 	MACRO(dnotify)		\
 	MACRO(dup)		\
 	MACRO(dynlib)		\
+	MACRO(eigen)		\
 	MACRO(efivar)		\
 	MACRO(enosys)		\
 	MACRO(env)		\
@@ -88,6 +91,7 @@
 	MACRO(filename)		\
 	MACRO(flock)		\
 	MACRO(flushcache)	\
+	MACRO(fma)		\
 	MACRO(fork)		\
 	MACRO(forkheavy)	\
 	MACRO(fp)		\
@@ -132,6 +136,7 @@
 	MACRO(l1cache)		\
 	MACRO(landlock)		\
 	MACRO(lease)		\
+	MACRO(led)		\
 	MACRO(link)		\
 	MACRO(list)		\
 	MACRO(llc_affinity)	\
@@ -155,6 +160,7 @@
 	MACRO(memrate)		\
 	MACRO(memthrash)	\
 	MACRO(mergesort)	\
+	MACRO(metamix)		\
 	MACRO(mincore)		\
 	MACRO(misaligned)	\
 	MACRO(mknod)		\
@@ -166,7 +172,9 @@
 	MACRO(mmapfork)		\
 	MACRO(mmaphuge)		\
 	MACRO(mmapmany)		\
+	MACRO(module)		\
 	MACRO(mprotect)		\
+	MACRO(mpfr)		\
 	MACRO(mq)		\
 	MACRO(mremap)		\
 	MACRO(msg)		\
@@ -230,6 +238,7 @@
 	MACRO(rotate)		\
 	MACRO(rseq)		\
 	MACRO(rtc)		\
+	MACRO(schedmix)		\
 	MACRO(schedpolicy)	\
 	MACRO(sctp)		\
 	MACRO(seal)		\
@@ -245,6 +254,7 @@
 	MACRO(shm)		\
 	MACRO(shm_sysv)		\
 	MACRO(sigabrt)		\
+	MACRO(sigbus)		\
 	MACRO(sigchld)		\
 	MACRO(sigfd)		\
 	MACRO(sigfpe)		\
@@ -258,6 +268,8 @@
 	MACRO(sigsegv)		\
 	MACRO(sigsuspend)	\
 	MACRO(sigtrap)		\
+	MACRO(sigxcpu)		\
+	MACRO(sigxfsz)		\
 	MACRO(skiplist)		\
 	MACRO(sleep)		\
 	MACRO(smi)		\
@@ -292,11 +304,13 @@
 	MACRO(tmpfs)		\
 	MACRO(touch)		\
 	MACRO(tree)		\
+	MACRO(trig)		\
 	MACRO(tsc)		\
 	MACRO(tsearch)		\
 	MACRO(tun)		\
 	MACRO(udp)		\
 	MACRO(udp_flood)	\
+	MACRO(umount)		\
 	MACRO(unshare)		\
 	MACRO(uprobe)		\
 	MACRO(urandom)		\
@@ -316,9 +330,13 @@
 	MACRO(vm_rw)		\
 	MACRO(vm_segv)		\
 	MACRO(vm_splice)	\
+	MACRO(vma)		\
+	MACRO(vnni)		\
 	MACRO(wait)		\
+	MACRO(waitcpu)		\
 	MACRO(watchdog)		\
 	MACRO(wcs)		\
+	MACRO(workload)		\
 	MACRO(x86cpuid)		\
 	MACRO(x86syscall)	\
 	MACRO(xattr)		\
