@@ -1,6 +1,6 @@
 /*
  * Copyright (C)      2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King
+ * Copyright (C) 2021-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,8 +18,13 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 #include "core-pragma.h"
 #include "core-thermal-zone.h"
+#include "core-vmstat.h"
+
+#include <sched.h>
 
 #if defined(HAVE_MACH_MACH_H)
 #include <mach/mach.h>
@@ -101,11 +106,16 @@ typedef struct {
 	uint64_t	discard_ticks;	/* total wait time for discard requests */
 } stress_iostat_t;
 
+static int32_t status_delay = 0;
 static int32_t vmstat_delay = 0;
 static int32_t thermalstat_delay = 0;
 static int32_t iostat_delay = 0;
 
 #if defined(__FreeBSD__)
+/*
+ *  freebsd_get_cpu_time()
+ *	get user, system, idle times; FreeBSD variant
+ */
 static void freebsd_get_cpu_time(
 	uint64_t *user_time,
 	uint64_t *system_time,
@@ -135,6 +145,10 @@ static void freebsd_get_cpu_time(
 #endif
 
 #if defined(__NetBSD__)
+/*
+ *  freebsd_get_cpu_time()
+ *	get user, system, idle times; NetBSD variant
+ */
 static void netbsd_get_cpu_time(
 	uint64_t *user_time,
 	uint64_t *system_time,
@@ -154,30 +168,57 @@ static void netbsd_get_cpu_time(
 }
 #endif
 
+/*
+ *  stress_set_generic_stat()
+ *	parse and check op for valud time range
+ */
 static int stress_set_generic_stat(
 	const char *const opt,
 	const char *name,
 	int32_t *delay)
 {
-	*delay = stress_get_int32(opt);
-        if ((*delay < 1) || (*delay > 3600)) {
-                (void)fprintf(stderr, "%s must in the range 1 to 3600.\n", name);
+	const uint64_t delay64 = stress_get_uint64_time(opt);
+
+        if ((delay64 < 1) || (delay64 > 3600)) {
+                (void)fprintf(stderr, "%s must in the range 1 to 3600 seconds.\n", name);
                 _exit(EXIT_FAILURE);
         }
+	*delay = (int32_t)(delay64 & 0x7fffffff);
 	return 0;
 }
 
+/*
+ *  stress_set_status()
+ *	parse --status option
+ */
+int stress_set_status(const char *const opt)
+{
+	return stress_set_generic_stat(opt, "status", &status_delay);
+}
+
+/*
+ *  stress_set_vmstat()
+ *	parse --vmstat option
+ */
 int stress_set_vmstat(const char *const opt)
 {
 	return stress_set_generic_stat(opt, "vmstat", &vmstat_delay);
 }
 
+/*
+ *  stress_set_thermalstat()
+ *	parse --thermalstat option
+ */
 int stress_set_thermalstat(const char *const opt)
 {
 	g_opt_flags |= OPT_FLAGS_TZ_INFO;
 	return stress_set_generic_stat(opt, "thermalstat", &thermalstat_delay);
 }
 
+/*
+ *  stress_set_iostat()
+ *	parse --iostat option
+ */
 int stress_set_iostat(const char *const opt)
 {
 	return stress_set_generic_stat(opt, "iostat", &iostat_delay);
@@ -199,7 +240,7 @@ char *stress_find_mount_dev(const char *name)
 	FILE *mtab_fp;
 	struct mntent *mnt;
 
-	if (stat(name, &statbuf) < 0)
+	if (shim_stat(name, &statbuf) < 0)
 		return NULL;
 
 	/* Cater for UBI char mounts */
@@ -222,11 +263,11 @@ char *stress_find_mount_dev(const char *name)
 			break;
 
 		if ((mnt->mnt_fsname[0] == '/') &&
-		    (stat(mnt->mnt_fsname, &statbuf) == 0) &&
+		    (shim_stat(mnt->mnt_fsname, &statbuf) == 0) &&
 		    (statbuf.st_rdev == dev))
 			break;
 
-		if ((stat(mnt->mnt_dir, &statbuf) == 0) &&
+		if ((shim_stat(mnt->mnt_dir, &statbuf) == 0) &&
 		    (statbuf.st_dev == dev))
 			break;
 	}
@@ -245,7 +286,7 @@ char *stress_find_mount_dev(const char *name)
 	struct dirent *d;
 	dev_t majdev;
 
-	if (stat(name, &statbuf) < 0)
+	if (shim_stat(name, &statbuf) < 0)
 		return NULL;
 
 	/* Cater for UBI char mounts */
@@ -265,7 +306,7 @@ char *stress_find_mount_dev(const char *name)
 		struct stat stat_buf;
 
 		stress_mk_filename(dev_path, sizeof(dev_path), "/dev", d->d_name);
-		ret = stat(dev_path, &stat_buf);
+		ret = shim_stat(dev_path, &stat_buf);
 		if ((ret == 0) &&
 		    (S_ISBLK(stat_buf.st_mode)) &&
 		    (stat_buf.st_rdev == majdev)) {
@@ -323,7 +364,7 @@ static char *stress_iostat_iostat_name(
 	 */
 	while (ptr >= dev) {
 		(void)snprintf(iostat_name, iostat_name_len, "/sys/block/%s/stat", dev);
-		if (stat(iostat_name, &statbuf) == 0)
+		if (shim_stat(iostat_name, &statbuf) == 0)
 			return iostat_name;
 		if (!isdigit(*ptr))
 			break;
@@ -366,7 +407,7 @@ static void stress_read_iostat(const char *iostat_name, stress_iostat_t *iostat)
 		(void)fclose(fp);
 
 		if (ret != 15)
-			(void)memset(iostat, 0, sizeof(*iostat));
+			(void)shim_memset(iostat, 0, sizeof(*iostat));
 	}
 }
 
@@ -383,7 +424,7 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	static stress_iostat_t iostat_prev;
 	stress_iostat_t iostat_current;
 
-	(void)memset(&iostat_current, 0, sizeof(iostat_current));
+	(void)shim_memset(&iostat_current, 0, sizeof(iostat_current));
 	stress_read_iostat(iostat_name, &iostat_current);
 	STRESS_IOSTAT_DELTA(read_io);
 	STRESS_IOSTAT_DELTA(read_merges);
@@ -400,7 +441,7 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	STRESS_IOSTAT_DELTA(discard_merges);
 	STRESS_IOSTAT_DELTA(discard_sectors);
 	STRESS_IOSTAT_DELTA(discard_ticks);
-	(void)memcpy(&iostat_prev, &iostat_current, sizeof(iostat_prev));
+	(void)shim_memcpy(&iostat_prev, &iostat_current, sizeof(iostat_prev));
 }
 #endif
 
@@ -694,11 +735,10 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 	struct xsw_usage xsu;
 	mach_port_t host = mach_host_self();
 	natural_t count = HOST_VM_INFO64_COUNT;
-	natural_t pcount, pi_array_count;
-	processor_info_array_t pi_array;
 	size_t page_size = stress_get_page_size();
 	int ret;
 
+	(void)shim_memset(&vm_stat, 0, sizeof(vmstat));
 	ret = host_statistics64(host, HOST_VM_INFO64, (host_info64_t)&vm_stat, &count);
 	if (ret >= 0) {
 		vmstat->swap_in = vm_stat.pageins;
@@ -721,17 +761,21 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
     defined(CPU_STATE_USER) &&		\
     defined(CPU_STATE_SYSTEM) &&	\
     defined(CPU_STATE_IDLE)
-	ret = host_processor_info(host, PROCESSOR_CPU_LOAD_INFO, &pcount, &pi_array, &pi_array_count);
-	if (ret >= 0) {
-		natural_t i;
+	{
+		natural_t pcount, pi_array_count;
+		processor_info_array_t pi_array;
 
+		ret = host_processor_info(host, PROCESSOR_CPU_LOAD_INFO, &pcount, &pi_array, &pi_array_count);
+		if (ret >= 0) {
+			natural_t i;
 
-		for (i = 0; i < pi_array_count; i++) {
-			integer_t *cpu_ticks = &pi_array[i * CPU_STATE_MAX];
+			for (i = 0; i < pi_array_count; i++) {
+				const integer_t *cpu_ticks = &pi_array[i * CPU_STATE_MAX];
 
-			vmstat->user_time += cpu_ticks[CPU_STATE_USER];
-			vmstat->system_time += cpu_ticks[CPU_STATE_SYSTEM];
-			vmstat->idle_time += cpu_ticks[CPU_STATE_IDLE];
+				vmstat->user_time += cpu_ticks[CPU_STATE_USER];
+				vmstat->system_time += cpu_ticks[CPU_STATE_SYSTEM];
+				vmstat->idle_time += cpu_ticks[CPU_STATE_IDLE];
+			}
 		}
 	}
 #endif
@@ -742,7 +786,8 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
     defined(SRUN)
 	{
 		size_t length;
-		static const int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
+		/* name must not be const, sysctl does not take const name param */
+		static int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
 
 		vmstat->procs_running = 0;
 		vmstat->procs_blocked = 0;
@@ -812,8 +857,8 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	static stress_vmstat_t vmstat_prev;
 	stress_vmstat_t vmstat_current;
 
-	(void)memset(&vmstat_current, 0, sizeof(vmstat_current));
-	(void)memset(vmstat, 0, sizeof(*vmstat));
+	(void)shim_memset(&vmstat_current, 0, sizeof(vmstat_current));
+	(void)shim_memset(vmstat, 0, sizeof(*vmstat));
 	stress_read_vmstat(&vmstat_current);
 	STRESS_VMSTAT_COPY(procs_running);
 	STRESS_VMSTAT_COPY(procs_blocked);
@@ -835,7 +880,7 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	STRESS_VMSTAT_DELTA(idle_time);
 	STRESS_VMSTAT_DELTA(wait_time);
 	STRESS_VMSTAT_DELTA(stolen_time);
-	(void)memcpy(&vmstat_prev, &vmstat_current, sizeof(vmstat_prev));
+	(void)shim_memcpy(&vmstat_prev, &vmstat_current, sizeof(vmstat_prev));
 }
 
 #if defined(__linux__)
@@ -843,7 +888,7 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
  *  stress_get_tz_info()
  *	get temperature in degrees C from a thermal zone
  */
-static double stress_get_tz_info(stress_tz_info_t *tz_info)
+static double stress_get_tz_info(const stress_tz_info_t *tz_info)
 {
 	double temp = 0.0;
 	FILE *fp;
@@ -862,7 +907,11 @@ static double stress_get_tz_info(stress_tz_info_t *tz_info)
 }
 #endif
 
-static void stress_zero_cpu_ghz(
+/*
+ *  stress_zero_cpu_ghz()
+ *	zero CPU clock freq stats
+ */
+static inline void stress_zero_cpu_ghz(
 	double *avg_ghz,
 	double *min_ghz,
 	double *max_ghz)
@@ -952,8 +1001,7 @@ static void stress_get_cpu_ghz(
 			(void)snprintf(name, sizeof(name), "dev.cpu.%" PRIi32 ".freq", i);
 			freq = (double)stress_bsd_getsysctl_uint(name) * ONE_THOUSANDTH;
 		}
-#endif
-#if defined(__APPLE__)
+#elif defined(__APPLE__)
 		freq = (double)stress_bsd_getsysctl_uint64("hw.cpufrequency") * ONE_BILLIONTH;
 #endif
 		if (freq >= 0.0) {
@@ -990,8 +1038,8 @@ void stress_vmstat_start(void)
 	stress_vmstat_t vmstat;
 	size_t tz_num = 0;
 	stress_tz_info_t *tz_info;
-	int32_t vmstat_sleep, thermalstat_sleep, iostat_sleep;
-	double t1, t2;
+	int32_t vmstat_sleep, thermalstat_sleep, iostat_sleep, status_sleep;
+	double t1, t2, t_start;
 #if defined(HAVE_SYS_SYSMACROS_H) &&	\
     defined(__linux__)
 	char iostat_name[PATH_MAX];
@@ -1000,17 +1048,22 @@ void stress_vmstat_start(void)
 
 	if ((vmstat_delay == 0) &&
 	    (thermalstat_delay == 0) &&
-	    (iostat_delay == 0))
+	    (iostat_delay == 0) &&
+	    (status_delay == 0))
 		return;
 
 	vmstat_sleep = vmstat_delay;
 	thermalstat_sleep = thermalstat_delay;
 	iostat_sleep = iostat_delay;
+	status_sleep = status_delay;
 
 	vmstat_pid = fork();
 	if ((vmstat_pid < 0) || (vmstat_pid > 0))
 		return;
 
+	stress_parent_died_alarm();
+	stress_set_proc_name("stat [periodic]");
+
 	if (vmstat_delay)
 		stress_get_vmstat(&vmstat);
 
@@ -1031,9 +1084,10 @@ void stress_vmstat_start(void)
 	VOID_RET(int, stress_set_sched(getpid(), SCHED_DEADLINE, 99, true));
 #endif
 
-	t1 = stress_time_now();
+	t_start = stress_time_now();
+	t1 = t_start;
 
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		int32_t sleep_delay = INT_MAX;
 		long clk_tick;
 		double delta;
@@ -1046,13 +1100,16 @@ void stress_vmstat_start(void)
     defined(__linux__)
 		if (iostat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(iostat_delay, sleep_delay);
-#endif
+#endif	
+		if (status_delay > 0)
+			sleep_delay = STRESS_MINIMUM(status_delay, sleep_delay);
 		t1 += sleep_delay;
 		t2 = stress_time_now();
 
 		delta = t1 - t2;
 		if (delta > 0) {
-			uint64_t nsec = (uint64_t)(delta * STRESS_DBL_NANOSECOND);
+			const uint64_t nsec = (uint64_t)(delta * STRESS_DBL_NANOSECOND);
+
 			(void)shim_nanosleep_uint64(nsec);
 		}
 
@@ -1062,6 +1119,7 @@ void stress_vmstat_start(void)
 		vmstat_sleep -= sleep_delay;
 		thermalstat_sleep -= sleep_delay;
 		iostat_sleep -= sleep_delay;
+		status_sleep -= sleep_delay;
 
 		if ((vmstat_delay > 0) && (vmstat_sleep <= 0))
 			vmstat_sleep = vmstat_delay;
@@ -1069,13 +1127,18 @@ void stress_vmstat_start(void)
 			thermalstat_sleep = thermalstat_delay;
 		if ((iostat_delay > 0) && (iostat_sleep <= 0))
 			iostat_sleep = iostat_delay;
+		if ((status_delay > 0) && (status_sleep <= 0))
+			status_sleep = status_delay;
 
 		if (vmstat_sleep == vmstat_delay) {
 			double clk_tick_vmstat_delay = (double)clk_tick * (double)vmstat_delay;
 			static uint32_t vmstat_count = 0;
 
-			if ((vmstat_count++ % 25) == 0)
-				pr_inf("vmstat: %2s %2s %9s %9s %9s %9s "
+			stress_get_vmstat(&vmstat);
+
+			pr_block_begin();
+			if (vmstat_count == 0)
+				pr_inf("vmstat: %3s %3s %9s %9s %9s %9s "
 					"%4s %4s %6s %6s %4s %4s %2s %2s "
 					"%2s %2s %2s\n",
 					"r", "b", "swpd", "free", "buff",
@@ -1083,8 +1146,7 @@ void stress_vmstat_start(void)
 					"in", "cs", "us", "sy", "id",
 					"wa", "st");
 
-			stress_get_vmstat(&vmstat);
-			pr_inf("vmstat: %2" PRIu64 " %2" PRIu64 /* procs */
+			pr_inf("vmstat: %3" PRIu64 " %3" PRIu64 /* procs */
 			       " %9" PRIu64 " %9" PRIu64	/* vm used */
 			       " %9" PRIu64 " %9" PRIu64	/* memory_buff */
 			       " %4" PRIu64 " %4" PRIu64	/* si, so*/
@@ -1110,6 +1172,11 @@ void stress_vmstat_start(void)
 				100.0 * (double)vmstat.idle_time / clk_tick_vmstat_delay,
 				100.0 * (double)vmstat.wait_time / clk_tick_vmstat_delay,
 				100.0 * (double)vmstat.stolen_time / clk_tick_vmstat_delay);
+			pr_block_end();
+
+			vmstat_count++;
+			if (vmstat_count >= 25)
+				vmstat_count = 0;
 		}
 
 		if (thermalstat_delay == thermalstat_sleep) {
@@ -1130,15 +1197,7 @@ void stress_vmstat_start(void)
 					ptr += 7;
 				}
 #endif
-				if ((thermalstat_count++ % 25) == 0)
-					pr_inf("therm: AvGHz MnGhz MxGHz  LdA1  LdA5 LdA15 %s\n", therms);
 
-#if defined(__linux__)
-				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
-					(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
-					ptr += 7;
-				}
-#endif
 				stress_get_cpu_ghz(&avg_ghz, &min_ghz, &max_ghz);
 				if (avg_ghz > 0.0)
 					(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.2f %5.2f %5.2f",
@@ -1147,6 +1206,16 @@ void stress_vmstat_start(void)
 					(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.5s %5.5s %5.5s",
 						" n/a ", " n/a ", " n/a ");
 
+				pr_block_begin();
+				if (thermalstat_count == 0)
+					pr_inf("therm: AvGHz MnGHz MxGHz  LdA1  LdA5 LdA15 %s\n", therms);
+
+#if defined(__linux__)
+				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
+					(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
+					ptr += 7;
+				}
+#endif
 				if (stress_get_load_avg(&min1, &min5, &min15) < 0)  {
 					pr_inf("therm: %18s %5.5s %5.5s %5.5s %s\n",
 						cpuspeed, "n/a", "n/a", "n/a", therms);
@@ -1154,7 +1223,12 @@ void stress_vmstat_start(void)
 					pr_inf("therm: %5s %5.2f %5.2f %5.2f %s\n",
 						cpuspeed, min1, min5, min15, therms);
 				}
+				pr_block_end();
 				free(therms);
+
+				thermalstat_count++;
+				if (thermalstat_count >= 25)
+					thermalstat_count = 0;
 			}
 		}
 
@@ -1164,10 +1238,12 @@ void stress_vmstat_start(void)
 			double clk_scale = (iostat_delay > 0) ? 1.0 / iostat_delay : 0.0;
 			static uint32_t iostat_count = 0;
 
-			if ((iostat_count++ % 25) == 0)
+			stress_get_iostat(iostat_name, &iostat);
+
+			pr_block_begin();
+			if (iostat_count == 0)
 				pr_inf("iostat: Inflght   Rd K/s   Wr K/s Dscd K/s     Rd/s     Wr/s   Dscd/s\n");
 
-			stress_get_iostat(iostat_name, &iostat);
 			/* sectors are 512 bytes, so >> 1 to get stats in 1024 bytes */
 			pr_inf("iostat: %7.0f %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f\n",
 				(double)iostat.in_flight * clk_scale,
@@ -1177,8 +1253,24 @@ void stress_vmstat_start(void)
 				(double)iostat.read_io * clk_scale,
 				(double)iostat.write_io * clk_scale,
 				(double)iostat.discard_io * clk_scale);
+			pr_block_end();
+
+			iostat_count++;
+			if (iostat_count >= 25)
+				iostat_count = 0;
 		}
 #endif
+		if (status_sleep == status_delay) {
+			const double runtime = round(stress_time_now() - g_shared->time_started);
+
+			pr_inf("status: %" PRIu32 " run, %" PRIu32 " exit, %" PRIu32 " reap, %" PRIu32 " fail, %" PRIu32 " sigalarm, %s\n",
+				g_shared->instance_count.started,
+				g_shared->instance_count.exited,
+				g_shared->instance_count.reaped,
+				g_shared->instance_count.failed,
+				g_shared->instance_count.alarmed,
+				stress_duration_to_str(runtime, false));
+		}
 	}
 	_exit(0);
 }
@@ -1189,10 +1281,6 @@ void stress_vmstat_start(void)
  */
 void stress_vmstat_stop(void)
 {
-	if (vmstat_pid > 0) {
-		int status;
-
-		(void)kill(vmstat_pid, SIGKILL);
-		(void)waitpid(vmstat_pid, &status, 0);
-	}
+	if (vmstat_pid > 0)
+		(void)stress_kill_pid_wait(vmstat_pid, NULL);
 }
