@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
+#include "core-builtin.h"
+
+#include <sys/socket.h>
 
 #if defined(HAVE_LINUX_IF_ALG_H)
 #include <linux/if_alg.h>
@@ -31,17 +35,14 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"af-alg N",	"start N workers that stress AF_ALG socket domain" },
-	{ NULL,	"af-alg-ops N",	"stop after N af-alg bogo operations" },
 	{ NULL,	"af-alg-dump",	"dump internal list from /proc/crypto to stdout" },
+	{ NULL,	"af-alg-ops N",	"stop after N af-alg bogo operations" },
 	{ NULL, NULL,		NULL }
 };
 
 static int stress_set_af_alg_dump(const char *opt)
 {
-	bool af_alg_dump = true;
-
-	(void)opt;
-	return stress_set_setting("af-alg-dump", TYPE_ID_BOOL, &af_alg_dump);
+	return stress_set_setting_true("af-alg-dump", opt);
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -49,8 +50,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ 0,			NULL }
 };
 
-#if defined(HAVE_LINUX_IF_ALG_H) && \
-    defined(HAVE_LINUX_SOCKET_H) && \
+#if defined(HAVE_LINUX_IF_ALG_H) &&	\
+    defined(HAVE_LINUX_SOCKET_H) &&	\
     defined(AF_ALG)
 
 #if !defined(SOL_ALG)
@@ -132,12 +133,13 @@ static void stress_af_alg_add_crypto_defconfigs(void);
  */
 static stress_crypto_type_t name_to_type(const char *buffer)
 {
-	const char *ptr = strchr(buffer, ':');
-	const char *end = ptr + strlen(buffer);
+	char *end, *ptr = strchr(buffer, ':');
 	size_t i;
 
 	if (!ptr)
 		return CRYPTO_UNKNOWN;
+
+	end = ptr + strlen(buffer);
 	ptr += 2;
 	if (ptr >= end)
 		return CRYPTO_UNKNOWN;
@@ -154,7 +156,7 @@ static stress_crypto_type_t name_to_type(const char *buffer)
  *   type_to_type_string()
  *	map type to stringified type
  */
-static const char *type_to_type_string(const stress_crypto_type_t type)
+static const char * PURE type_to_type_string(const stress_crypto_type_t type)
 {
 	size_t i;
 
@@ -171,7 +173,7 @@ static const char *type_to_type_string(const stress_crypto_type_t type)
  *	debug and ignore them for the next iteration
  */
 static void stress_af_alg_ignore(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_crypto_info_t *info)
 {
 	if ((args->instance == 0) && (!info->ignore)) {
@@ -182,7 +184,7 @@ static void stress_af_alg_ignore(
 }
 
 static int stress_af_alg_hash(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int sockfd,
 	stress_crypto_info_t *info)
 {
@@ -202,7 +204,7 @@ static int stress_af_alg_hash(
 		return EXIT_NO_RESOURCE;
 	}
 
-	(void)memset(&sa, 0, sizeof(sa));
+	(void)shim_memset(&sa, 0, sizeof(sa));
 	sa.salg_family = AF_ALG;
 	(void)shim_strlcpy((char *)sa.salg_type, "hash", sizeof(sa.salg_type));
 	(void)shim_strlcpy((char *)sa.salg_name, info->name, sizeof(sa.salg_name) - 1);
@@ -210,16 +212,15 @@ static int stress_af_alg_hash(
 retry_bind:
 	if (bind(sockfd, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
 		/* Perhaps the hash does not exist with this kernel */
-		if (errno == ENOENT) {
+		switch (errno) {
+		case ENOENT:
 			rc = EXIT_SUCCESS;
 			goto err;
-		}
-		if (errno == ELIBBAD) {
+		case ELIBBAD:
 			if (info->selftest) {
 				pr_fail("%s: bind failed but %s self test passed, errno=%d (%s)\n",
 					args->name, info->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
-				goto err;
 			} else {
 				/*
 				 *  self test was not marked as passed, this
@@ -227,33 +228,40 @@ retry_bind:
 				 *  FIPS enabled, so silently ignore bind failure
 				 */
 				rc = EXIT_SUCCESS;
-				goto err;
 			}
-		}
-		if (errno == EBUSY) {
+			goto err;
+		case EBUSY:
+		case EINTR:
 			rc = EXIT_SUCCESS;
 			goto err;
+		case ETIMEDOUT:
+			if (retries-- > 0)
+				goto retry_bind;
+			rc = EXIT_NO_RESOURCE;
+			goto err;
 		}
-		if (errno == ETIMEDOUT && retries-- > 0)
-			goto retry_bind;
-		pr_fail("%s: bind failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: %s: bind failed, errno=%d (%s)\n",
+			args->name, info->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto err;
 	}
 
 	fd = accept(sockfd, NULL, 0);
 	if (fd < 0) {
-		pr_fail("%s: accept failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		if (errno == EINTR) {
+			rc = EXIT_SUCCESS;
+			goto err;
+		}
+		pr_fail("%s: %s: accept failed, errno=%d (%s)\n",
+			args->name, info->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto err;
 	}
 
-	stress_strnrnd(input, DATA_LEN);
+	stress_rndbuf(input, DATA_LEN);
 
 	for (j = 32; j < DATA_LEN; j += 32) {
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		if (send(fd, input, j, 0) != (ssize_t)j) {
 			if ((errno == 0) || (errno == ENOKEY) || (errno == ENOENT))
@@ -262,25 +270,28 @@ retry_bind:
 				stress_af_alg_ignore(args, info);
 				break;
 			}
-			pr_fail("%s: send using %s failed: errno=%d (%s)\n",
+			pr_fail("%s: %s: send failed: errno=%d (%s)\n",
 					args->name, info->name,
 					errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
 		if (recv(fd, digest, (size_t)digest_size, MSG_WAITALL) != digest_size) {
-			pr_fail("%s: recv using %s failed: errno=%d (%s)\n",
+			if (errno == EOPNOTSUPP)
+				goto err_abort;
+			pr_fail("%s: %s: recv failed: errno=%d (%s)\n",
 				args->name, info->name,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
-		inc_counter(args);
-		if (args->max_ops && (get_counter(args) >= args->max_ops)) {
+		stress_bogo_inc(args);
+		if (args->max_ops && (stress_bogo_get(args) >= args->max_ops)) {
 			rc = EXIT_SUCCESS;
 			goto err_close;
 		}
 	}
+err_abort:
 	rc = EXIT_SUCCESS;
 err_close:
 	(void)close(fd);
@@ -292,7 +303,7 @@ err:
 }
 
 static int stress_af_alg_cipher(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int sockfd,
 	stress_crypto_info_t *info)
 {
@@ -321,7 +332,7 @@ static int stress_af_alg_cipher(
 		return EXIT_NO_RESOURCE;
 	}
 
-	(void)memset(&sa, 0, sizeof(sa));
+	(void)shim_memset(&sa, 0, sizeof(sa));
 	sa.salg_family = AF_ALG;
 	(void)shim_strlcpy((char *)sa.salg_type, salg_type, sizeof(sa.salg_type));
 	(void)shim_strlcpy((char *)sa.salg_name, info->name, sizeof(sa.salg_name) - 1);
@@ -331,7 +342,7 @@ retry_bind:
 		/* Perhaps the cipher does not exist with this kernel */
 		if (errno == ELIBBAD) {
 			if (info->selftest) {
-				pr_fail("%s: bind failed but %s self test passed, errno=%d (%s)\n",
+				pr_fail("%s: %s: bind failed but self test passed, errno=%d (%s)\n",
 					args->name, info->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 				goto err;
@@ -357,10 +368,14 @@ retry_bind:
 			rc = EXIT_SUCCESS;
 			goto err;
 		}
-		if (errno == ETIMEDOUT && retries-- > 0)
-			goto retry_bind;
-		pr_fail("%s: bind failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		if (errno == ETIMEDOUT) {
+			if (retries-- > 0)
+				goto retry_bind;
+			rc = EXIT_NO_RESOURCE;
+			goto err;
+		}
+		pr_fail("%s: %s: bind failed, errno=%d (%s)\n",
+			args->name, info->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto err;
 	}
@@ -375,15 +390,15 @@ retry_bind:
 			goto err;
 		}
 
-		stress_strnrnd(key, (size_t)info->max_key_size);
+		stress_rndbuf(key, (size_t)info->max_key_size);
 		if (setsockopt(sockfd, SOL_ALG, ALG_SET_KEY, key, (socklen_t)info->max_key_size) < 0) {
 			free(key);
 			if (errno == ENOPROTOOPT) {
 				rc = EXIT_SUCCESS;
 				goto err;
 			}
-			pr_fail("%s: setsockopt ALG_SET_KEY failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: %s: setsockopt ALG_SET_KEY failed, errno=%d (%s)\n",
+				args->name, info->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err;
 		}
@@ -403,15 +418,15 @@ retry_bind:
 			goto err;
 		}
 
-		stress_strnrnd(assocdata, (size_t)info->max_auth_size);
+		stress_rndbuf(assocdata, (size_t)info->max_auth_size);
 		if (setsockopt(sockfd, SOL_ALG, ALG_SET_AEAD_ASSOCLEN, assocdata, (socklen_t)info->max_auth_size) < 0) {
 			free(assocdata);
 			if (errno == ENOPROTOOPT) {
 				rc = EXIT_SUCCESS;
 				goto err;
 			}
-			pr_fail("%s: setsockopt ALG_SET_AEAD_ASSOCLEN failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: %s: setsockopt ALG_SET_AEAD_ASSOCLEN failed, errno=%d (%s)\n",
+				args->name, info->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err;
 		}
@@ -425,8 +440,8 @@ retry_bind:
 
 	fd = accept(sockfd, NULL, 0);
 	if (fd < 0) {
-		pr_fail("%s: accept failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: %s: accept failed, errno=%d (%s)\n",
+			args->name, info->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto err;
 	}
@@ -438,10 +453,10 @@ retry_bind:
 		struct af_alg_iv *iv;	/* Initialisation Vector */
 		struct iovec iov;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
-		(void)memset(&msg, 0, sizeof(msg));
-		(void)memset(cbuf, 0, cbuf_size);
+		(void)shim_memset(&msg, 0, sizeof(msg));
+		(void)shim_memset(cbuf, 0, cbuf_size);
 
 		msg.msg_control = cbuf;
 		msg.msg_controllen = cbuf_size;
@@ -450,15 +465,15 @@ retry_bind:
 		cmsg = CMSG_FIRSTHDR(&msg);
 		/* Keep static analysis happy */
 		if (!cmsg) {
-			pr_fail("%s: unexpected null cmsg found\n",
-				args->name);
+			pr_fail("%s: %s: unexpected null cmsg found\n",
+				args->name, info->name);
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
 		cmsg->cmsg_level = SOL_ALG;
 		cmsg->cmsg_type = ALG_SET_OP;
 		cmsg->cmsg_len = CMSG_LEN(4);
-		u32ptr = (__u32 *)CMSG_DATA(cmsg);
+		u32ptr = (__u32 *)(uintptr_t)CMSG_DATA(cmsg);
 		*u32ptr = ALG_OP_ENCRYPT;
 
 		/* Set up random Initialization Vector */
@@ -471,10 +486,10 @@ retry_bind:
 		iv = (void *)CMSG_DATA(cmsg);
 		iv->ivlen = (uint32_t)iv_size;
 
-		stress_strnrnd((char *)iv->iv, (size_t)iv_size);
+		stress_rndbuf((char *)iv->iv, (size_t)iv_size);
 
 		/* Generate random message to encrypt */
-		stress_strnrnd(input, DATA_LEN);
+		stress_rndbuf(input, DATA_LEN);
 		iov.iov_base = input;
 		iov.iov_len = DATA_LEN;
 
@@ -488,14 +503,16 @@ retry_bind:
 				stress_af_alg_ignore(args, info);
 				break;
 			}
-			pr_fail("%s: sendmsg using %s failed: errno=%d (%s)\n",
+			pr_fail("%s: %s: sendmsg failed: errno=%d (%s)\n",
 				args->name, info->name,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
-		if (read(fd, output, DATA_LEN) != DATA_LEN) {
-			pr_fail("%s: read using %s failed: errno=%d (%s)\n",
+		if (recv(fd, output, DATA_LEN, 0) != DATA_LEN) {
+			if (errno == EOPNOTSUPP)
+				goto err_abort;
+			pr_fail("%s: %s: read failed: errno=%d (%s)\n",
 				args->name, info->name,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -509,7 +526,7 @@ retry_bind:
 		cmsg->cmsg_level = SOL_ALG;
 		cmsg->cmsg_type = ALG_SET_OP;
 		cmsg->cmsg_len = CMSG_LEN(4);
-		u32ptr = (__u32 *)CMSG_DATA(cmsg);
+		u32ptr = (__u32 *)(uintptr_t)CMSG_DATA(cmsg);
 		*u32ptr = ALG_OP_DECRYPT;
 
 		/* Set up random Initialization Vector */
@@ -536,31 +553,31 @@ retry_bind:
 				stress_af_alg_ignore(args, info);
 				break;
 			}
-			pr_fail("%s: sendmsg using %s failed: errno=%d (%s)\n",
+			pr_fail("%s: %s: sendmsg failed: errno=%d (%s)\n",
 				args->name, info->name,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
 		if (read(fd, output, DATA_LEN) != DATA_LEN) {
-			pr_fail("%s: read using %s failed: errno=%d (%s)\n",
+			pr_fail("%s: %s: read failed: errno=%d (%s)\n",
 				args->name, info->name,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		} else {
-			if (memcmp(input, output, DATA_LEN)) {
-				pr_fail("%s: decrypted data "
+			if (shim_memcmp(input, output, DATA_LEN)) {
+				pr_fail("%s: %s: decrypted data "
 					"different from original data "
-					"using kernel crypto engine %s "
 					"(possible kernel bug)\n",
 					args->name, info->name);
 			}
 		}
 	}
 
+err_abort:
 	rc = EXIT_SUCCESS;
-	inc_counter(args);
+	stress_bogo_inc(args);
 err_close:
 	(void)close(fd);
 err:
@@ -572,7 +589,7 @@ err:
 }
 
 static int stress_af_alg_rng(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int sockfd,
 	stress_crypto_info_t *info)
 {
@@ -587,7 +604,7 @@ static int stress_af_alg_rng(
 	if (!output)
 		return EXIT_NO_RESOURCE;
 
-	(void)memset(&sa, 0, sizeof(sa));
+	(void)shim_memset(&sa, 0, sizeof(sa));
 	sa.salg_family = AF_ALG;
 	(void)shim_strlcpy((char *)sa.salg_type, "rng", sizeof(sa.salg_type));
 	(void)shim_strlcpy((char *)sa.salg_name, info->name, sizeof(sa.salg_name) - 1);
@@ -605,8 +622,12 @@ retry_bind:
 			rc = EXIT_SUCCESS;
 			goto err;
 		}
-		if (errno == ETIMEDOUT && retries-- > 0)
-			goto retry_bind;
+		if (errno == ETIMEDOUT) {
+			if (retries-- > 0)
+				goto retry_bind;
+			rc = EXIT_NO_RESOURCE;
+			goto err;
+		}
 		pr_fail("%s: bind failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
@@ -622,7 +643,7 @@ retry_bind:
 	}
 
 	for (j = 0; j < 16; j++) {
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		if (read(fd, output, output_size) != output_size) {
 			if (errno != EINVAL) {
@@ -632,8 +653,8 @@ retry_bind:
 				goto err_close;
 			}
 		}
-		inc_counter(args);
-		if (args->max_ops && (get_counter(args) >= args->max_ops)) {
+		stress_bogo_inc(args);
+		if (args->max_ops && (stress_bogo_get(args) >= args->max_ops)) {
 			rc = EXIT_SUCCESS;
 			goto err_close;
 		}
@@ -647,7 +668,7 @@ err:
 	return rc;
 }
 
-static void stress_af_alg_count_crypto(int *count, int *internal)
+static void stress_af_alg_count_crypto(size_t *count, size_t *internal)
 {
 	stress_crypto_info_t *ci;
 
@@ -666,7 +687,7 @@ static void stress_af_alg_count_crypto(int *count, int *internal)
  *  stress_af_alg_cmp_crypto()
  *	qsort comparison on type then name
  */
-static int stress_af_alg_cmp_crypto(const void *p1, const void *p2)
+static int PURE stress_af_alg_cmp_crypto(const void *p1, const void *p2)
 {
 	int n;
 	const stress_crypto_info_t * const *ci1 = (const stress_crypto_info_t * const *)p1;
@@ -694,12 +715,12 @@ static void stress_af_alg_sort_crypto(void)
 {
 	stress_crypto_info_t **array, *ci;
 
-	int i, n, internal;
+	size_t i, n, internal;
 
 	stress_af_alg_count_crypto(&n, &internal);
 
 	/* Attempt to sort, if we can't silently don't sort */
-	array = calloc((size_t)n, sizeof(*array));
+	array = calloc(n, sizeof(*array));
 	if (!array)
 		return;
 
@@ -755,18 +776,18 @@ static void stress_af_alg_dump_crypto_list(void)
  *  stress_af_alg()
  *	stress socket AF_ALG domain
  */
-static int stress_af_alg(const stress_args_t *args)
+static int stress_af_alg(stress_args_t *args)
 {
 	int sockfd = -1, rc = EXIT_FAILURE;
 	int retries = MAX_AF_ALG_RETRIES;
-	int proc_count, count, internal;
+	size_t proc_count, count, internal;
 	bool af_alg_dump = false;
 
 	stress_af_alg_count_crypto(&proc_count, &internal);
 
 	(void)stress_get_setting("af-alg-dump", &af_alg_dump);
 
-	if (af_alg_dump && args->instance == 0) {
+	if (af_alg_dump && (args->instance == 0)) {
 		pr_inf("%s: dumping cryptographic algorithms found in /proc/crypto to stdout\n",
 			args->name);
 		stress_af_alg_sort_crypto();
@@ -778,17 +799,15 @@ static int stress_af_alg(const stress_args_t *args)
 	stress_af_alg_count_crypto(&count, &internal);
 
 	if (args->instance == 0) {
-		bool lock;
-
-		pr_lock(&lock);
-		pr_inf("%s: %d cryptographic algorithms found in /proc/crypto\n",
+		pr_block_begin();
+		pr_inf("%s: %zd cryptographic algorithms found in /proc/crypto\n",
 			args->name, proc_count);
-		pr_inf("%s: %d cryptographic algorithms in total (with defconfigs)\n",
+		pr_inf("%s: %zd cryptographic algorithms in total (with defconfigs)\n",
 			args->name, count);
 		if (internal)
-			pr_inf("%s: %d cryptographic algorithms are internal and may be unused\n",
+			pr_inf("%s: %zd cryptographic algorithms are internal and may be unused\n",
 				args->name, internal);
-		pr_unlock(&lock);
+		pr_block_end();
 	}
 
 	for (;;) {
@@ -797,7 +816,7 @@ static int stress_af_alg(const stress_args_t *args)
 			break;
 
 		retries--;
-		if ((!keep_stressing_flag()) ||
+		if ((!stress_continue_flag()) ||
                     (retries < 0) ||
                     (errno != EAFNOSUPPORT)) {
 			if (errno == EAFNOSUPPORT) {
@@ -825,7 +844,7 @@ static int stress_af_alg(const stress_args_t *args)
 	do {
 		stress_crypto_info_t *info;
 
-		for (info = crypto_info_list; keep_stressing(args) && info; info = info->next) {
+		for (info = crypto_info_list; info && stress_continue(args); info = info->next) {
 			if (info->internal || info->ignore)
 				continue;
 			switch (info->crypto_type) {
@@ -848,12 +867,11 @@ static int stress_af_alg(const stress_args_t *args)
 				(void)rc;
 				break;
 			case CRYPTO_UNKNOWN:
-				CASE_FALLTHROUGH;
 			default:
 				break;
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -884,9 +902,9 @@ static char *dup_field(const char *buffer)
  *  int_field()
  *	parse an integer from a numeric field
  */
-static int int_field(const char *buffer)
+static int PURE int_field(const char *buffer)
 {
-	char *ptr = strchr(buffer, ':');
+	const char *ptr = strchr(buffer, ':');
 
 	if (!ptr)
 		return -1;
@@ -898,9 +916,9 @@ static int int_field(const char *buffer)
  *	parse a boolean from a string field
  *	error/default is false.
  */
-static bool bool_field(const char *buffer)
+static bool PURE bool_field(const char *buffer)
 {
-	char *ptr = strchr(buffer, ':');
+	const char *ptr = strchr(buffer, ':');
 
 	if (!ptr)
 		return false;
@@ -908,7 +926,6 @@ static bool bool_field(const char *buffer)
 		return true;
 	if (!strncmp("no", ptr + 2, 2))
 		return false;
-
 	return false;
 }
 
@@ -916,7 +933,7 @@ static bool bool_field(const char *buffer)
  *  stress_af_alg_add_crypto()
  *	add crypto algorithm to list if it is unique
  */
-static bool stress_af_alg_add_crypto(stress_crypto_info_t *info)
+static bool stress_af_alg_add_crypto(const stress_crypto_info_t *info)
 {
 	stress_crypto_info_t *ci;
 
@@ -930,6 +947,15 @@ static bool stress_af_alg_add_crypto(stress_crypto_info_t *info)
          */
 	if (strcmp(info->name, "ecb(arc4)") == 0)
 		return false;
+	/*
+	 * Don't support non-mainline tk transformations that some
+	 * kernels use, see
+	 * https://lore.kernel.org/lkml/1594591536-531-1-git-send-email-iuliana.prodan@nxp.com/t/#Z2e.:..:1594591536-531-3-git-send-email-iuliana.prodan::40nxp.com:1drivers:crypto:caam:caamalg.c
+	 */
+	if (strcmp(info->name, "tk(cbc(aes))") == 0)
+		return false;
+	if (strcmp(info->name, "tk(ecb(aes))") == 0)
+		return false;
 
 	/* Scan for duplications */
 	for (ci = crypto_info_list; ci; ci = ci->next) {
@@ -998,8 +1024,8 @@ static void stress_af_alg_init(void)
 	if (!fp)
 		return;
 
-	(void)memset(buffer, 0, sizeof(buffer));
-	(void)memset(&info, 0, sizeof(info));
+	(void)shim_memset(buffer, 0, sizeof(buffer));
+	(void)shim_memset(&info, 0, sizeof(info));
 
 	while (fgets(buffer, sizeof(buffer) - 1, fp)) {
 		if (!strncmp(buffer, "name", 4)) {
@@ -1037,7 +1063,7 @@ static void stress_af_alg_init(void)
 			} else {
 				stress_af_alg_info_free(&info);
 			}
-			(void)memset(&info, 0, sizeof(info));
+			(void)shim_memset(&info, 0, sizeof(info));
 		}
 	}
 	stress_af_alg_info_free(&info);
@@ -1076,9 +1102,11 @@ stressor_info_t stress_af_alg_info = {
 
 #else
 stressor_info_t stress_af_alg_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without linux/if_alg.h"
 };
 #endif
