@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -159,7 +159,7 @@ static int issue_aio_request(
 	const uint32_t request,
 	int (*aio_func)(struct aiocb *aiocbp))
 {
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		int ret;
 
 		io_req->request = (int)request;
@@ -200,7 +200,7 @@ static int issue_aio_sync_request(
 	const int fd,
 	stress_io_req_t *const io_req)
 {
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		int ret;
 		const int op = stress_mwc1() ? O_SYNC : O_DSYNC;
 
@@ -233,7 +233,7 @@ static int issue_aio_sync_request(
  *  stress_aio
  *	stress asynchronous I/O
  */
-static int stress_aio(const stress_args_t *args)
+static int stress_aio(stress_args_t *args)
 {
 	int ret, fd, rc = EXIT_FAILURE;
 	stress_io_req_t *io_reqs;
@@ -273,9 +273,11 @@ static int stress_aio(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto finish;
 	}
-	fs_type = stress_fs_type(filename);
+	fs_type = stress_get_fs_type(filename);
 	(void)shim_unlink(filename);
 
+	stress_file_rw_hint_short(fd);
+
 	(void)sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART | SA_SIGINFO;
 	sa.sa_sigaction = aio_signal_handler;
@@ -301,7 +303,7 @@ static int stress_aio(const stress_args_t *args)
 	do {
 		(void)shim_usleep_interruptible(250000); /* wait until a signal occurs */
 
-		for (i = 0; keep_stressing(args) && (i < opt_aio_requests); i++) {
+		for (i = 0; stress_continue(args) && (i < opt_aio_requests); i++) {
 			if (io_reqs[i].status != EINPROGRESS)
 				continue;
 
@@ -310,7 +312,7 @@ static int stress_aio(const stress_args_t *args)
 			case ECANCELED:
 			case 0:
 				/* Succeeded or cancelled, so redo another */
-				inc_counter(args);
+				stress_bogo_inc(args);
 #if defined(HAVE_AIO_FSYNC) &&	\
     defined(O_SYNC) &&		\
     defined(O_DSYNC)
@@ -347,7 +349,7 @@ static int stress_aio(const stress_args_t *args)
 				goto cancel;
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 	t2 = stress_time_now();
 
 	rc = EXIT_SUCCESS;
@@ -372,7 +374,8 @@ finish:
 
 	dt = t2 - t1;
 	if (dt > 0.0)
-		stress_misc_stats_set(args->misc_stats, 0, "async I/O signals per sec", (double)total / dt);
+		stress_metrics_set(args, 0, "async I/O signals per sec",
+			(double)total / dt, STRESS_HARMONIC_MEAN);
 	(void)stress_temp_dir_rm_args(args);
 	return rc;
 }
@@ -389,6 +392,7 @@ stressor_info_t stress_aio_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without aio.h"
 };
