@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-killpid.h"
+#include "core-lock.h"
 
 #if defined(HAVE_SYS_APPARMOR_H)
 #include <sys/apparmor.h>
@@ -40,11 +43,17 @@ static const stress_help_t help[] = {
 
 #define APPARMOR_BUF_SZ	(4096)
 
-typedef int (*stress_apparmor_func)(const stress_args_t *args);
+typedef int (*stress_apparmor_func)(stress_args_t *args);
 
+typedef struct {
+	void *counter_lock;
+	void *failure_lock;
+	uint32_t failure_count;
+} stress_apparmor_shared_info_t;
+
+static stress_apparmor_shared_info_t *stress_apparmor_shared_info;
 static volatile bool apparmor_run = true;
 static char *apparmor_path = NULL;
-static void *counter_lock;
 static char *data_copy, *data_prev;
 
 extern char g_apparmor_data[];
@@ -106,13 +115,25 @@ static int stress_apparmor_supported(const char *name)
 	return 0;
 }
 
-static bool stress_apparmor_keep_stressing_inc(const stress_args_t *args, bool inc)
+static bool stress_apparmor_stress_continue_inc(stress_args_t *args, bool inc)
 {
 	/* fast check */
 	if (!apparmor_run)
 		return false;
+	if (!stress_apparmor_shared_info)
+		return false;
 
-	return inc_counter_lock(args, counter_lock, inc);
+	return stress_bogo_inc_lock(args, stress_apparmor_shared_info->counter_lock, inc);
+}
+
+static void stress_apparmor_failure_inc(void)
+{
+	if (!stress_apparmor_shared_info)
+		return;
+	if (stress_lock_acquire(stress_apparmor_shared_info->failure_lock) < 0)
+		return;
+	stress_apparmor_shared_info->failure_count++;
+	(void)stress_lock_release(stress_apparmor_shared_info->failure_lock);
 }
 
 /*
@@ -131,7 +152,7 @@ static void MLOCKED_TEXT stress_apparmor_alrm_handler(int signum)
  *	read a proc file
  */
 static void stress_apparmor_read(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *path)
 {
 	int fd;
@@ -144,9 +165,9 @@ static void stress_apparmor_read(
 	 *  Multiple randomly sized reads
 	 */
 	while (i < (4096 * APPARMOR_BUF_SZ)) {
-		ssize_t ret, sz = 1 + (stress_mwc32() % sizeof(buffer));
+		ssize_t ret, sz = 1 + stress_mwc32modn((uint32_t)sizeof(buffer));
 redo:
-		if (!stress_apparmor_keep_stressing_inc(args, false))
+		if (!stress_apparmor_stress_continue_inc(args, false))
 			break;
 		ret = read(fd, buffer, (size_t)sz);
 		if (ret < 0) {
@@ -166,7 +187,7 @@ redo:
  *	read directory
  */
 static void stress_apparmor_dir(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *path,
 	const bool recurse,
 	const int depth)
@@ -174,7 +195,7 @@ static void stress_apparmor_dir(
 	DIR *dp;
 	struct dirent *d;
 
-	if (!stress_apparmor_keep_stressing_inc(args, false))
+	if (!stress_apparmor_stress_continue_inc(args, false))
 		return;
 
 	/* Don't want to go too deep */
@@ -188,7 +209,7 @@ static void stress_apparmor_dir(
 	while ((d = readdir(dp)) != NULL) {
 		char name[PATH_MAX];
 
-		if (!stress_apparmor_keep_stressing_inc(args, false))
+		if (!stress_apparmor_stress_continue_inc(args, false))
 			break;
 		if (stress_is_dot_filename(d->d_name))
 			continue;
@@ -215,7 +236,7 @@ static void stress_apparmor_dir(
  *	spawn a process
  */
 static pid_t apparmor_spawn(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_apparmor_func func)
 {
 	pid_t pid;
@@ -223,14 +244,14 @@ static pid_t apparmor_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
 	if (pid == 0) {
 		int ret = EXIT_SUCCESS;
 
-		if (!stress_apparmor_keep_stressing_inc(args, false))
+		if (!stress_apparmor_stress_continue_inc(args, false))
 			goto abort;
 
 		if (stress_sighandler(args->name, SIGALRM,
@@ -240,12 +261,12 @@ again:
 
 		(void)sched_settings_apply(true);
 		stress_parent_died_alarm();
-		if (!stress_apparmor_keep_stressing_inc(args, false))
+		if (!stress_apparmor_stress_continue_inc(args, false))
 			goto abort;
 		ret = func(args);
 abort:
 		free(apparmor_path);
-		(void)kill(args->pid, SIGUSR1);
+		(void)shim_kill(args->pid, SIGUSR1);
 		_exit(ret);
 	}
 	return pid;
@@ -255,7 +276,7 @@ abort:
  *  apparmor_stress_profiles()
  *	hammer profile reading
  */
-static int apparmor_stress_profiles(const stress_args_t *args)
+static int apparmor_stress_profiles(stress_args_t *args)
 {
 	char path[PATH_MAX];
 
@@ -263,7 +284,7 @@ static int apparmor_stress_profiles(const stress_args_t *args)
 
 	do {
 		stress_apparmor_read(args, path);
-	} while (stress_apparmor_keep_stressing_inc(args, true));
+	} while (stress_apparmor_stress_continue_inc(args, true));
 
 	return EXIT_SUCCESS;
 }
@@ -272,7 +293,7 @@ static int apparmor_stress_profiles(const stress_args_t *args)
  *  apparmor_stress_features()
  *	hammer features reading
  */
-static int apparmor_stress_features(const stress_args_t *args)
+static int apparmor_stress_features(stress_args_t *args)
 {
 	char path[PATH_MAX];
 
@@ -280,7 +301,7 @@ static int apparmor_stress_features(const stress_args_t *args)
 
 	do {
 		stress_apparmor_dir(args, path, true, 0);
-	} while (stress_apparmor_keep_stressing_inc(args, true));
+	} while (stress_apparmor_stress_continue_inc(args, true));
 
 	return EXIT_SUCCESS;
 }
@@ -289,7 +310,7 @@ static int apparmor_stress_features(const stress_args_t *args)
  *  apparmor_stress_kernel_interface()
  *	load/replace/unload stressing
  */
-static int apparmor_stress_kernel_interface(const stress_args_t *args)
+static int apparmor_stress_kernel_interface(stress_args_t *args)
 {
 	int rc = EXIT_SUCCESS;
 	aa_kernel_interface *kern_if;
@@ -302,6 +323,7 @@ static int apparmor_stress_kernel_interface(const stress_args_t *args)
 		if (ret < 0) {
 			pr_fail("%s: aa_kernel_interface_new() failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			stress_apparmor_failure_inc();
 			rc = EXIT_FAILURE;
 			break;
 		}
@@ -316,7 +338,9 @@ static int apparmor_stress_kernel_interface(const stress_args_t *args)
 			if (errno != EEXIST) {
 				pr_fail("%s: aa_kernel_interface_load_policy() failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				stress_apparmor_failure_inc();
 				rc = EXIT_FAILURE;
+				break;
 			}
 		}
 
@@ -342,13 +366,14 @@ static int apparmor_stress_kernel_interface(const stress_args_t *args)
 
 				pr_fail("%s: aa_kernel_interface_remove_policy() failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				stress_apparmor_failure_inc();
 				rc = EXIT_FAILURE;
 				break;
 			}
 		}
 		aa_kernel_interface_unref(kern_if);
 
-	} while (stress_apparmor_keep_stressing_inc(args, true));
+	} while (stress_apparmor_stress_continue_inc(args, true));
 
 	return rc;
 }
@@ -361,7 +386,7 @@ static inline void apparmor_corrupt_flip_bits_random(
 	char *copy, const size_t len)
 {
 	uint32_t i;
-	const uint32_t n = stress_mwc32() % 17;
+	const uint32_t n = stress_mwc32modn(17);
 
 	for (i = 0; i < n; i++) {
 		uint32_t rnd = stress_mwc32();
@@ -425,7 +450,7 @@ static inline void apparmor_corrupt_set_seq(
 static inline void apparmor_corrupt_flip_byte_random(
 	char *copy, const size_t len)
 {
-	copy[stress_mwc32() % len] ^= 0xff;
+	copy[stress_mwc32modn((uint32_t)len)] ^= 0xff;
 }
 
 /*
@@ -436,7 +461,7 @@ static inline void apparmor_corrupt_clr_bits_random(
 	char *copy, const size_t len)
 {
 	uint32_t i;
-	const uint32_t n = stress_mwc32() % 17;
+	const uint32_t n = stress_mwc32modn(17);
 
 	for (i = 0; i < n; i++) {
 		uint32_t rnd = stress_mwc32();
@@ -453,7 +478,7 @@ static inline void apparmor_corrupt_set_bits_random(
 	char *copy, const size_t len)
 {
 	uint32_t i;
-	const uint32_t n = stress_mwc32() % 17;
+	const uint32_t n = stress_mwc32modn(17);
 
 	for (i = 0; i < n; i++) {
 		uint32_t rnd = stress_mwc32();
@@ -469,7 +494,7 @@ static inline void apparmor_corrupt_set_bits_random(
 static inline void apparmor_corrupt_clr_byte_random(
 	char *copy, const size_t len)
 {
-	copy[stress_mwc32() % len] = 0;
+	copy[stress_mwc32modn((uint32_t)len)] = 0;
 }
 
 /*
@@ -479,7 +504,7 @@ static inline void apparmor_corrupt_clr_byte_random(
 static inline void apparmor_corrupt_set_byte_random(
 	char *copy, const size_t len)
 {
-	copy[stress_mwc32() % len] = (char)0xff;
+	copy[stress_mwc32modn((uint32_t)len)] = (char)0xff;
 }
 
 /*
@@ -490,7 +515,7 @@ static inline void apparmor_corrupt_flip_bits_random_burst(
 	char *copy, const size_t len)
 {
 	uint32_t i;
-	size_t p = (size_t)stress_mwc32() % (len * sizeof(*copy));
+	size_t p = (size_t)stress_mwc32modn((uint32_t)len * sizeof(*copy));
 
 	for (i = 0; i < 32; i++) {
 		if (p > len)
@@ -510,7 +535,7 @@ static inline void apparmor_corrupt_flip_one_bit_random(
 {
 	uint32_t rnd = stress_mwc32();
 
-	copy[rnd % len] ^= (1 << ((stress_mwc8() & 7)));
+	copy[rnd % len] ^= 1 << stress_mwc8modn(8);
 }
 
 /*
@@ -518,7 +543,7 @@ static inline void apparmor_corrupt_flip_one_bit_random(
  *	corrupt data and see if we can oops the loader
  *	parser.
  */
-static int apparmor_stress_corruption(const stress_args_t *args)
+static int apparmor_stress_corruption(stress_args_t *args)
 {
 	int rc = EXIT_SUCCESS, i = (int)args->instance, ret = -1;
 	int j = 0;
@@ -529,7 +554,7 @@ static int apparmor_stress_corruption(const stress_args_t *args)
 	 */
 	do {
 		if (ret < 0 || j > 1024) {
-			(void)memcpy(data_copy, g_apparmor_data, g_apparmor_data_len);
+			(void)shim_memcpy(data_copy, g_apparmor_data, g_apparmor_data_len);
 			j = 0;
 		}
 		/*
@@ -588,6 +613,7 @@ static int apparmor_stress_corruption(const stress_args_t *args)
 		if (ret < 0) {
 			pr_fail("%s: aa_kernel_interface_new() failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			stress_apparmor_failure_inc();
 			return EXIT_FAILURE;
 		}
 		/*
@@ -604,13 +630,13 @@ static int apparmor_stress_corruption(const stress_args_t *args)
 					"errno=%d (%s)\n", args->name, errno,
 					strerror(errno));
 			}
-			(void)memcpy(data_copy, data_prev, g_apparmor_data_len);
+			(void)shim_memcpy(data_copy, data_prev, g_apparmor_data_len);
 		} else {
-			(void)memcpy(data_prev, data_copy, g_apparmor_data_len);
+			(void)shim_memcpy(data_prev, data_copy, g_apparmor_data_len);
 		}
 		aa_kernel_interface_unref(kern_if);
 		i++;
-	} while (stress_apparmor_keep_stressing_inc(args, true));
+	} while (stress_apparmor_stress_continue_inc(args, true));
 
 	return rc;
 }
@@ -628,7 +654,7 @@ static const stress_apparmor_func apparmor_funcs[] = {
  *  stress_apparmor()
  *	stress AppArmor
  */
-static int stress_apparmor(const stress_args_t *args)
+static int stress_apparmor(stress_args_t *args)
 {
 	pid_t pids[MAX_APPARMOR_FUNCS];
 	size_t i;
@@ -637,22 +663,34 @@ static int stress_apparmor(const stress_args_t *args)
 	if (stress_sighandler(args->name, SIGUSR1, stress_sighandler_nop, NULL) < 0)
 		return EXIT_FAILURE;
 
+	stress_apparmor_shared_info = (stress_apparmor_shared_info_t *)
+		stress_mmap_populate(NULL, sizeof(*stress_apparmor_shared_info),
+			PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+	if (stress_apparmor_shared_info == MAP_FAILED) {
+		pr_inf_skip("%s: failed to allocated shared memory, skipping stressor\n", args->name);
+		return rc;
+	}
 	data_copy = malloc(g_apparmor_data_len);
 	if (!data_copy) {
 		pr_inf_skip("%s: failed to allocate apparmor data copy buffer, skipping stressor\n", args->name);
-		return rc;
+		goto err_free_shared_info;
 	}
 	data_prev = malloc(g_apparmor_data_len);
 	if (!data_prev) {
 		pr_inf_skip("%s: failed to allocate apparmor data prev buffer, skipping stressor\n", args->name);
 		goto err_free_data_copy;
 	}
-
-	counter_lock = stress_lock_create();
-	if (!counter_lock) {
+	stress_apparmor_shared_info->counter_lock = stress_lock_create();
+	if (!stress_apparmor_shared_info->counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		goto err_free_data_prev;
 	}
+	stress_apparmor_shared_info->failure_lock = stress_lock_create();
+	if (!stress_apparmor_shared_info->counter_lock) {
+		pr_inf_skip("%s: failed to create failure counter lock. skipping stressor\n", args->name);
+		goto err_free_counter_lock;
+	}
 
 	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
 		pids[i] = apparmor_spawn(args, apparmor_funcs[i]);
@@ -660,7 +698,7 @@ static int stress_apparmor(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	while (stress_apparmor_keep_stressing_inc(args, false)) {
+	while (stress_apparmor_stress_continue_inc(args, false)) {
 #if defined(HAVE_SELECT)
 		(void)select(0, NULL, NULL, NULL, NULL);
 #else
@@ -671,30 +709,22 @@ static int stress_apparmor(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	/* Wakeup, time to die */
-	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
-		if (pids[i] >= 0)
-			(void)kill(pids[i], SIGALRM);
-	}
-	/* Now apply death grip */
-	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
-		int status;
-
-		if (pids[i] >= 0) {
-			(void)kill(pids[i], SIGKILL);
-			(void)shim_waitpid(pids[i], &status, 0);
-		}
-	}
+	stress_kill_and_wait_many(args, pids, MAX_APPARMOR_FUNCS, SIGALRM, true);
 
 	free(apparmor_path);
 	apparmor_path = NULL;
-	(void)stress_lock_destroy(counter_lock);
 
-	rc = EXIT_SUCCESS;
+	rc = (stress_apparmor_shared_info->failure_count > 0) ? EXIT_FAILURE : EXIT_SUCCESS;
 
+	(void)stress_lock_destroy(stress_apparmor_shared_info->failure_lock);
+err_free_counter_lock:
+	(void)stress_lock_destroy(stress_apparmor_shared_info->counter_lock);
 err_free_data_prev:
 	free(data_prev);
 err_free_data_copy:
 	free(data_copy);
+err_free_shared_info:
+	(void)munmap((void *)stress_apparmor_shared_info, sizeof(*stress_apparmor_shared_info));
 
 	return rc;
 }
@@ -703,6 +733,7 @@ stressor_info_t stress_apparmor_info = {
 	.stressor = stress_apparmor,
 	.supported = stress_apparmor_supported,
 	.class = CLASS_OS | CLASS_SECURITY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
