@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-
 static const mode_t modes[] = {
 #if defined(S_ISUID)
 	S_ISUID,
@@ -70,24 +69,37 @@ static const mode_t modes[] = {
 	0
 };
 
-/*
- *  BSD systems can return EFTYPE which we can ignore
- *  as a "known" error on invalid chmod mode bits
- */
-#if defined(EFTYPE)
-#define CHECK(x) 			\
-	if ((x) &&			\
-	    ((errno != ENOSYS) &&	\
-	     (errno != EPERM) &&	\
-	     (errno != EFTYPE)))	\
-		return -1
-#else
-#define CHECK(x)			\
-	if ((x) &&			\
-	    ((errno != ENOSYS) &&	\
-	     (errno != EPERM)))		\
-		return -1
+static int OPTIMIZE3 stress_chmod_check(const int ret)
+{
+	static const int ignore_errors[] = {
+#if defined(ENOENT)
+		ENOENT,
 #endif
+#if defined(ENOTDIR)
+		ENOTDIR,
+#endif
+#if defined(ENOSYS)
+		ENOSYS,
+#endif
+#if defined(EFTYPE)
+		EFTYPE,
+#endif
+#if defined(EPERM)
+		EPERM,
+#endif
+	};
+
+	size_t i;
+
+	if (!ret)
+		return 0;
+
+	for (i = 0; i < SIZEOF_ARRAY(ignore_errors); i++) {
+		if (errno == ignore_errors[i])
+			return 0;
+	}
+	return -1;
+}
 
 /*
  *  do_fchmod()
@@ -104,10 +116,10 @@ static int do_fchmod(
 	const mode_t mask,
 	const mode_t all_mask)
 {
-	CHECK(fchmod(fd, modes[i]) < 0);
-	CHECK(fchmod(fd, mask) < 0);
-	CHECK(fchmod(fd, modes[i] ^ all_mask) < 0);
-	CHECK(fchmod(fd, mask ^ all_mask) < 0);
+	stress_chmod_check(fchmod(fd, modes[i]) < 0);
+	stress_chmod_check(fchmod(fd, mask) < 0);
+	stress_chmod_check(fchmod(fd, modes[i] ^ all_mask) < 0);
+	stress_chmod_check(fchmod(fd, mask ^ all_mask) < 0);
 
 	/*
 	 *  Exercise bad fchmod, ignore failure
@@ -137,38 +149,71 @@ static int do_chmod(
 	const size_t mode_count,
 	const int *mode_perms)
 {
-	static int index;
+	static size_t index;
 
 	if (!mode_count)
 		return 0;
 
 	(void)chmod(filename, (mode_t)mode_perms[index]);
 	index++;
-	index %= mode_count;
+	if (index >= mode_count)
+		index = 0;
 
-	CHECK(chmod(filename, modes[i]) < 0);
-	CHECK(chmod(filename, mask) < 0);
-	CHECK(chmod(filename, modes[i] ^ all_mask) < 0);
-	CHECK(chmod(filename, mask ^ all_mask) < 0);
+	stress_chmod_check(chmod(filename, modes[i]) < 0);
+	stress_chmod_check(chmod(filename, mask) < 0);
+	stress_chmod_check(chmod(filename, modes[i] ^ all_mask) < 0);
+	stress_chmod_check(chmod(filename, mask ^ all_mask) < 0);
 
-#if defined(HAVE_FCHMODAT)
 	if (dfd >= 0) {
-		CHECK(fchmodat(dfd, filebase, modes[i], 0) < 0);
-		CHECK(fchmodat(dfd, filebase, mask, 0) < 0);
-		CHECK(fchmodat(dfd, filebase, modes[i] ^ all_mask, 0) < 0);
-		CHECK(fchmodat(dfd, filebase, mask ^ all_mask, 0) < 0);
-
+#if defined(HAVE_FCHMODAT)
+		/*
+		 *  If we have fchmodat libc wrapper to system call then
+		 *  perform the call and also check
+		 */
+		stress_chmod_check(fchmodat(dfd, filebase, modes[i], 0) < 0);
+		stress_chmod_check(fchmodat(dfd, filebase, mask, 0) < 0);
+		stress_chmod_check(fchmodat(dfd, filebase, modes[i] ^ all_mask, 0) < 0);
+		stress_chmod_check(fchmodat(dfd, filebase, mask ^ all_mask, 0) < 0);
 		/*
 		 *  Exercise bad fchmodat, ignore failure
 		 */
 		VOID_RET(int, fchmodat(bad_fd, filebase, modes[i], 0));
-	}
 #else
-	(void)dfd;
-	(void)bad_fd;
-	(void)filebase;
+		shim_fchmodat(dfd, filebase, modes[i], 0);
+		shim_fchmodat(dfd, filebase, mask, 0);
+		shim_fchmodat(dfd, filebase, modes[i] ^ all_mask, 0);
+		shim_fchmodat(dfd, filebase, mask ^ all_mask, 0);
+		/*
+		 *  Exercise bad fchmodat, ignore failure
+		 */
+		VOID_RET(int, shim_fchmodat(bad_fd, filebase, modes[i], 0));
 #endif
 
+#if defined(HAVE_FCHMODAT2)
+		/*
+		 *  If we have fchmodat2 libc wrapper to system call then
+		 *  perform the call and also check
+		 */
+		stress_chmod_check(fchmodat2(dfd, filebase, modes[i], 0) < 0);
+		stress_chmod_check(fchmodat2(dfd, filebase, mask, 0) < 0);
+		stress_chmod_check(fchmodat2(dfd, filebase, modes[i] ^ all_mask, 0) < 0);
+		stress_chmod_check(fchmodat2(dfd, filebase, mask ^ all_mask, 0) < 0);
+		/*
+		 *  Exercise bad fchmodat2, ignore failure
+		 */
+		VOID_RET(int, fchmodat2(bad_fd, filebase, modes[i], 0));
+#else
+		shim_fchmodat2(dfd, filebase, modes[i], 0);
+		shim_fchmodat2(dfd, filebase, mask, 0);
+		shim_fchmodat2(dfd, filebase, modes[i] ^ all_mask, 0);
+		shim_fchmodat2(dfd, filebase, mask ^ all_mask, 0);
+		/*
+		 *  Exercise bad fchmodat2, ignore failure
+		 */
+		VOID_RET(int, shim_fchmodat2(bad_fd, filebase, modes[i], 0));
+#endif
+	}
+
 	/*
 	 *  Exercise illegal filename
 	 */
@@ -186,7 +231,7 @@ static int do_chmod(
  *  stress_chmod
  *	stress chmod
  */
-static int stress_chmod(const stress_args_t *args)
+static int stress_chmod(stress_args_t *args)
 {
 	const pid_t ppid = getppid();
 	int fd = -1, rc = EXIT_FAILURE, retries = 0, dfd = -1;
@@ -240,7 +285,7 @@ static int stress_chmod(const stress_args_t *args)
 	} else {
 		/* Other instances must try to open the file */
 		for (;;) {
-			if ((fd = open(filename, O_RDWR, S_IRUSR | S_IWUSR)) > - 1)
+			if ((fd = open(filename, O_RDWR)) > - 1)
 				break;
 
 #if defined(__NetBSD__)
@@ -250,7 +295,7 @@ static int stress_chmod(const stress_args_t *args)
 			(void)shim_usleep(100000);
 #endif
 			/* Timed out, then give up */
-			if (!keep_stressing_flag()) {
+			if (!stress_continue_flag()) {
 				rc = EXIT_SUCCESS;
 				goto tidy;
 			}
@@ -260,7 +305,7 @@ static int stress_chmod(const stress_args_t *args)
 					"retries to open and gave up "
 					"(instance %" PRIu32 ")%s\n",
 					args->name, filename, retries, args->instance,
-					stress_fs_type(filename));
+					stress_get_fs_type(filename));
 				goto tidy;
 			}
 		}
@@ -276,7 +321,7 @@ static int stress_chmod(const stress_args_t *args)
 			if (do_fchmod(fd, bad_fd, i, mask, all_mask) < 0) {
 				pr_fail("%s: fchmod failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno),
-					stress_fs_type(filename));
+					stress_get_fs_type(filename));
 			}
 			if (do_chmod(dfd, bad_fd, filebase, filename, longpath, i,
 				     mask, all_mask, mode_count, mode_perms) < 0) {
@@ -290,12 +335,12 @@ static int stress_chmod(const stress_args_t *args)
 				}
 				pr_fail("%s: chmod %s failed, errno=%d (%s)%s\n",
 					args->name, filename, errno, strerror(errno),
-					stress_fs_type(filename));
+					stress_get_fs_type(filename));
 			}
 		}
 		shim_fsync(fd);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 tidy:
