@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -74,7 +74,7 @@ static int stress_set_dirdeep_inodes(const char *opt)
 
 
 	if (inodes == 0) {
-		const char *type = stress_fs_type(stress_get_temp_path());
+		const char *type = stress_get_fs_type(stress_get_temp_path());
 
 		pr_inf("cannot determine number of available free inodes, defaulting to maximum allowed%s\n", type);
 		dirdeep_inodes = ~0ULL;
@@ -97,7 +97,6 @@ static int stress_set_dirdeep_files(const char *opt)
 	dirdeep_files = stress_get_uint32(opt);
 	stress_check_range("dirdeep-files", (uint64_t)dirdeep_files, 0, 65535);
 	return stress_set_setting("dirdeep-files", TYPE_ID_UINT32, &dirdeep_files);
-
 }
 
 /*
@@ -106,7 +105,7 @@ static int stress_set_dirdeep_files(const char *opt)
  *	dirdeep_dir number of subtress per level.
  */
 static bool stress_dirdeep_make(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *linkpath,
 	char *const path,
 	const size_t len,
@@ -139,7 +138,7 @@ static bool stress_dirdeep_make(
 	}
 	if (len + 2 >= path_len)
 		return true;
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return true;
 
 	errno = 0;
@@ -154,7 +153,7 @@ static bool stress_dirdeep_make(
 		printf("%s\n", path);
 		return true;
 	}
-	inc_counter(args);
+	stress_bogo_inc(args);
 	(*inodes_estimate)++;
 
 	/*
@@ -183,7 +182,7 @@ static bool stress_dirdeep_make(
 	path[len + 1] = 'h';	/* hardlink */
 	VOID_RET(int, link(linkpath, path));
 
-	for (i = 0; keep_stressing(args) && (i < dirdeep_dirs); i++) {
+	for (i = 0; stress_continue(args) && (i < dirdeep_dirs); i++) {
 		uint32_t j;
 		bool finish;
 
@@ -194,7 +193,7 @@ static bool stress_dirdeep_make(
 				dirdeep_bytes, inodes_start, inodes_estimate, inodes_min,
 				depth + 1);
 		if (len + 6 < path_len) {
-			for (j = 0; keep_stressing(args) && (j < dirdeep_files); j++) {
+			for (j = 0; stress_continue(args) && (j < dirdeep_files); j++) {
 				int fd;
 
 				(void)snprintf(path + len, path_len - len, "/%-4.4" PRIx32, j);
@@ -203,7 +202,7 @@ static bool stress_dirdeep_make(
 					break;
 				if (dirdeep_bytes > 0) {
 #if defined(HAVE_POSIX_FALLOCATE)
-					ret = posix_fallocate(fd, (off_t)0, dirdeep_bytes);
+					ret = shim_posix_fallocate(fd, (off_t)0, dirdeep_bytes);
 #else
 					ret = shim_fallocate(fd, 0, (off_t)0, dirdeep_bytes);
 #endif
@@ -220,7 +219,7 @@ static bool stress_dirdeep_make(
 			break;
 	}
 	path[len] = '\0';
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return true;
 
 #if defined(HAVE_LINKAT) &&	\
@@ -297,7 +296,7 @@ static bool stress_dirdeep_make(
  *	exercise files and directories in the tree
  */
 static int stress_dir_exercise(
-	const stress_args_t *args,
+	stress_args_t *args,
 	char *const path,
 	const size_t len,
 	const size_t path_len)
@@ -313,7 +312,7 @@ static int stress_dir_exercise(
 		{ sec, nsec }
 	};
 #endif
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return 0;
 
 	if (len + 2 >= path_len)
@@ -323,7 +322,7 @@ static int stress_dir_exercise(
 	if (n < 0)
 		return -1;
 
-	for (i = 0; (i < n) && keep_stressing(args); i++) {
+	for (i = 0; (i < n) && stress_continue(args); i++) {
 		register int ch;
 
 		/* Sanity check */
@@ -362,7 +361,7 @@ static int stress_dir_exercise(
 				}
 				(void)close(fd);
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 	}
 	path[len] = '\0';
@@ -377,12 +376,14 @@ static int stress_dir_exercise(
  *	clean up all files and directories in the tree
  */
 static void stress_dir_tidy(
-	const stress_args_t *args,
+	stress_args_t *args,
 	char *const path,
 	const size_t len,
-	const size_t path_len)
+	const size_t path_len,
+	const double start_time)
 {
 	struct dirent **namelist = NULL;
+	static bool tidy_info = false;
 	int n;
 
 	if (len + 2 >= path_len)
@@ -392,6 +393,16 @@ static void stress_dir_tidy(
 	if (n < 0)
 		return;
 
+	/*
+	 * If recursive deletion is taking more than 1 second then
+	 * inform the user
+	 */
+	if ((args->instance == 0) && !tidy_info &&
+	    (stress_time_now() > start_time + 1.0)) {
+		tidy_info = true;
+		pr_tidy("%s: removing directories\n", args->name);
+	}
+
 	while (n--) {
 		char *name = namelist[n]->d_name;
 		register const int ch = (int)name[0];
@@ -401,14 +412,10 @@ static void stress_dir_tidy(
 			continue;
 		}
 
-		snprintf(path + len, path_len - len, "/%s", name);
-		//path[len] = '/';
-		//path[len + 1] = ch;
-		//path[len + 2] = '\0';
-
+		(void)snprintf(path + len, path_len - len, "/%s", name);
 		if (name[1] == '\0' && (isdigit(ch) || isupper(ch))) {
 			free(namelist[n]);
-			stress_dir_tidy(args, path, len + 2, path_len);
+			stress_dir_tidy(args, path, len + 2, path_len, start_time);
 		} else {
 			free(namelist[n]);
 			(void)shim_unlink(path);
@@ -425,7 +432,7 @@ static void stress_dir_tidy(
  *  stress_dir
  *	stress deep recursive directory mkdir and rmdir
  */
-static int stress_dirdeep(const stress_args_t *args)
+static int stress_dirdeep(stress_args_t *args)
 {
 	int ret = EXIT_SUCCESS;
 	char path[PATH_MAX + 16];
@@ -483,23 +490,22 @@ static int stress_dirdeep(const stress_args_t *args)
 	(void)shim_strlcpy(path, rootpath, sizeof(path));
 	inodes_estimate = 1;		/* created one for root */
 	inodes_min = inodes_start;
+
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 	stress_dirdeep_make(args, linkpath, path, path_len, sizeof(path),
 		dirdeep_dirs, dirdeep_inodes, dirdeep_files, dirdeep_bytes,
 		inodes_start, &inodes_estimate, &inodes_min, 0);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
 	do {
 		(void)shim_strlcpy(path, rootpath, sizeof(path));
 		if (stress_dir_exercise(args, path, path_len, sizeof(path)) < 0)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	(void)shim_strlcpy(path, rootpath, sizeof(path));
-	pr_tidy("%s: removing directories\n", args->name);
-	stress_dir_tidy(args, path, path_len, sizeof(path));
+	stress_dir_tidy(args, path, path_len, sizeof(path), stress_time_now());
 
 	inodes_exercised = (inodes_start == 0) ? inodes_estimate : inodes_start - inodes_min;
 	pr_dbg("%s: %" PRIu64 " inodes exercised%s\n",
