@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 #include "core-net.h"
+#include "core-pragma.h"
 
 #if defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
@@ -56,7 +59,7 @@ static const stress_help_t help[] = {
     NEED_GLIBC(2,3,2)
 
 typedef void (stress_epoll_func_t)(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int child,
 	const pid_t mypid,
 	const int epoll_port,
@@ -166,7 +169,10 @@ static int stress_epoll_pwait(
 			return ret;
 	}
 #endif
+STRESS_PRAGMA_PUSH
+STRESS_PRAGMA_WARN_OFF
 	return epoll_pwait(epfd, events, maxevents, timeout, sigmask);
+STRESS_PRAGMA_POP
 }
 
 static void NORETURN MLOCKED_TEXT stress_segv_handler(int num)
@@ -185,7 +191,7 @@ static void MLOCKED_TEXT epoll_timer_handler(int sig)
 	(void)sig;
 
 	/* Cancel timer if we detect no more runs */
-	if (!keep_stressing_flag()) {
+	if (!stress_continue_flag()) {
 		struct itimerspec timer;
 
 		timer.it_value.tv_sec = 0;
@@ -202,7 +208,7 @@ static void MLOCKED_TEXT epoll_timer_handler(int sig)
  *	spawn a process
  */
 static pid_t epoll_spawn(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_epoll_func_t func,
 	const int child,
 	const pid_t mypid,
@@ -215,7 +221,7 @@ static pid_t epoll_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
@@ -249,7 +255,7 @@ static int epoll_set_fd_nonblock(const int fd)
  */
 static void epoll_recv_data(const int fd, int *fd_count)
 {
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		char buf[8192];
 		ssize_t n;
 
@@ -276,10 +282,10 @@ static int epoll_ctl_add(const int efd, const int fd, const uint32_t events)
 {
 	struct epoll_event event;
 
-	(void)memset(&event, 0, sizeof(event));
+	(void)shim_memset(&event, 0, sizeof(event));
 	event.data.fd = fd;
 	event.events = events;
-	if (epoll_ctl(efd, EPOLL_CTL_ADD, fd, &event) < 0)
+	if (UNLIKELY(epoll_ctl(efd, EPOLL_CTL_ADD, fd, &event) < 0))
 		return -1;
 
 	return 0;
@@ -294,9 +300,9 @@ static int epoll_ctl_mod(const int efd, const int fd, const uint32_t events)
 {
 	struct epoll_event event;
 
-	(void)memset(&event, 0, sizeof(event));
+	(void)shim_memset(&event, 0, sizeof(event));
 	event.events = events;
-	if (epoll_ctl(efd, EPOLL_CTL_MOD, fd, &event) < 0)
+	if (UNLIKELY(epoll_ctl(efd, EPOLL_CTL_MOD, fd, &event) < 0))
 		return -1;
 
 	return 0;
@@ -311,8 +317,8 @@ static int epoll_ctl_del(const int efd, const int fd)
 {
 	struct epoll_event event;
 
-	(void)memset(&event, 0, sizeof(event));
-	if (epoll_ctl(efd, EPOLL_CTL_DEL, fd, &event) < 0)
+	(void)shim_memset(&event, 0, sizeof(event));
+	if (UNLIKELY(epoll_ctl(efd, EPOLL_CTL_DEL, fd, &event) < 0))
 		return -1;
 
 	return 0;
@@ -324,7 +330,7 @@ static int epoll_ctl_del(const int efd, const int fd)
  *	fd's to epoll event list
  */
 static int epoll_notification(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int efd,
 	const int sfd,
 	const int epoll_sockets,
@@ -338,13 +344,13 @@ static int epoll_notification(
 		int fd;
 		struct epoll_event event;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return -1;
 		/* Try to limit too many open fds */
 		if (*fd_count > epoll_sockets)
 			return 0;
 
-		if ((fd = accept(sfd, &saddr, &slen)) < 0) {
+		if (UNLIKELY((fd = accept(sfd, &saddr, &slen)) < 0)) {
 			if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
 				/* all incoming connections handled so finish */
 				break;
@@ -364,7 +370,7 @@ static int epoll_notification(
 		/*
 		 *  Add non-blocking fd to epoll event list
 		 */
-		if (epoll_set_fd_nonblock(fd) < 0) {
+		if (UNLIKELY(epoll_set_fd_nonblock(fd) < 0)) {
 			pr_fail("%s: setting socket to non-blocking failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -377,12 +383,12 @@ static int epoll_notification(
 		 *  Exercise invalid epoll_ctl syscall with EPOLL_CTL_DEL
 		 *  and EPOLL_CTL_MOD on fd not registered with efd
 		 */
-		(void)memset(&event, 0, sizeof(event));
+		(void)shim_memset(&event, 0, sizeof(event));
 		(void)epoll_ctl(efd, EPOLL_CTL_DEL, fd, &event);
 		(void)epoll_ctl(efd, EPOLL_CTL_MOD, fd, &event);
 #endif
 
-		if (epoll_ctl_add(efd, fd, EPOLLIN) < 0) {
+		if (UNLIKELY(epoll_ctl_add(efd, fd, EPOLLIN) < 0)) {
 			pr_fail("%s: epoll_ctl_add failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -395,15 +401,15 @@ static int epoll_notification(
 		(void)epoll_ctl_add(efd, bad_fd, EPOLLIN | EPOLLET);
 
 		/* Exercise epoll_ctl syscall with invalid operation */
-		(void)memset(&event, 0, sizeof(event));
+		(void)shim_memset(&event, 0, sizeof(event));
 		(void)epoll_ctl(efd, INT_MIN, fd, &event);
 
 		/*
 		 *  Exercise illegal epoll_ctl_add having fd
 		 *  same as efd, resulting in EINVAL error
 		 */
-		if ((epoll_ctl_add(efd, efd, EPOLLIN | EPOLLET) == 0) &&
-                    (errno == 0)) {
+		if (UNLIKELY((epoll_ctl_add(efd, efd, EPOLLIN | EPOLLET) == 0) &&
+			     (errno == 0))) {
 			pr_fail("%s: epoll_ctl_add unexpectedly succeeded with "
 				"invalid arguments\n", args->name);
 			(void)close(fd);
@@ -415,8 +421,8 @@ static int epoll_notification(
 		 *  fd which is already registered with efd
 		 *  resulting in EEXIST error
 		 */
-		if ((epoll_ctl_add(efd, fd, EPOLLIN | EPOLLET) == 0) &&
-                    (errno == 0)) {
+		if (UNLIKELY((epoll_ctl_add(efd, fd, EPOLLIN | EPOLLET) == 0) &&
+			     (errno == 0))) {
 			pr_fail("%s: epoll_ctl_add unexpectedly succeeded "
 				"with a file descriptor that has already "
 				"been registered\n", args->name);
@@ -427,8 +433,8 @@ static int epoll_notification(
 		/*
 		 *  Exercise epoll_ctl_add on a illegal epoll_fd
 		 */
-		if ((epoll_ctl_add(-1, fd, EPOLLIN | EPOLLET) == 0) &&
-                    (errno == 0)) {
+		if (UNLIKELY((epoll_ctl_add(-1, fd, EPOLLIN | EPOLLET) == 0) &&
+			     (errno == 0))) {
 			pr_fail("%s: epoll_ctl_add unexpectedly succeeded "
 				"with an illegal file descriptor\n", args->name);
 			(void)close(fd);
@@ -444,20 +450,20 @@ static int epoll_notification(
  *	one another could not succeed.
  */
 static int test_eloop(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int efd,
 	const int efd2)
 {
 	int ret;
 
 	ret = epoll_ctl_add(efd, efd2, EPOLLIN | EPOLLET);
-	if (ret < 0) {
+	if (UNLIKELY(ret < 0)) {
 		pr_fail("%s: epoll_ctl_add failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	}
 
 	ret = epoll_ctl_add(efd2, efd, EPOLLIN | EPOLLET);
-	if (ret == 0) {
+	if (UNLIKELY(ret == 0)) {
 		pr_fail("%s: epoll_ctl_add failed, expected ELOOP, instead got "
 			"errno=%d (%s)\n", args->name, errno, strerror(errno));
 		(void)epoll_ctl_del(efd2, efd);
@@ -476,7 +482,7 @@ static int test_eloop(
  *      an error due to the EPOLL_EXCLUSIVE event type
  */
 static int test_epoll_exclusive(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int efd,
 	const int efd2,
 	const int sfd)
@@ -493,14 +499,14 @@ static int test_epoll_exclusive(
 	 *  Invalid epoll_ctl syscall as EPOLLEXCLUSIVE event
 	 *  cannot be operated by EPOLL_CTL_MOD operation
 	 */
-	if ((epoll_ctl_mod(efd, sfd, EPOLLEXCLUSIVE) == 0) &&
-	    (errno == 0)) {
+	if (UNLIKELY((epoll_ctl_mod(efd, sfd, EPOLLEXCLUSIVE) == 0) &&
+		     (errno == 0))) {
 		pr_fail("%s: epoll_ctl failed, expected EINVAL or ENOENT, instead got "
 			"errno=%d (%s)\n", args->name, errno, strerror(errno));
 		goto err;
 	}
 
-	if (epoll_ctl_add(efd, sfd, EPOLLEXCLUSIVE) < 0) {
+	if (UNLIKELY(epoll_ctl_add(efd, sfd, EPOLLEXCLUSIVE) < 0)) {
 		pr_fail("%s: epoll_ctl_add failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto err;
@@ -510,8 +516,8 @@ static int test_epoll_exclusive(
 	 *  Invalid epoll_ctl syscall as sfd was registered
 	 *  as EPOLLEXCLUSIVE event so it can't be modified
 	 */
-	if ((epoll_ctl_mod(efd, sfd, 0) == 0) &&
-	    (errno == 0)) {
+	if (UNLIKELY((epoll_ctl_mod(efd, sfd, 0) == 0) &&
+		     (errno == 0))) {
 		pr_fail("%s: epoll_ctl failed, expected EINVAL instead got "
 			"errno=%d (%s)\n", args->name, errno, strerror(errno));
 		goto err;
@@ -521,8 +527,8 @@ static int test_epoll_exclusive(
 	 *  Invalid epoll_ctl syscall as EPOLLEXCLUSIVE was
 	 *  specified in event and fd refers to an epoll instance.
 	 */
-	if ((epoll_ctl_add(efd, efd2, EPOLLEXCLUSIVE) == 0) &&
-	    (errno == 0)) {
+	if (UNLIKELY((epoll_ctl_add(efd, efd2, EPOLLEXCLUSIVE) == 0) &&
+		     (errno == 0))) {
 		pr_fail("%s: epoll_ctl failed, expected EINVAL, instead got "
 			"errno=%d (%s)\n", args->name, errno, strerror(errno));
 		goto err;
@@ -531,7 +537,7 @@ static int test_epoll_exclusive(
 	rc = 0;
 err:
 	epoll_ctl_del(efd, sfd);
-	if (epoll_ctl_add(efd, sfd, EPOLLIN | EPOLLET) < 0)
+	if (UNLIKELY(epoll_ctl_add(efd, sfd, EPOLLIN | EPOLLET) < 0))
 		rc = -1;
 
 	return rc;
@@ -544,7 +550,7 @@ err:
  *	send a relatively short message
  */
 static int epoll_client(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const pid_t mypid,
 	const int epoll_port,
 	const int epoll_domain)
@@ -554,26 +560,32 @@ static int epoll_client(
 	struct sigevent sev;
 	struct itimerspec timer;
 	struct sockaddr *addr = NULL;
+	uint64_t buf[4096 / sizeof(uint64_t)];
 
 	if (stress_sighandler(args->name, SIGRTMIN, epoll_timer_handler, NULL) < 0)
 		return EXIT_FAILURE;
 
+	stress_rndbuf((void *)buf, sizeof(buf));
+
 	do {
-		char buf[4096];
 		int fd, saved_errno;
+#if defined(STRESS_EPOLL_RETRY_COUNT)
 		int retries = 0;
+#endif
 		int ret;
 		const int port = epoll_port + port_counter +
 				(max_servers * (int)args->instance);
 		socklen_t addr_len = 0;
 
 		/* Cycle through the servers */
-		port_counter = (port_counter + 1) % max_servers;
+		port_counter = 0;
+		if (port_counter >= max_servers)
+			port_counter = 0;
 retry:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
-		if ((fd = socket(epoll_domain, SOCK_STREAM, 0)) < 0) {
+		if (UNLIKELY((fd = socket(epoll_domain, SOCK_STREAM, 0)) < 0)) {
 			if ((errno == EMFILE) ||
 			    (errno == ENFILE) ||
 			    (errno == ENOMEM) ||
@@ -584,11 +596,11 @@ retry:
 			return EXIT_FAILURE;
 		}
 
-		(void)memset(&sev, 0, sizeof(sev));
+		(void)shim_memset(&sev, 0, sizeof(sev));
 		sev.sigev_notify = SIGEV_SIGNAL;
 		sev.sigev_signo = SIGRTMIN;
 		sev.sigev_value.sival_ptr = &epoll_timerid;
-		if (timer_create(CLOCK_REALTIME, &sev, &epoll_timerid) < 0) {
+		if (UNLIKELY(timer_create(CLOCK_REALTIME, &sev, &epoll_timerid) < 0)) {
 			if ((errno == EAGAIN) || (errno == ENOMEM)) {
 				(void)close(fd);
 				continue;
@@ -609,15 +621,15 @@ retry:
 		timer.it_value.tv_nsec = 250000000;
 		timer.it_interval.tv_sec = timer.it_value.tv_sec;
 		timer.it_interval.tv_nsec = timer.it_value.tv_nsec;
-		if (timer_settime(epoll_timerid, 0, &timer, NULL) < 0) {
+		if (UNLIKELY(timer_settime(epoll_timerid, 0, &timer, NULL) < 0)) {
 			pr_fail("%s: timer_settime failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
 
-		if (stress_set_sockaddr(args->name, args->instance, mypid,
-			epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+		if (UNLIKELY(stress_set_sockaddr(args->name, args->instance, mypid,
+			epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY) < 0)) {
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
@@ -627,14 +639,14 @@ retry:
 		saved_errno = errno;
 
 		/* No longer need timer */
-		if (timer_delete(epoll_timerid) < 0) {
+		if (UNLIKELY(timer_delete(epoll_timerid) < 0)) {
 			pr_fail("%s: timer_delete failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
 
-		if (ret < 0) {
+		if (UNLIKELY(ret < 0)) {
 			switch (saved_errno) {
 			case EINTR:
 				connect_timeouts++;
@@ -650,29 +662,31 @@ retry:
 			(void)close(fd);
 			(void)shim_usleep(100000);	/* Twiddle fingers for a moment */
 
+#if defined(STRESS_EPOLL_RETRY_COUNT)
 			retries++;
-			if (retries > 1000) {
+			if (retries > 10) {
 				/* Sigh, give up.. */
-				pr_fail("%s: giving up too many failed connects;, errno = %d (%s)\n",
+				pr_fail("%s: giving up, too many failed connects, errno=%d (%s)\n",
 					args->name, saved_errno, strerror(saved_errno));
 				return EXIT_FAILURE;
 			}
+#endif
 			goto retry;
 		}
 
-		(void)memset(buf, 'A' + (get_counter(args) % 26), sizeof(buf));
-		if (send(fd, buf, sizeof(buf), 0) < 0) {
+		buf[0]++;
+		if (UNLIKELY(send(fd, (void *)buf, sizeof(buf), 0) < 0)) {
 			(void)close(fd);
 			pr_dbg("%s: send failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
 		(void)close(fd);
-		inc_counter(args);
-		if (!keep_stressing(args))
+		stress_bogo_inc(args);
+		if (!stress_continue(args))
 			break;
 		(void)shim_sched_yield();
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
@@ -695,7 +709,7 @@ retry:
  *	wait on connections and read data
  */
 static void NORETURN epoll_server(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int child,
 	const pid_t mypid,
 	const int epoll_port,
@@ -863,18 +877,18 @@ static void NORETURN epoll_server(
 		(void)sigemptyset(&sigmask);
 		(void)sigaddset(&sigmask, SIGALRM);
 
-		(void)memset(events, 0, MAX_EPOLL_EVENTS * sizeof(*events));
+		(void)shim_memset(events, 0, MAX_EPOLL_EVENTS * sizeof(*events));
 		errno = 0;
 
 		ret = sigsetjmp(jmp_env, 1);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
-		if (ret != 0)
+		if (UNLIKELY(ret != 0))
 			wait_segv = true;
 
 		/*
 		 * Wait for 100ms for an event, allowing us to
-		 * to break out if keep_stressing_flag has been changed.
+		 * to break out if stress_continue_flag has been changed.
 		 * Note: epoll_wait maps to epoll_pwait in glibc, ho hum.
 		 */
 		if (stress_mwc1()) {
@@ -885,7 +899,7 @@ static void NORETURN epoll_server(
 				 *  -EFAULT.
 				 */
 				n = epoll_wait(efd, args->mapped->page_none, 1, 100);
-				if (n > 0) {
+				if (UNLIKELY(n > 0)) {
 					pr_fail("%s: epoll_wait unexpectedly succeeded, "
 						"expected -EFAULT, instead got errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -905,7 +919,7 @@ static void NORETURN epoll_server(
 				 *  -EFAULT.
 				 */
 				n = stress_epoll_pwait(efd, args->mapped->page_none, 1, 100, &sigmask);
-				if (n > 1) {
+				if (UNLIKELY(n > 1)) {
 					pr_fail("%s: epoll_pwait unexpectedly succeeded, "
 						"expected -EFAULT, instead got errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -918,7 +932,7 @@ static void NORETURN epoll_server(
 			(void)stress_epoll_pwait(efd, events, INT_MIN, 100, &sigmask);
 
 		}
-		if (n < 0) {
+		if (UNLIKELY(n < 0)) {
 			if ((saved_errno != EINTR) &&
 			    (saved_errno != EINVAL)) {
 				pr_fail("%s: epoll_wait failed, errno=%d (%s)\n",
@@ -944,12 +958,12 @@ static void NORETURN epoll_server(
 				 *  The listening socket has notification(s)
 				 *  pending, so handle incoming connections
 				 */
-				if (epoll_notification(args, efd, sfd, epoll_sockets, &fd_count) < 0)
+				if (UNLIKELY(epoll_notification(args, efd, sfd, epoll_sockets, &fd_count) < 0))
 					break;
-				if (test_eloop(args, efd, efd2) < 0)
+				if (UNLIKELY(test_eloop(args, efd, efd2) < 0))
 					break;
 #if defined(EPOLLEXCLUSIVE)
-				if (test_epoll_exclusive(args, efd, efd2, sfd) < 0)
+				if (UNLIKELY(test_epoll_exclusive(args, efd, efd2, sfd) < 0))
 					break;
 #endif
 			} else {
@@ -969,7 +983,7 @@ static void NORETURN epoll_server(
 			n = stress_epoll_pwait(bad_fd, events, MAX_EPOLL_EVENTS, 100, &sigmask);
 			(void)n;
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 die_close:
 	if (efd != -1)
@@ -996,7 +1010,7 @@ die:
  *  stress_epoll
  *	stress by heavy socket I/O
  */
-static int stress_epoll(const stress_args_t *args)
+static int stress_epoll(stress_args_t *args)
 {
 	pid_t pids[MAX_SERVERS], mypid = getpid();
 	int i, rc = EXIT_SUCCESS;
@@ -1063,7 +1077,7 @@ static int stress_epoll(const stress_args_t *args)
 	 *  Typically, we are limited to ~500 connections per second
 	 *  on a default Linux configuration.
 	 */
-	(void)memset(pids, 0, sizeof(pids));
+	(void)shim_memset(pids, 0, sizeof(pids));
 	for (i = 0; i < max_servers; i++) {
 		pids[i] = epoll_spawn(args, epoll_server, i, mypid, epoll_port, epoll_domain, epoll_sockets);
 		if (pids[i] < 0) {
@@ -1073,22 +1087,12 @@ static int stress_epoll(const stress_args_t *args)
 		}
 	}
 
-	epoll_client(args, mypid, epoll_port, epoll_domain);
+	rc = epoll_client(args, mypid, epoll_port, epoll_domain);
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	stress_net_release_ports(start_port, end_port);
 
-	for (i = 0; i < max_servers; i++) {
-		int status;
-
-		if (pids[i] > 0) {
-			(void)kill(pids[i], SIGALRM);
-			if (shim_waitpid(pids[i], &status, 0) < 0) {
-				pr_dbg("%s: waitpid failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-			}
-		}
-	}
+	stress_kill_and_wait_many(args, pids, max_servers, SIGALRM, true);
 
 	return rc;
 }
@@ -1104,6 +1108,7 @@ stressor_info_t stress_epoll_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/epoll.h or librt or timer support"
 };
