@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_SYS_STATVFS_H)
 #include <sys/statvfs.h>
@@ -84,7 +87,7 @@ static int stress_set_filename_opts(const char *opt)
  *	necessary
  */
 static int stress_filename_probe_length(
-	const stress_args_t *args,
+	stress_args_t *args,
 	char *filename,
 	char *ptr,
 	size_t *sz_max)
@@ -103,7 +106,7 @@ static int stress_filename_probe_length(
 				break;
 			pr_err("%s: creat() failed when probing "
 				"for filename length, "
-				"errno = %d (%s)\n",
+				"errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			*sz_max = 0;
 			return -1;
@@ -122,7 +125,7 @@ static int stress_filename_probe_length(
  *	determine allowed filename chars by probing
  */
 static int stress_filename_probe(
-	const stress_args_t *args,
+	stress_args_t *args,
 	char *filename,
 	char *ptr,
 	size_t sz_max,
@@ -164,7 +167,7 @@ static int stress_filename_probe(
 			    (errno != EILSEQ)) {
 				pr_err("%s: creat() failed when probing "
 					"for allowed filename characters, "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				pr_inf("%s: perhaps retry and use "
 					"--filename-opts posix\n", args->name);
@@ -259,7 +262,7 @@ static void stress_filename_generate_random(
 	size_t i;
 
 	for (i = 0; i < sz_max; i++) {
-		const size_t j = (size_t)stress_mwc32() % chars_allowed;
+		const size_t j = (size_t)stress_mwc32modn(chars_allowed);
 		filename[i] = allowed[j];
 	}
 	if (*filename == '.')
@@ -274,7 +277,7 @@ static void stress_filename_generate_random(
  *	should_pass = false - expect it to fail (name too long)
  */
 static void stress_filename_test(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *filename,
 	const size_t sz_max,
 	const bool should_pass,
@@ -285,7 +288,7 @@ static void stress_filename_test(
 	struct stat buf;
 
 	/* exercise dcache lookup of non-existent filename */
-	ret = stat(filename, &buf);
+	ret = shim_stat(filename, &buf);
 	if (ret == 0)
 		pr_fail("%s: stat succeeded on non-existent file\n",
 			args->name);
@@ -302,13 +305,13 @@ static void stress_filename_test(
 		(void)close(fd);
 
 		/* exercise dcache lookup of existent filename */
-		VOID_RET(int, stat(filename, &buf));
+		VOID_RET(int, shim_stat(filename, &buf));
 
 		(void)shim_unlink(filename);
 	}
 
 	/* exercise dcache lookup of non-existent filename */
-	ret = stat(filename, &buf);
+	ret = shim_stat(filename, &buf);
 	if (ret == 0)
 		pr_fail("%s: stat succeeded on non-existent unlinked file\n",
 			args->name);
@@ -318,7 +321,7 @@ static void stress_filename_test(
  *  stress_filename()
  *	stress filename sizes etc
  */
-static int stress_filename(const stress_args_t *args)
+static int stress_filename(stress_args_t *args)
 {
 	int ret, rc = EXIT_FAILURE;
 	size_t sz_left, sz_max;
@@ -348,11 +351,11 @@ static int stress_filename(const stress_args_t *args)
 	}
 
 #if defined(HAVE_SYS_STATVFS_H)
-	(void)memset(&buf, 0, sizeof(buf));
+	(void)shim_memset(&buf, 0, sizeof(buf));
 	if (statvfs(pathname, &buf) < 0) {
 		pr_fail("%s: statvfs %s failed, errno=%d (%s)%s\n",
 			args->name, pathname, errno, strerror(errno),
-			stress_fs_type(pathname));
+			stress_get_fs_type(pathname));
 		goto tidy_dir;
 	}
 #else
@@ -385,7 +388,7 @@ static int stress_filename(const stress_args_t *args)
 
 	if (stress_filename_probe_length(args, filename, ptr, &sz_max) < 0) {
 		pr_fail("%s: failed to determine maximum filename length%s\n",
-			args->name, stress_fs_type(pathname));
+			args->name, stress_get_fs_type(pathname));
 		goto tidy_dir;
 	}
 
@@ -401,7 +404,7 @@ static int stress_filename(const stress_args_t *args)
 	default:
 		ret = stress_filename_probe(args, filename, ptr, sz_max, &chars_allowed);
 		if (ret < 0) {
-			rc = exit_status(-ret);
+			rc = stress_exit_status(-ret);
 			goto tidy_dir;
 		}
 		break;
@@ -420,16 +423,16 @@ static int stress_filename(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing_flag()) {
+	if (!stress_continue_flag()) {
 		/* Time to die */
 		rc = EXIT_SUCCESS;
 		goto tidy_dir;
 	}
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto tidy_dir;
 		}
@@ -438,16 +441,14 @@ again:
 	} else if (pid > 0) {
 		int status;
 
-		(void)setpgid(pid, g_pgrp);
 		/* Parent, wait for child */
 		ret = shim_waitpid(pid, &status, 0);
 		if (ret < 0) {
 			if (errno != EINTR)
 				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			(void)kill(pid, SIGTERM);
-			(void)kill(pid, SIGKILL);
-			(void)shim_waitpid(pid, &status, 0);
+			stress_force_killed_bogo(args);
+			(void)stress_kill_pid_wait(pid, NULL);
 		} else if (WIFSIGNALED(status)) {
 			pr_dbg("%s: child died: %s (instance %d)\n",
 				args->name, stress_strsignal(WTERMSIG(status)),
@@ -475,18 +476,17 @@ again:
 		/* Child, wrapped to catch OOMs */
 		const pid_t mypid = getpid();
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		/* Make sure this is killable by OOM killer */
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 
 		i = 0;
 		sz = 1;
 		do {
 			const char ch = allowed[i];
-			const size_t rnd_sz = 1 + (stress_mwc32() % sz_max);
+			const size_t rnd_sz = 1 + stress_mwc32modn((uint32_t)sz_max);
 
 			i++;
 			if (i >= chars_allowed)
@@ -495,68 +495,68 @@ again:
 			/* Should succeed */
 			stress_filename_generate(ptr, 1, ch);
 			stress_filename_test(args, filename, 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, 1, chars_allowed);
 			stress_filename_test(args, filename, 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max, ch);
 			stress_filename_test(args, filename, sz_max, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max, chars_allowed);
 			stress_filename_test(args, filename, sz_max, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max - 1, ch);
 			stress_filename_test(args, filename, sz_max - 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max - 1, chars_allowed);
 			stress_filename_test(args, filename, sz_max - 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should fail */
 			stress_filename_generate(ptr, sz_max + 1, ch);
 			stress_filename_test(args, filename, sz_max + 1, false, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max + 1, chars_allowed);
 			stress_filename_test(args, filename, sz_max + 1, false, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz, ch);
 			stress_filename_test(args, filename, sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz, chars_allowed);
 			stress_filename_test(args, filename, sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, rnd_sz, ch);
 			stress_filename_test(args, filename, rnd_sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, rnd_sz, chars_allowed);
 			stress_filename_test(args, filename, rnd_sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			sz++;
 			if (sz > sz_max)
 				sz = 1;
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 		_exit(EXIT_SUCCESS);
 	}
 	rc = EXIT_SUCCESS;
