@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_LINUX_FUTEX_H)
 #include <linux/futex.h>
@@ -47,10 +49,10 @@ static int stress_futex_wait(uint32_t *futex, const int val, const long nsec)
 	static int try_futex_waitv = true;
 	static int try = 0;
 
-	if (try_futex_waitv && try++ > 16) {
+	if (try_futex_waitv && (try++ > 16)) {
 		struct shim_futex_waitv w;
 
-		(void)memset(&w, 0, sizeof(w));
+		(void)shim_memset(&w, 0, sizeof(w));
 
 		w.val = (uint64_t)val;
 		w.uaddr = (uintptr_t)futex;
@@ -89,19 +91,21 @@ static int stress_futex_wait(uint32_t *futex, const int val, const long nsec)
  * 	efficiently use futex, but to stress the futex system call
  *	by rapidly calling it on wait and wakes
  */
-static int stress_futex(const stress_args_t *args)
+static int stress_futex(stress_args_t *args)
 {
 	uint64_t *timeout = &g_shared->futex.timeout[args->instance];
 	uint32_t *futex = &g_shared->futex.futex[args->instance];
 	pid_t pid;
+	int parent_cpu;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_err("%s: fork failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
@@ -116,7 +120,7 @@ again:
 			 * Break early in case wake gets stuck
 			 * (which it shouldn't)
 			 */
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			ret = shim_futex_wake(futex, 1);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
@@ -125,10 +129,10 @@ again:
 						args->name, errno, strerror(errno));
 				}
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
-		/* Kill waiter process */
-		(void)kill(pid, SIGKILL);
+		/* send alarm to waiter process */
+		(void)shim_kill(pid, SIGALRM);
 		(void)shim_waitpid(pid, &status, 0);
 
 		pr_dbg("%s: futex timeouts: %" PRIu64 "\n",
@@ -136,6 +140,7 @@ again:
 	} else {
 		uint64_t threshold = THRESHOLD;
 
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -144,7 +149,7 @@ again:
 			int ret;
 
 			/* Break early before potential long wait */
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 
 			ret = stress_futex_wait(futex, 0, 5000);
@@ -167,9 +172,9 @@ again:
 							args->name, errno, strerror(errno));
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -187,6 +192,7 @@ stressor_info_t stress_futex_info = {
 stressor_info_t stress_futex_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without linux/futex.h or futex() system call"
 };
