@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King.
+ * Copyright (C) 2021-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-cpu.h"
+#include "core-madvise.h"
+#include "core-out-of-memory.h"
 #include "core-target-clones.h"
 
 #define MIN_MEMFD_BYTES		(2 * MB)
@@ -27,14 +30,22 @@
 #define MAX_MEMFD_FDS		(4096)
 #define DEFAULT_MEMFD_FDS	(256)
 
+#define MEMFD_STRIDE		(8)	/* 16 * sizeof(uint64_t) = 64 bytes */
+
 static const stress_help_t help[] = {
 	{ NULL,	"memfd N",	 "start N workers allocating memory with memfd_create" },
 	{ NULL,	"memfd-bytes N", "allocate N bytes for each stress iteration" },
 	{ NULL,	"memfd-fds N",	 "number of memory fds to open per stressors" },
+	{ NULL,	"memfd-mlock",	 "attempt to mlock pages into memory" },
 	{ NULL,	"memfd-ops N",	 "stop after N memfd bogo operations" },
 	{ NULL,	NULL,		 NULL }
 };
 
+static int stress_set_memfd_mlock(const char *opt)
+{
+	return stress_set_setting_true("memfd-mlock", opt);
+}
+
 /*
  *  stress_set_memfd_bytes
  *	set max size of each memfd size
@@ -66,6 +77,7 @@ static int stress_set_memfd_fds(const char *opt)
 static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_memfd_bytes,	stress_set_memfd_bytes },
 	{ OPT_memfd_fds,	stress_set_memfd_fds },
+	{ OPT_memfd_mlock,	stress_set_memfd_mlock },
 	{ 0,			NULL }
 };
 
@@ -87,6 +99,12 @@ static const unsigned int flags[] = {
     defined(MFD_HUGE_1GB)
 	MFD_HUGETLB | MFD_HUGE_1GB,
 #endif
+#if defined(MFD_NOEXEC_SEAL)
+	MFD_NOEXEC_SEAL,
+#endif
+#if defined(MFD_EXEC)
+	MFD_EXEC,
+#endif
 };
 
 /*
@@ -102,74 +120,109 @@ static inline const uint64_t *uint64_ptr_offset(const uint64_t *ptr, const size_
  *  stress_memfd_fill_pages()
  *	fill pages with random uin64_t values
  */
-static inline void TARGET_CLONES stress_memfd_fill_pages(void *ptr, const size_t size)
+static void stress_memfd_fill_pages_generic(const uint64_t val, void *ptr, const size_t size)
 {
 	register uint64_t *u64ptr = (uint64_t *)ptr;
 	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
-	register uint64_t v = stress_mwc64();
+	register uint64_t v = val;
 
 	while (u64ptr < u64end) {
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
-		*u64ptr++ = v;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
+		*u64ptr = v;
+		u64ptr += MEMFD_STRIDE;
 		v++;
 	}
 }
 
+#if defined(HAVE_NT_STORE64)
+/*
+ *  stress_memfd_fill_pages_nt_store()
+ *	fill pages with random uin64_t values using nt_store
+ */
+static void stress_memfd_fill_pages_nt_store(const uint64_t val, void *ptr, const size_t size)
+{
+	register uint64_t *u64ptr = (uint64_t *)ptr;
+	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
+	register uint64_t v = val;
+
+	while (u64ptr < u64end) {
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		stress_nt_store64(u64ptr, v);
+		u64ptr += MEMFD_STRIDE;
+		v++;
+	}
+}
+#endif
+
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_PAGEOUT)
 /*
  *  stress_memfd_check()
  *	check if buffer buf contains uint64_t values val, return true if OK, false if not
  */
-static bool stress_memfd_check(
+static inline bool stress_memfd_check(
 	const uint64_t val,
-	const uint64_t *buf,
-	const size_t size)
+	const uint64_t *ptr,
+	const size_t size,
+	const uint64_t inc)
 {
-	const uint64_t *ptr, *end_ptr = uint64_ptr_offset(buf, size);
+	register const uint64_t *u64ptr = ptr, *u64_end = uint64_ptr_offset(ptr, size);
+	register uint64_t v = val;
+	bool passed = true;
 
-	for (ptr = buf; ptr < end_ptr; ptr++) {
-		if (*ptr != val)
-			return false;
+	while (u64ptr < u64_end) {
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		passed &= (*u64ptr == v);
+		u64ptr += MEMFD_STRIDE;
+		v += inc;
 	}
-	return true;
+	return passed;
 }
 #endif
 
 /*
  *  Create allocations using memfd_create, ftruncate and mmap
  */
-static int stress_memfd_child(const stress_args_t *args, void *context)
+static int stress_memfd_child(stress_args_t *args, void *context)
 {
 	int *fds, fd;
 	void **maps;
@@ -179,15 +232,21 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 	size_t size;
 	size_t memfd_bytes = DEFAULT_MEMFD_BYTES;
 	uint32_t memfd_fds = DEFAULT_MEMFD_FDS;
-	int mmap_flags = MAP_FILE | MAP_SHARED;
 	double duration = 0.0, count = 0.0, rate;
-
-#if defined(MAP_POPULATE)
-	mmap_flags |= MAP_POPULATE;
+	bool memfd_mlock = false;
+#if defined(HAVE_NT_STORE64)
+	void (*stress_memfd_fill_pages)(uint64_t val, void *ptr, const size_t size) =
+		stress_cpu_x86_has_sse2() ? stress_memfd_fill_pages_nt_store : stress_memfd_fill_pages_generic;
+#else
+	void (*stress_memfd_fill_pages)(uint64_t val, void *ptr, const size_t size) = stress_memfd_fill_pages_generic;
 #endif
 
+	stress_catch_sigill();
+
 	(void)context;
 
+	(void)stress_get_setting("memfd-mlock", &memfd_mlock);
+
 	if (!stress_get_setting("memfd-bytes", &memfd_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			memfd_bytes = MAX_32;
@@ -255,11 +314,11 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 				default:
 					pr_fail("%s: memfd_create failed: errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-					keep_stressing_set_flag(false);
+					stress_continue_set_flag(false);
 					goto memfd_unmap;
 				}
 			}
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
@@ -274,7 +333,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			if (fds[i] < 0)
 				continue;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 
 			/* Allocate space */
@@ -290,12 +349,18 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 				}
 			}
 			/*
-			 * ..and map it in, using MAP_POPULATE
-			 * to force page it in
+			 *  ..and map it in, using MAP_POPULATE
+			 *  to force page it in
 			 */
-			maps[i] = mmap(NULL, size, PROT_WRITE, mmap_flags, fds[i], 0);
-			stress_memfd_fill_pages(maps[i], size);
+			maps[i] = stress_mmap_populate(NULL, size, PROT_WRITE,
+					MAP_FILE | MAP_SHARED, fds[i], 0);
+			if (maps[i] == MAP_FAILED)
+				continue;
+			if (memfd_mlock)
+				(void)shim_mlock(maps[i], size);
+			stress_memfd_fill_pages(stress_mwc64(), maps[i], size);
 			(void)stress_madvise_random(maps[i], size);
+			(void)stress_madvise_mergeable(maps[i], size);
 
 #if defined(FALLOC_FL_PUNCH_HOLE) &&	\
     defined(FALLOC_FL_KEEP_SIZE)
@@ -313,13 +378,15 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			 */
 			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, (off_t)size, 0));
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
 		for (i = 0; i < memfd_fds; i++) {
 			if (fds[i] < 0)
 				continue;
+			if (maps[i] == MAP_FAILED)
+				continue;
 #if defined(SEEK_SET)
 			if (lseek(fds[i], (off_t)size >> 1, SEEK_SET) < 0) {
 				if ((errno != ENXIO) && (errno != EINVAL))
@@ -355,7 +422,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 						args->name, errno, strerror(errno));
 			}
 #endif
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
@@ -370,10 +437,9 @@ memfd_unmap:
 		 *  Check for zap_pte bug, see Linux commit
 		 *  5abfd71d936a8aefd9f9ccd299dea7a164a5d455
 		 */
-		for (i = 0; keep_stressing_flag() && (i < memfd_fds); i++) {
-			uint64_t *buf, *ptr;
-			const uint64_t *end_ptr;
-			uint64_t val = stress_mwc64();
+		for (i = 0; stress_continue_flag() && (i < memfd_fds); i++) {
+			uint64_t *buf;
+			uint64_t val;
 			const ssize_t test_size = page_size << 1;
 
 			if (ftruncate(fds[i], (off_t)test_size) < 0)
@@ -382,9 +448,10 @@ memfd_unmap:
 					MAP_PRIVATE, fds[i], 0);
 			if (buf == MAP_FAILED)
 				continue;
-			end_ptr = uint64_ptr_offset(buf, test_size);
-			for (ptr = buf; ptr < end_ptr; ptr++)
-				*ptr = val;
+			if (memfd_mlock)
+				(void)shim_mlock(buf, test_size);
+			val = stress_mwc64();
+			stress_memfd_fill_pages(val, buf, test_size);
 
 			if (madvise(buf, test_size, MADV_PAGEOUT) < 0)
 				goto buf_unmap;
@@ -392,10 +459,10 @@ memfd_unmap:
 				goto buf_unmap;
 			if (ftruncate(fds[i], (off_t)test_size) < 0)
 				goto buf_unmap;
-			if (!stress_memfd_check(val, buf, page_size))
+			if (!stress_memfd_check(val, buf, page_size, 1))
 				pr_fail("%s: unexpected memfd %d data mismatch in first page\n",
 					args->name, fds[i]);
-			if (!stress_memfd_check(0ULL, uint64_ptr_offset(buf, page_size), page_size))
+			if (!stress_memfd_check(0ULL, uint64_ptr_offset(buf, page_size), page_size, 0))
 				pr_fail("%s: unexpected memfd %d data mismatch in zero'd second page\n",
 					args->name, fds[i]);
 buf_unmap:
@@ -404,10 +471,7 @@ buf_unmap:
 		}
 #endif
 
-		for (i = 0; i < memfd_fds; i++) {
-			if (fds[i] >= 0)
-				(void)close(fds[i]);
-		}
+		stress_close_fds(fds, memfd_fds);
 
 		/* Exercise illegal memfd name */
 		stress_rndstr(filename, sizeof(filename));
@@ -463,13 +527,14 @@ buf_unmap:
 				(void)close(fd);
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	rate = (count > 0.0) ? duration / count : 0.0;
-	stress_metrics_set(args, 0, "nanosecs per memfd_create call", rate * STRESS_DBL_NANOSECOND);
+	stress_metrics_set(args, 0, "nanosecs per memfd_create call",
+		rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
 
 	free(maps);
 	free(fds);
@@ -481,7 +546,7 @@ buf_unmap:
  *  stress_memfd()
  *	stress memfd
  */
-static int stress_memfd(const stress_args_t *args)
+static int stress_memfd(stress_args_t *args)
 {
 	return stress_oomable_child(args, NULL, stress_memfd_child, STRESS_OOMABLE_NORMAL);
 }
@@ -498,6 +563,7 @@ stressor_info_t stress_memfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without memfd_create() system call"
 };
