@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 #include "core-pthread.h"
 #include "core-put.h"
 
@@ -47,14 +48,14 @@ static const stress_help_t help[] = {
 #define MAX_PROCFS_THREADS	(4)
 
 typedef struct stress_ctxt {
-	const stress_args_t *args;
+	stress_args_t *args;
 	const char *path;
 	bool writeable;
 } stress_ctxt_t;
 
 typedef struct {
 	const char *filename;
-	void (*stress_func)(const stress_args_t *args, const int fd);
+	void (*stress_func)(stress_args_t *args, const int fd);
 } stress_proc_info_t;
 
 #if !defined(NSIO)
@@ -159,7 +160,7 @@ static int mixup_sort(const struct dirent **d1, const struct dirent **d2)
  *	check if /proc/self/mem can be mmap'd and read and values
  *	match that of mmap'd page and page read via the fd
  */
-static void stress_proc_self_mem(const stress_args_t *args, const int fd)
+static void stress_proc_self_mem(stress_args_t *args, const int fd)
 {
 	uint8_t *page, *buf, rnd = stress_mwc8();
 	const size_t page_size = stress_get_page_size();
@@ -177,10 +178,10 @@ static void stress_proc_self_mem(const stress_args_t *args, const int fd)
 	}
 	offset = (off_t)(uintptr_t)page;
 
-	(void)memset(page, rnd, page_size);
+	(void)shim_memset(page, rnd, page_size);
 	if (lseek(fd, offset, SEEK_SET) == offset) {
 		ssize_t ret;
-		size_t mem_offset = stress_mwc32() % page_size;
+		size_t mem_offset = stress_mwc32modn((uint32_t)page_size);
 
 		ret = read(fd, buf, page_size);
 		if ((ret == (ssize_t)page_size) &&
@@ -201,13 +202,13 @@ static void stress_proc_self_mem(const stress_args_t *args, const int fd)
  *  stress_proc_mtrr()
  *	exercise /proc/mtrr ioctl MTRRIOC_GET_ENTRY
  */
-static void stress_proc_mtrr(const stress_args_t *args, const int fd)
+static void stress_proc_mtrr(stress_args_t *args, const int fd)
 {
 	struct mtrr_gentry gentry;
 
 	(void)args;
 
-	(void)memset(&gentry, 0, sizeof(gentry));
+	(void)shim_memset(&gentry, 0, sizeof(gentry));
 	while (ioctl(fd, MTRRIOC_GET_ENTRY, &gentry) == 0) {
 		gentry.regnum++;
 	}
@@ -221,7 +222,7 @@ static void stress_proc_mtrr(const stress_args_t *args, const int fd)
 #if defined(HAVE_LINUX_PCI_H) &&	\
     defined(PCIIOC_CONTROLLER) &&	\
     !defined(STRESS_ARCH_SH4)
-static void stress_proc_pci(const stress_args_t *args, const int fd)
+static void stress_proc_pci(stress_args_t *args, const int fd)
 {
 	(void)args;
 
@@ -281,7 +282,7 @@ static inline void stress_proc_rw(
 		(void)shim_strlcpy(path, proc_path, sizeof(path));
 		(void)shim_pthread_spin_unlock(&lock);
 
-		if (!*path || !keep_stressing_flag())
+		if (!*path || !stress_continue_flag())
 			break;
 
 		if (!strncmp(path, "/proc/self", 10))
@@ -294,12 +295,8 @@ static inline void stress_proc_rw(
 		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
 			return;
 
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
-
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
 		/*
 		 *  Check if there any special features to exercise
 		 */
@@ -313,7 +310,7 @@ static inline void stress_proc_rw(
 		/*
 		 *  fstat the file
 		 */
-		ret = fstat(fd, &statbuf);
+		ret = shim_fstat(fd, &statbuf);
 		if (ret == 0) {
 #if defined(S_IFMT) &&	\
     defined(S_IFIFO)
@@ -348,7 +345,7 @@ static inline void stress_proc_rw(
 		/*
 		 *  Random sized reads at non-power of two integers
 		 *  on /proc/bus/pci/00 on SH4 5.16 kernels trips
-		 *  SIGBUS/SIGSEGV faults. Currenly skip this test.
+		 *  SIGBUS/SIGSEGV faults. Currently skip this test.
 		 */
 #if defined(STRESS_ARCH_SH4)
 		if (!strncmp(path, "/proc/bus/pci/00", 16))
@@ -358,9 +355,9 @@ static inline void stress_proc_rw(
 		 *  Multiple randomly sized reads
 		 */
 		for (i = 0; i < 4096 * PROC_BUF_SZ; i++) {
-			const ssize_t sz = 1 + (stress_mwc32() % sizeof(buffer));
+			const ssize_t sz = 1 + stress_mwc32modn((uint32_t)sizeof(buffer));
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			ret = read(fd, buffer, (size_t)sz);
 			if (ret < 0)
@@ -369,55 +366,35 @@ static inline void stress_proc_rw(
 				break;
 			i += sz;
 
-			if (stress_time_now() - t_start > threshold) {
-				timeout = true;
-				(void)close(fd);
-				goto next;
-			}
+			if ((stress_time_now() - t_start) > threshold)
+				goto timeout_close;
 		}
 		(void)close(fd);
 
 		/* Multiple 1 char sized reads */
 		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
 			return;
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
 		for (i = 0; ; i++) {
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
+			if (((i & 0x0f) == 0) && ((stress_time_now() - t_start) > threshold))
+				goto timeout_close;
 			ret = read(fd, buffer, 1);
 			if (ret < 1)
 				break;
-			if (stress_time_now() - t_start > threshold) {
-				timeout = true;
-				(void)close(fd);
-				goto next;
-			}
 		}
 		(void)close(fd);
 
 		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
 			return;
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
 		/*
 		 *  Zero sized reads
 		 */
 		ret = read(fd, buffer, 0);
 		if (ret < 0)
 			goto err;
-
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
 		/*
 		 *  Broken offset reads, see Linux commit
 		 *  3bfa7e141b0bbb818b25e0daafb65aee92e49ac4
@@ -427,14 +404,14 @@ static inline void stress_proc_rw(
 		pos = lseek(fd, 0, SEEK_SET);
 		if (pos < 0)
 			goto mmap_test;
-		(void)memset(buffer, 0, sizeof(buffer));
+		(void)shim_memset(buffer, 0, sizeof(buffer));
 		ret = read(fd, buffer, sizeof(buffer));
 		if (ret < 0)
 			goto mmap_test;
 		if (ret < (ssize_t)(sizeof(buffer) >> 1)) {
 			char *bptr;
 
-			for (bptr = buffer; *bptr && *bptr != '\n'; bptr++)
+			for (bptr = buffer; *bptr && (*bptr != '\n'); bptr++)
 				;
 			if (*bptr == '\n') {
 				const off_t offset = 2 + (bptr - buffer);
@@ -479,17 +456,18 @@ mmap_test:
 		 *  mmap it
 		 */
 		ptr = mmap(NULL, page_size, PROT_READ,
-			MAP_SHARED | MAP_ANONYMOUS, fd, 0);
+#if defined(MAP_POPULATE)
+			MAP_POPULATE |
+#endif
+			MAP_SHARED | MAP_ANONYMOUS,
+			fd, 0);
 		if (ptr != MAP_FAILED) {
 			stress_uint8_put(*ptr);
 			(void)munmap(ptr, page_size);
 		}
 
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
 
 #if defined(FIONREAD)
 		{
@@ -500,11 +478,8 @@ mmap_test:
 			 */
 			VOID_RET(int, ioctl(fd, FIONREAD, &nbytes));
 		}
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
 #endif
 
 #if defined(HAVE_POLL_H)
@@ -557,19 +532,15 @@ mmap_test:
 		if (pos == (off_t)-1)
 			goto err;
 
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			(void)close(fd);
-			goto next;
-		}
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
 
 		VOID_RET(ssize_t, read(fd, buffer, 1));
 err:
+		if ((stress_time_now() - t_start) > threshold)
+			goto timeout_close;
+
 		(void)close(fd);
-		if (stress_time_now() - t_start > threshold) {
-			timeout = true;
-			goto next;
-		}
 
 		if (writeable && ctxt->writeable) {
 			/*
@@ -589,7 +560,7 @@ err:
 
 		/* /proc + ... */
 		if (len > 5) {
-			char *pptr = path + 5 + (stress_mwc16() % (len - 5));
+			char *pptr = path + 5 + stress_mwc16modn(len - 5);
 
 			/* Skip over / */
 			while (*pptr == '/')
@@ -613,6 +584,12 @@ next:
 				break;
 			loops--;
 		}
+		continue;
+
+timeout_close:
+		(void)close(fd);
+		timeout = true;
+		goto next;
 	}
 }
 
@@ -624,15 +601,16 @@ next:
 static void *stress_proc_rw_thread(void *ctxt_ptr)
 {
 	static void *nowt = NULL;
-	stress_ctxt_t *ctxt = (stress_ctxt_t *)ctxt_ptr;
+	const stress_ctxt_t *ctxt = (stress_ctxt_t *)ctxt_ptr;
 
 	/*
 	 *  Block all signals, let controlling thread
 	 *  handle these
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
+	(void)pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		stress_proc_rw(ctxt, -1);
 		if (!*proc_path)
 			break;
@@ -652,13 +630,13 @@ static void stress_proc_dir(
 	const int depth)
 {
 	struct dirent **dlist;
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
 	int32_t loops = args->instance < 8 ?
 			(int32_t)(args->instance + 1) : 8;
 	int i, n, ret;
 	char tmp[PATH_MAX];
 
-	if (!keep_stressing_flag())
+	if (!stress_continue_flag())
 		return;
 
 	/* Don't want to go too deep */
@@ -674,7 +652,7 @@ static void stress_proc_dir(
 	}
 
 	/* Non-directories files first */
-	for (i = 0; (i < n) && keep_stressing_flag(); i++) {
+	for (i = 0; (i < n) && stress_continue_flag(); i++) {
 		struct dirent *d = dlist[i];
 
 		if (stress_is_dot_filename(d->d_name)) {
@@ -691,7 +669,7 @@ static void stress_proc_dir(
 				(void)shim_pthread_spin_unlock(&lock);
 
 				stress_proc_rw(ctxt, loops);
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 			free(d);
 			dlist[i] = NULL;
@@ -704,17 +682,17 @@ static void stress_proc_dir(
 	}
 
 	/* Now recurse on directories */
-	for (i = 0; i < n && keep_stressing_flag(); i++) {
+	for (i = 0; (i < n) && stress_continue_flag(); i++) {
 		struct dirent *d = dlist[i];
 
-		if (d && d->d_type == DT_DIR) {
+		if (d && (d->d_type == DT_DIR)) {
 			(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
 
 			free(d);
 			dlist[i] = NULL;
 
 			stress_proc_dir(ctxt, tmp, recurse, depth + 1);
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 	}
 	stress_dirent_list_free(dlist, n);
@@ -764,7 +742,7 @@ static char *stress_random_pid(void)
  *  stress_procfs_no_entries()
  *	report when no /proc entries are found
  */
-static int stress_procfs_no_entries(const stress_args_t *args)
+static int stress_procfs_no_entries(stress_args_t *args)
 {
 	if (args->instance == 0)
 		pr_inf_skip("%s: no /proc entries found, skipping stressor\n", args->name);
@@ -775,7 +753,7 @@ static int stress_procfs_no_entries(const stress_args_t *args)
  *  stress_procfs
  *	stress reading all of /proc
  */
-static int stress_procfs(const stress_args_t *args)
+static int stress_procfs(stress_args_t *args)
 {
 	int i, n;
 	pthread_t pthreads[MAX_PROCFS_THREADS];
@@ -789,7 +767,7 @@ static int stress_procfs(const stress_args_t *args)
 
 	(void)sigfillset(&set);
 
-	shim_strlcpy(proc_path, "/proc/self", sizeof(proc_path));
+	(void)shim_strlcpy(proc_path, "/proc/self", sizeof(proc_path));
 
 	ctxt.args = args;
 	ctxt.writeable = (geteuid() != 0);
@@ -801,7 +779,7 @@ static int stress_procfs(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	(void)memset(ret, 0, sizeof(ret));
+	(void)shim_memset(ret, 0, sizeof(ret));
 
 	for (i = 0; i < MAX_PROCFS_THREADS; i++) {
 		ret[i] = pthread_create(&pthreads[i], NULL,
@@ -815,9 +793,9 @@ static int stress_procfs(const stress_args_t *args)
 
 		for (i = 0; i < n; i++) {
 			char procfspath[PATH_MAX];
-			struct dirent *d = dlist[i];
+			const struct dirent *d = dlist[i];
 
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			stress_mk_filename(procfspath, sizeof(procfspath), "/proc", d->d_name);
@@ -827,37 +805,39 @@ static int stress_procfs(const stress_args_t *args)
 					(void)shim_pthread_spin_unlock(&lock);
 
 					stress_proc_rw(&ctxt, 8);
-					inc_counter(args);
+					stress_bogo_inc(args);
 				}
 			} else if (d->d_type == DT_DIR) {
 				stress_proc_dir(&ctxt, procfspath, true, 0);
 			}
 
 			j = (j + args->num_instances) % (size_t)n;
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		stress_proc_dir(&ctxt, stress_random_pid(), true, 0);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = shim_pthread_spin_lock(&lock);
 	if (rc) {
 		pr_dbg("%s: spin lock failed for %s\n", args->name, proc_path);
 	} else {
-		shim_strlcpy(proc_path, "", sizeof(proc_path));
+		(void)shim_strlcpy(proc_path, "", sizeof(proc_path));
 		VOID_RET(int, shim_pthread_spin_unlock(&lock));
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 0; i < MAX_PROCFS_THREADS; i++) {
-		if (ret[i] == 0)
+		if (ret[i] == 0) {
+			(void)pthread_cancel(pthreads[i]);
 			(void)pthread_join(pthreads[i], NULL);
+		}
 	}
 	(void)shim_pthread_spin_destroy(&lock);
 
