@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 #include "core-arch.h"
 #include "core-cpu.h"
 #include "core-put.h"
+#include "core-target-clones.h"
 
 #if defined(STRESS_ARCH_X86) &&	\
     defined(HAVE_INT128_T)
@@ -33,11 +34,11 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if (defined(__GNUC__) && NEED_GNUC(8, 0, 0)) &&	\
-    !defined(__clang__)	&&	\
-    !defined(__ICC) && 		\
-    !defined(__PCC__) &&	\
-    !defined(__TINYC__)
+#if (defined(HAVE_COMPILER_GCC_OR_MUSL) && NEED_GNUC(8, 0, 0)) &&	\
+    !defined(HAVE_COMPILER_CLANG) &&					\
+    !defined(HAVE_COMPILER_ICC) &&					\
+    !defined(HAVE_COMPILER_PCC) &&					\
+    !defined(HAVE_COMPILER_TCC)
 
 static volatile uint32_t stash32;
 static volatile uint64_t stash64;
@@ -71,7 +72,7 @@ do {				\
 } while (0);
 
 static void regs_check32(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *reg,
 	const uint32_t expected,
 	const uint32_t value)
@@ -84,7 +85,7 @@ static void regs_check32(
 }
 
 static void regs_check64(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *reg,
 	const uint64_t expected,
 	const uint64_t value)
@@ -98,7 +99,7 @@ static void regs_check64(
 
 #if defined(HAVE_INT128_T)
 static void regs_check128(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *reg,
 	const __uint128_t expected,
 	const __uint128_t value)
@@ -137,14 +138,14 @@ do {								\
 #define STRESS_REGS_HELPER
 
 #if defined(HAVE_INT128_T)
-static void OPTIMIZE0 stress_regs_helper_mmx(const stress_args_t *args, register uint64_t v)
+static void OPTIMIZE0 stress_regs_helper_mmx(stress_args_t *args, register uint64_t v)
 {
 	__uint128_t v128 = ((__uint128_t)v << 64) | (v ^ 0xa55a5555aaaaULL);
 	register __uint128_t xmm0 __asm__("xmm0") = v128;
-	register __uint128_t xmm1 __asm__("xmm1") = v128 >> 1;
-	register __uint128_t xmm2 __asm__("xmm2") = v128 << 1;
-	register __uint128_t xmm3 __asm__("xmm3") = v128 >> 2;
-	register __uint128_t xmm4 __asm__("xmm4") = v128 << 2;
+	register __uint128_t xmm1 __asm__("xmm1") = xmm0 >> 1;
+	register __uint128_t xmm2 __asm__("xmm2") = xmm0 << 1;
+	register __uint128_t xmm3 __asm__("xmm3") = xmm0 >> 2;
+	register __uint128_t xmm4 __asm__("xmm4") = xmm0 << 2;
 	register __uint128_t xmm5 __asm__("xmm5") = ~xmm0;
 	register __uint128_t xmm6 __asm__("xmm6") = ~xmm1;
 	register __uint128_t xmm7 __asm__("xmm7") = ~xmm2;
@@ -174,14 +175,14 @@ do {			\
 #endif
 
 #if defined(HAVE_INT128_T)
-static void OPTIMIZE0 stress_regs_helper_sse(const stress_args_t *args, register uint64_t v)
+static void OPTIMIZE0 stress_regs_helper_sse(stress_args_t *args, register uint64_t v)
 {
 	__uint128_t v128 = ((__uint128_t)v << 64) | (v ^ 0xa55a5555aaaaULL);
 	register __uint128_t xmm0  __asm__("xmm0")  = v128;
-	register __uint128_t xmm1  __asm__("xmm1")  = v128 >> 1;
-	register __uint128_t xmm2  __asm__("xmm2")  = v128 << 1;
-	register __uint128_t xmm3  __asm__("xmm3")  = v128 >> 2;
-	register __uint128_t xmm4  __asm__("xmm4")  = v128 << 2;
+	register __uint128_t xmm1  __asm__("xmm1")  = xmm0 >> 1;
+	register __uint128_t xmm2  __asm__("xmm2")  = xmm0 << 1;
+	register __uint128_t xmm3  __asm__("xmm3")  = xmm0 >> 2;
+	register __uint128_t xmm4  __asm__("xmm4")  = xmm0 << 2;
 	register __uint128_t xmm5  __asm__("xmm5")  = ~xmm0;
 	register __uint128_t xmm6  __asm__("xmm6")  = ~xmm1;
 	register __uint128_t xmm7  __asm__("xmm7")  = ~xmm2;
@@ -232,13 +233,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress x86_64 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t rax __asm__("rax") = v;
-	register uint64_t rbx __asm__("rbx") = v >> 1;
-	register uint64_t rcx __asm__("rcx") = v << 1;
-	register uint64_t rdx __asm__("rdx") = v >> 2;
-	register uint64_t rsi __asm__("rsi") = v << 2;
+	register uint64_t rbx __asm__("rbx") = rax >> 1;
+	register uint64_t rcx __asm__("rcx") = rax << 1;
+	register uint64_t rdx __asm__("rdx") = rax >> 2;
+	register uint64_t rsi __asm__("rsi") = rax << 2;
 	register uint64_t rdi __asm__("rdi") = ~rax;
 	register uint64_t r8  __asm__("r8")  = ~rbx;
 	register uint64_t r9  __asm__("r9")  = ~rcx;
@@ -296,14 +297,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress i386 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
-
 	register uint32_t eax __asm__("eax") = v32;
-	register uint32_t ecx __asm__("ecx") = v32 >> 1;
-	register uint32_t ebx __asm__("ebx") = v32 << 1;
-	register uint32_t edx __asm__("edx") = v32 >> 2;
+	register uint32_t ecx __asm__("ecx") = eax >> 1;
+	register uint32_t ebx __asm__("ebx") = eax << 1;
+	register uint32_t edx __asm__("edx") = eax >> 2;
 
 #define SHUFFLE_REGS()	\
 do {			\
@@ -329,15 +329,14 @@ do {			\
  *  stress_regs_helper(void)
  *	stress hppa PA risc 1 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
-
 	register uint32_t r4  __asm__("r4")  = v32;
-	register uint32_t r5  __asm__("r5")  = v32 >> 1;
-	register uint32_t r6  __asm__("r6")  = v32 << 1;
-	register uint32_t r7  __asm__("r7")  = v32 >> 2;
-	register uint32_t r8  __asm__("r8")  = v32 << 2;
+	register uint32_t r5  __asm__("r5")  = r4 >> 1;
+	register uint32_t r6  __asm__("r6")  = r4 << 1;
+	register uint32_t r7  __asm__("r7")  = r4 >> 2;
+	register uint32_t r8  __asm__("r8")  = r4 << 2;
 	register uint32_t r9  __asm__("r9")  = ~r4;
 	register uint32_t r10 __asm__("r10") = ~r5;
 	register uint32_t r11 __asm__("r11") = ~r6;
@@ -390,16 +389,15 @@ do {			\
  *  stress_regs_helper(void)
  *	stress m68000 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
-
 	register uint32_t d1 __asm__("d1") = v32;
-	register uint32_t d2 __asm__("d2") = v32 >> 1;
-	register uint32_t d3 __asm__("d3") = v32 << 1;
-	register uint32_t d4 __asm__("d4") = v32 >> 2;
-	register uint32_t d5 __asm__("d5") = v32 << 2;
-	register uint32_t d6 __asm__("d6") = v32 << 2;
+	register uint32_t d2 __asm__("d2") = d1 >> 1;
+	register uint32_t d3 __asm__("d3") = d1 << 1;
+	register uint32_t d4 __asm__("d4") = d1 >> 2;
+	register uint32_t d5 __asm__("d5") = d1 << 2;
+	register uint32_t d6 __asm__("d6") = d1 << 2;
 
 #define SHUFFLE_REGS()	\
 do {			\
@@ -429,15 +427,14 @@ do {			\
  *  stress_regs_helper(void)
  *	stress 32 bit sh4 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
-
 	register uint32_t r2 __asm__("r2") = v32;
-	register uint32_t r3 __asm__("r3") = v32 >> 1;
-	register uint32_t r4 __asm__("r4") = v32 << 1;
-	register uint32_t r5 __asm__("r5") = v32 >> 2;
-	register uint32_t r6 __asm__("r6") = v32 << 2;
+	register uint32_t r3 __asm__("r3") = r2 >> 1;
+	register uint32_t r4 __asm__("r4") = r2 << 1;
+	register uint32_t r5 __asm__("r5") = r2 >> 2;
+	register uint32_t r6 __asm__("r6") = r2 << 2;
 	register uint32_t r7 __asm__("r7") = ~r2;
 	register uint32_t r8 __asm__("r8") = ~r3;
 	register uint32_t r9 __asm__("r9") = ~r4;
@@ -473,20 +470,20 @@ do {			\
 }
 #endif
 
-#if defined(STRESS_ARCH_RISCV)
+#if defined(STRESS_ARCH_RISCV) && (__riscv_xlen == 64)
 
 #define STRESS_REGS_HELPER
 /*
  *  stress_regs_helper(void)
  *	stress RISCV registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t s1  __asm__("s1")  = v;
-	register uint64_t s2  __asm__("s2")  = v >> 1;
-	register uint64_t s3  __asm__("s3")  = v << 1;
-	register uint64_t s4  __asm__("s4")  = v >> 2;
-	register uint64_t s5  __asm__("s5")  = v << 2;
+	register uint64_t s2  __asm__("s2")  = s1 >> 1;
+	register uint64_t s3  __asm__("s3")  = s1 << 1;
+	register uint64_t s4  __asm__("s4")  = s1 >> 2;
+	register uint64_t s5  __asm__("s5")  = s1 << 2;
 	register uint64_t s6  __asm__("s6")  = ~s1;
 	register uint64_t s7  __asm__("s7")  = ~s2;
 	register uint64_t s8  __asm__("s8")  = ~s3;
@@ -527,13 +524,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress Alpha registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t t0  __asm__("$1")  = v;
-	register uint64_t t1  __asm__("$2")  = v >> 1;
-	register uint64_t t2  __asm__("$3")  = v << 1;
-	register uint64_t t3  __asm__("$4")  = v >> 2;
-	register uint64_t t4  __asm__("$5")  = v << 2;
+	register uint64_t t1  __asm__("$2")  = t0 >> 1;
+	register uint64_t t2  __asm__("$3")  = t0 << 1;
+	register uint64_t t3  __asm__("$4")  = t0 >> 2;
+	register uint64_t t4  __asm__("$5")  = t0 << 2;
 	register uint64_t t5  __asm__("$6")  = ~t0;
 	register uint64_t t6  __asm__("$7")  = ~t1;
 	register uint64_t t7  __asm__("$8")  = ~t2;
@@ -576,13 +573,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress PPC64 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t r14 __asm__("r14") = v;
-	register uint64_t r15 __asm__("r15") = v >> 1;
-	register uint64_t r16 __asm__("r16") = v << 1;
-	register uint64_t r17 __asm__("r17") = v >> 2;
-	register uint64_t r18 __asm__("r18") = v << 2;
+	register uint64_t r15 __asm__("r15") = r14 >> 1;
+	register uint64_t r16 __asm__("r16") = r14 << 1;
+	register uint64_t r17 __asm__("r17") = r14 >> 2;
+	register uint64_t r18 __asm__("r18") = r14 << 2;
 	register uint64_t r19 __asm__("r19") = ~r14;
 	register uint64_t r20 __asm__("r20") = ~r15;
 	register uint64_t r21 __asm__("r21") = ~r16;
@@ -639,13 +636,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress sparc registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t l0 __asm__("l0") = v;
-	register uint64_t l1 __asm__("l1") = v >> 1;
-	register uint64_t l2 __asm__("l2") = v << 1;
-	register uint64_t l3 __asm__("l3") = v >> 2;
-	register uint64_t l4 __asm__("l4") = v << 2;
+	register uint64_t l1 __asm__("l1") = l0 >> 1;
+	register uint64_t l2 __asm__("l2") = l0 << 1;
+	register uint64_t l3 __asm__("l3") = l0 >> 2;
+	register uint64_t l4 __asm__("l4") = l0 << 2;
 	register uint64_t l5 __asm__("l5") = ~l0;
 	register uint64_t l6 __asm__("l6") = ~l1;
 	register uint64_t l7 __asm__("l7") = ~l2;
@@ -682,13 +679,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress MIPS64R2 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t s0 __asm__("s0") = v;
-	register uint64_t s1 __asm__("s1") = v >> 1;
-	register uint64_t s2 __asm__("s2") = v << 1;
-	register uint64_t s3 __asm__("s3") = v >> 2;
-	register uint64_t s4 __asm__("s4") = v << 2;
+	register uint64_t s1 __asm__("s1") = s0 >> 1;
+	register uint64_t s2 __asm__("s2") = s0 << 1;
+	register uint64_t s3 __asm__("s3") = s0 >> 2;
+	register uint64_t s4 __asm__("s4") = s0 << 2;
 	register uint64_t s5 __asm__("s5") = ~s0;
 	register uint64_t s6 __asm__("s6") = ~s1;
 	register uint64_t s7 __asm__("s7") = ~s2;
@@ -725,15 +722,15 @@ do {			\
  *  stress_regs_helper(void)
  *	stress MIPS324R2 registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
 
 	register uint32_t s0 __asm__("s0") = v32;
-	register uint32_t s1 __asm__("s1") = v32 >> 1;
-	register uint32_t s2 __asm__("s2") = v32 << 1;
-	register uint32_t s3 __asm__("s3") = v32 >> 2;
-	register uint32_t s4 __asm__("s4") = v32 << 2;
+	register uint32_t s1 __asm__("s1") = s0 >> 1;
+	register uint32_t s2 __asm__("s2") = s0 << 1;
+	register uint32_t s3 __asm__("s3") = s0 >> 2;
+	register uint32_t s4 __asm__("s4") = s0 << 2;
 	register uint32_t s5 __asm__("s5") = ~s0;
 	register uint32_t s6 __asm__("s6") = ~s1;
 	register uint32_t s7 __asm__("s7") = ~s2;
@@ -770,13 +767,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress ARM64 (aarch64) registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t x0  __asm__("x0")  = v;
-	register uint64_t x1  __asm__("x1")  = v >> 1;
-	register uint64_t x2  __asm__("x2")  = v << 1;
-	register uint64_t x3  __asm__("x3")  = v >> 2;
-	register uint64_t x4  __asm__("x4")  = v << 2;
+	register uint64_t x1  __asm__("x1")  = x0 >> 1;
+	register uint64_t x2  __asm__("x2")  = x0 << 1;
+	register uint64_t x3  __asm__("x3")  = x0 >> 2;
+	register uint64_t x4  __asm__("x4")  = x0 << 2;
 	register uint64_t x5  __asm__("x5")  = ~x0;
 	register uint64_t x6  __asm__("x6")  = ~x1;
 	register uint64_t x7  __asm__("x7")  = ~x2;
@@ -863,15 +860,15 @@ do {			\
  *  stress_regs_helper(void)
  *	stress 32 bit ARM registers
  */
-static void NOINLINE OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void NOINLINE OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	uint32_t v32 = (uint32_t)v;
 
 	register uint32_t r0  __asm__("r0")  = v32;
-	register uint32_t r1  __asm__("r1")  = v32 >> 1;
-	register uint32_t r2  __asm__("r2")  = v32 << 1;
-	register uint32_t r3  __asm__("r3")  = v32 >> 2;
-	register uint32_t r4  __asm__("r4")  = v32 << 2;
+	register uint32_t r1  __asm__("r1")  = r0 >> 1;
+	register uint32_t r2  __asm__("r2")  = r0 << 1;
+	register uint32_t r3  __asm__("r3")  = r0 >> 2;
+	register uint32_t r4  __asm__("r4")  = r0 << 2;
 	register uint32_t r5  __asm__("r5")  = ~r0;
 	register uint32_t r6  __asm__("r6")  = ~r1;
 	register uint32_t r8  __asm__("r8")  = ~r2;
@@ -908,13 +905,13 @@ do {			\
  *  stress_regs_helper(void)
  *	stress registers, generic version
  */
-static void OPTIMIZE0 stress_regs_helper(const stress_args_t *args, register uint64_t v)
+static void OPTIMIZE0 stress_regs_helper(stress_args_t *args, register uint64_t v)
 {
 	register uint64_t r1  = v;
-	register uint64_t r2  = v >> 1;
-	register uint64_t r3  = v << 1;
-	register uint64_t r4  = v >> 2;
-	register uint64_t r5  = v << 2;
+	register uint64_t r2  = r1 >> 1;
+	register uint64_t r3  = r1 << 1;
+	register uint64_t r4  = r1 >> 2;
+	register uint64_t r5  = r1 << 2;
 	register uint64_t r6  = ~r1;
 	register uint64_t r7  = ~r2;
 	register uint64_t r8  = ~r3;
@@ -952,7 +949,7 @@ do {			\
  *  stress_regs()
  *	stress x86 syscall instruction
  */
-static int stress_regs(const stress_args_t *args)
+static int stress_regs(stress_args_t *args)
 {
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 	uint64_t v = stress_mwc64();
@@ -967,11 +964,11 @@ static int stress_regs(const stress_args_t *args)
 	do {
 		int i;
 
-		for (i = 0; keep_stressing_flag() & (i < 1000); i++)
+		for (i = 0; stress_continue_flag() & (i < 1000); i++)
 			stress_regs_helper(args, v);
 		v++;
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -989,7 +986,7 @@ stressor_info_t stress_regs_info = {
 
 stressor_info_t stress_regs_info = {
 	.stressor = stress_unimplemented,
-	.verify = VERIFY_NONE,
+	.verify = VERIFY_ALWAYS,
 	.class = CLASS_CPU,
 	.help = help,
 	.unimplemented_reason = "built without gcc 8 or higher supporting asm register assignments"
