@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 
 #if defined(HAVE_SYS_FSUID_H)
@@ -102,11 +103,11 @@ static const stress_help_t help[] = {
  *  stress on set*() calls
  *	stress system by rapid get*() system calls
  */
-static int stress_set(const stress_args_t *args)
+static int stress_set(stress_args_t *args)
 {
 	size_t i;
 	int ret_hostname;
-	const size_t hostname_len = stress_hostname_length();
+	const size_t hostname_len = stress_get_hostname_length();
 	const size_t longname_len = hostname_len << 1;
 	char *hostname;
 	char *longname;
@@ -129,13 +130,13 @@ static int stress_set(const stress_args_t *args)
 
 	hostname = calloc(hostname_len, sizeof(*hostname));
 	if (!hostname) {
-		pr_inf("%s: cannot allocate hostname array of %zu bytes, skipping stessor\n",
+		pr_inf_skip("%s: cannot allocate hostname array of %zu bytes, skipping stessor\n",
 			args->name, hostname_len);
 		return EXIT_NO_RESOURCE;
 	}
 	longname = calloc(longname_len, sizeof(*hostname));
 	if (!longname) {
-		pr_inf("%s: cannot allocate longname array of %zu bytes, skipping stessor\n",
+		pr_inf_skip("%s: cannot allocate longname array of %zu bytes, skipping stessor\n",
 			args->name, longname_len);
 		free(hostname);
 		return EXIT_NO_RESOURCE;
@@ -143,7 +144,7 @@ static int stress_set(const stress_args_t *args)
 	ret_hostname = gethostname(hostname, hostname_len - 1);
 	if (ret_hostname == 0) {
 		hostname[hostname_len - 1] = '\0';
-		shim_strlcpy(longname, hostname, longname_len);
+		(void)shim_strlcpy(longname, hostname, longname_len);
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -161,13 +162,13 @@ static int stress_set(const stress_args_t *args)
 
 		/* setsid will fail, ignore return */
 		VOID_RET(pid_t, setsid());
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		/* getgid always succeeds */
 		gid = getgid();
 		VOID_RET(int, setgid(gid));
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		if (*longname) {
@@ -195,7 +196,7 @@ static int stress_set(const stress_args_t *args)
 					VOID_RET(int, setpgid(bad_pid, bad_pid));
 				}
 				VOID_RET(int, setpgid(mypid, pid));
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					break;
 			}
 		}
@@ -217,6 +218,12 @@ static int stress_set(const stress_args_t *args)
 						"set time, expected -EPERM, instead got errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				}
+				/*
+				 *  exercise bogus times west tz, possibly undefined behaviour but
+				 *  on Linux anything +/- 15*60 is -EINVAL
+				 */
+				tz.tz_minuteswest = 36 * 60;
+				VOID_RET(int, settimeofday(&tv, &tz));
 			}
 		}
 #else
@@ -232,7 +239,7 @@ static int stress_set(const stress_args_t *args)
 			pid = getpgrp();
 			if (pid != -1) {
 				VOID_RET(int, setpgrp());
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					break;
 			}
 		}
@@ -243,7 +250,7 @@ static int stress_set(const stress_args_t *args)
 		/* getuid always succeeds */
 		uid = getuid();
 		VOID_RET(int, setuid(uid));
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 #if defined(HAVE_GRP_H)
@@ -252,11 +259,11 @@ static int stress_set(const stress_args_t *args)
 			gid_t groups[GIDS_MAX];
 			int n;
 
-			(void)memset(groups, 0, sizeof(groups));
+			(void)shim_memset(groups, 0, sizeof(groups));
 			ret = STRESS_MINIMUM(ret, (int)SIZEOF_ARRAY(groups));
 			n = getgroups(ret, groups);
 			if (n > 0) {
-				gid_t bad_groups[1] = { (gid_t)-1 };
+				const gid_t bad_groups[1] = { (gid_t)-1 };
 
 				/* Exercise invalid groups */
 				VOID_RET(int, shim_setgroups(INT_MIN, groups));
@@ -461,7 +468,7 @@ static int stress_set(const stress_args_t *args)
 				/* Set name back */
 				VOID_RET(int, shim_setdomainname(name, strlen(name)));
 			}
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 		}
 #else
@@ -471,7 +478,7 @@ static int stress_set(const stress_args_t *args)
 		 *  Invalid setrlimit syscall with invalid
 		 *  resource attribute resulting in EINVAL error
 		 */
-		(void)memset(&rlim, 0, sizeof(rlim));
+		(void)shim_memset(&rlim, 0, sizeof(rlim));
 		if ((getrlimit((shim_rlimit_resource_t)INT_MAX, &rlim) < 0) && (errno == EINVAL)) {
 			(void)setrlimit((shim_rlimit_resource_t)INT_MAX, &rlim);
 		}
@@ -529,8 +536,8 @@ static int stress_set(const stress_args_t *args)
 				test_stime = false;
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
