@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -67,11 +67,11 @@ static void MLOCKED_TEXT stress_shellsort_handler(int signum)
 	}
 }
 
-static int shellsort32(void *base, size_t nmemb,
+static inline int OPTIMIZE3 shellsort32(void *base, size_t nmemb,
 	int (*compar)(const void *, const void *))
 {
 	register size_t gap;
-	uint32_t *array = (uint32_t *)base;
+	register uint32_t *const array = (uint32_t *)base;
 
 	for (gap = nmemb >> 1; gap > 0; gap >>= 1) {
 		register size_t i;
@@ -80,7 +80,7 @@ static int shellsort32(void *base, size_t nmemb,
 			register size_t j;
 			const uint32_t temp = array[i];
 
-			for (j = i; j >= gap && compar(&array[j - gap], &temp) > 0; j -= gap) {
+			for (j = i; (j >= gap) && (compar(&array[j - gap], &temp) > 0); j -= gap) {
 				array[j] = array[j - gap];
 			}
 			array[j] = temp;
@@ -89,11 +89,11 @@ static int shellsort32(void *base, size_t nmemb,
 	return 0;
 }
 
-static int shellsort8(void *base, size_t nmemb,
+static inline int OPTIMIZE3 shellsort8(void *base, size_t nmemb,
 	int (*compar)(const void *, const void *))
 {
 	register size_t gap;
-	uint8_t *array = (uint8_t *)base;
+	register uint8_t *const array = (uint8_t *)base;
 
 	for (gap = nmemb >> 1; gap > 0; gap >>= 1) {
 		register size_t i;
@@ -102,7 +102,7 @@ static int shellsort8(void *base, size_t nmemb,
 			register size_t j;
 			const uint8_t temp = array[i];
 
-			for (j = i; j >= gap && compar(&array[j - gap], &temp) > 0; j -= gap) {
+			for (j = i; (j >= gap) && (compar(&array[j - gap], &temp) > 0); j -= gap) {
 				array[j] = array[j - gap];
 			}
 			array[j] = temp;
@@ -111,7 +111,7 @@ static int shellsort8(void *base, size_t nmemb,
 	return 0;
 }
 
-static int shellsort(void *base, size_t nmemb, size_t size,
+static inline int OPTIMIZE3 shellsort(void *base, size_t nmemb, size_t size,
 	int (*compar)(const void *, const void *))
 {
 	if (size == sizeof(uint32_t))
@@ -125,15 +125,16 @@ static int shellsort(void *base, size_t nmemb, size_t size,
  *  stress_shellsort()
  *	stress shellsort
  */
-static int stress_shellsort(const stress_args_t *args)
+static int OPTIMIZE3 stress_shellsort(stress_args_t *args)
 {
 	uint64_t shellsort_size = DEFAULT_SHELLSORT_SIZE;
 	int32_t *data, *ptr;
-	size_t n, i;
+	size_t n;
 	struct sigaction old_action;
 	int ret;
 	double rate;
 	NOCLOBBER double duration = 0.0, count = 0.0, sorted = 0.0;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	if (!stress_get_setting("shellsort-size", &shellsort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -149,10 +150,6 @@ static int stress_shellsort(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	if (stress_sighandler(args->name, SIGALRM, stress_shellsort_handler, &old_action) < 0) {
-		free(data);
-		return EXIT_FAILURE;
-	}
 
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
@@ -162,6 +159,10 @@ static int stress_shellsort(const stress_args_t *args)
 		(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 		goto tidy;
 	}
+	if (stress_sighandler(args->name, SIGALRM, stress_shellsort_handler, &old_action) < 0) {
+		free(data);
+		return EXIT_FAILURE;
+	}
 
 	stress_sort_data_int32_init(data, n);
 
@@ -175,7 +176,7 @@ static int stress_shellsort(const stress_args_t *args)
 		/* Sort "random" data */
 		stress_sort_compare_reset();
 		t = stress_time_now();
-		if (shellsort(data, n, sizeof(*data), stress_sort_cmp_int32) < 0) {
+		if (shellsort(data, n, sizeof(*data), stress_sort_cmp_fwd_int32) < 0) {
 			pr_fail("%s: shellsort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
@@ -183,7 +184,9 @@ static int stress_shellsort(const stress_args_t *args)
 			count += (double)stress_sort_compare_get();
 			sorted += (double)n;
 
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (UNLIKELY(verify)) {
+				register size_t i;
+
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 					if (*ptr > *(ptr + 1)) {
 						pr_fail("%s: sort error "
@@ -194,7 +197,7 @@ static int stress_shellsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/* Reverse sort */
@@ -208,7 +211,9 @@ static int stress_shellsort(const stress_args_t *args)
 			count += (double)stress_sort_compare_get();
 			sorted += (double)n;
 
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (UNLIKELY(verify)) {
+				register size_t i;
+
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 					if (*ptr < *(ptr + 1)) {
 						pr_fail("%s: reverse sort "
@@ -219,19 +224,12 @@ static int stress_shellsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
-		/* And re-order by byte compare */
+
+		/* And re-order */
+		stress_sort_data_int32_mangle(data, n);
 		stress_sort_compare_reset();
-		t = stress_time_now();
-		if (shellsort(data, n * 4, sizeof(uint8_t), stress_sort_cmp_int8) < 0) {
-			pr_fail("%s: shellsort failed: %d (%s)\n",
-				args->name, errno, strerror(errno));
-		} else {
-			duration += stress_time_now() - t;
-			count += (double)stress_sort_compare_get();
-			sorted += (double)n;
-		}
 
 		/* Reverse sort this again */
 		stress_sort_compare_reset();
@@ -244,7 +242,9 @@ static int stress_shellsort(const stress_args_t *args)
 			count += (double)stress_sort_compare_get();
 			sorted += (double)n;
 
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (UNLIKELY(verify)) {
+				register size_t i;
+
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 					if (*ptr < *(ptr + 1)) {
 						pr_fail("%s: reverse sort "
@@ -255,19 +255,21 @@ static int stress_shellsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	rate = (duration > 0.0) ? count / duration : 0.0;
-	stress_metrics_set(args, 0, "shellsort comparisons per sec", rate);
-	stress_metrics_set(args, 1, "shellsort comparisons per item", count / sorted);
+	stress_metrics_set(args, 0, "shellsort comparisons per sec",
+		rate, STRESS_HARMONIC_MEAN);
+	stress_metrics_set(args, 1, "shellsort comparisons per item",
+		count / sorted, STRESS_HARMONIC_MEAN);
 
 	free(data);
 
