@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,11 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
 
 #define MIN_SHM_POSIX_BYTES	(1 * MB)
 #define MAX_SHM_POSIX_BYTES	(1 * GB)
@@ -37,11 +42,17 @@ typedef struct {
 static const stress_help_t help[] = {
 	{ NULL,	"shm N",	"start N workers that exercise POSIX shared memory" },
 	{ NULL,	"shm-bytes N",	"allocate/free N bytes of POSIX shared memory" },
+	{ NULL,	"shm-mlock",	"attempt to mlock pages into memory" },
+	{ NULL,	"shm-objs N",	"allocate N POSIX shared memory objects per iteration" },
 	{ NULL,	"shm-ops N",	"stop after N POSIX shared memory bogo operations" },
-	{ NULL,	"shm-segs N",	"allocate N POSIX shared memory segments per iteration" },
 	{ NULL,	NULL,		NULL }
 };
 
+static int stress_set_shm_mlock(const char *opt)
+{
+	return stress_set_setting_true("shm-mlock", opt);
+}
+
 static int stress_set_shm_posix_bytes(const char *opt)
 {
 	size_t shm_posix_bytes;
@@ -64,6 +75,7 @@ static int stress_set_shm_posix_objects(const char *opt)
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_shm_bytes,	stress_set_shm_posix_bytes },
+	{ OPT_shm_mlock,	stress_set_shm_mlock },
 	{ OPT_shm_objects,	stress_set_shm_posix_objects },
 	{ 0,			NULL }
 };
@@ -79,10 +91,10 @@ static int stress_shm_posix_check(
 	const size_t sz,
 	const size_t page_size)
 {
-	uint8_t *ptr, *end = buf + sz;
-	uint8_t val;
+	uint8_t *ptr, val;
+	const uint8_t *end = buf + sz;
 
-	(void)memset(buf, 0xa5, sz);
+	(void)shim_memset(buf, 0xa5, sz);
 
 	for (val = 0, ptr = buf; ptr < end; ptr += page_size, val++) {
 		*ptr = val;
@@ -104,14 +116,15 @@ static int stress_shm_posix_check(
  *	be reaped cleanly if this process gets prematurely killed.
  */
 static int stress_shm_posix_child(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
-	size_t sz,
-	size_t shm_posix_objects)
+	const size_t sz,
+	const size_t shm_posix_objects,
+	const bool shm_mlock)
 {
 	void **addrs;
 	char *shm_names;
-	stress_shm_msg_t msg;
+	stress_shm_msg_t msg ALIGN64;
 	int i;
 	int rc = EXIT_SUCCESS;
 	bool ok = true;
@@ -135,10 +148,10 @@ static int stress_shm_posix_child(
 	}
 
 	/* Make sure this is killable by OOM killer */
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
-	(void)memset(&msg, 0, sizeof(msg));
-	(void)memset(&sa, 0, sizeof(sa));
+	(void)shim_memset(&msg, 0, sizeof(msg));
+	(void)shim_memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = SIG_IGN;
 #if defined(SA_NOCLDWAIT)
 	sa.sa_flags = SA_NOCLDWAIT;
@@ -161,7 +174,7 @@ static int stress_shm_posix_child(
 
 			shm_name[0] = '\0';
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto reap;
 
 			(void)snprintf(shm_name, SHM_NAME_LEN,
@@ -170,7 +183,7 @@ static int stress_shm_posix_child(
 
 			shm_fd = shm_open(shm_name, O_CREAT | O_RDWR | O_TRUNC,
 				S_IRUSR | S_IWUSR);
-			if (shm_fd < 0) {
+			if (UNLIKELY(shm_fd < 0)) {
 				ok = false;
 				pr_fail("%s: shm_open %s failed, errno=%d (%s)\n",
 					args->name, shm_name, errno, strerror(errno));
@@ -179,11 +192,11 @@ static int stress_shm_posix_child(
 			}
 
 			/* Inform parent of the new shm name */
-			(void)memset(&msg, 0, sizeof(msg));
+			(void)shim_memset(&msg, 0, sizeof(msg));
 			msg.index = i;
 			shm_name[SHM_NAME_LEN - 1] = '\0';
 			(void)shim_strlcpy(msg.shm_name, shm_name, SHM_NAME_LEN);
-			if (write(fd, &msg, sizeof(msg)) < 0) {
+			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
 				pr_err("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -193,7 +206,7 @@ static int stress_shm_posix_child(
 
 			addr = mmap(NULL, sz, PROT_READ | PROT_WRITE,
 				MAP_PRIVATE | MAP_ANONYMOUS, shm_fd, 0);
-			if (addr == MAP_FAILED) {
+			if (UNLIKELY(addr == MAP_FAILED)) {
 				ok = false;
 				pr_fail("%s: mmap failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
@@ -202,26 +215,21 @@ static int stress_shm_posix_child(
 				goto reap;
 			}
 			addrs[i] = addr;
+			(void)stress_madvise_mergeable(addr, sz);
+			if (shm_mlock)
+				(void)shim_mlock(addr, sz);
 
-			if (!keep_stressing_flag()) {
+			if (UNLIKELY(!stress_continue_flag())) {
 				(void)close(shm_fd);
 				goto reap;
 			}
 			(void)stress_mincore_touch_pages(addr, sz);
 
-			if (!keep_stressing_flag()) {
+			if (UNLIKELY(!stress_continue_flag())) {
 				(void)close(shm_fd);
 				goto reap;
 			}
 
-#if defined(_POSIX_MEMLOCK_RANGE) &&	\
-    defined(HAVE_MLOCK)
-			/*
-			 *  Exercise mlock on 1st page of shm
-			 */
-			VOID_RET(int, shim_mlock(addr, 4096));
-#endif
-
 			/*
 			 *  Exercise shm duplication and reaping
 			 *  on a fork and exit
@@ -240,18 +248,35 @@ static int stress_shm_posix_child(
 			(void)shim_fsync(shm_fd);
 			VOID_RET(off_t, lseek(shm_fd, (off_t)0, SEEK_SET));
 
+#if defined(FALLOC_FL_UNSHARE_RANGE)
+			/* Exercise range unsharing */
+			(void)shim_fallocate(shm_fd, FALLOC_FL_UNSHARE_RANGE, 0, (off_t)page_size);
+#endif
+#if defined(FALLOC_FL_COLLAPSE_RANGE)
+			/* Exercise range collapsing */
+			(void)shim_fallocate(shm_fd, FALLOC_FL_COLLAPSE_RANGE, 0, (off_t)page_size);
+#endif
+#if defined(FALLOC_FL_PUNCH_HOLE) &&	\
+    defined(FALLOC_FL_KEEP_SIZE)
+			/* Exercise hole punching */
+			(void)shim_fallocate(shm_fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE, 0, (off_t)sz);
+#endif
+#if defined(FALLOC_FL_ZERO_RANGE)
+			/* Exercise unsupported zero'ing */
+			(void)shim_fallocate(shm_fd, FALLOC_FL_ZERO_RANGE, 0, (off_t)sz);
+#endif
 			/* Shrink the mapping */
 			(void)shim_fallocate(shm_fd, 0, 0, (off_t)sz);
 
 			/* Now truncated it back */
 			ret = ftruncate(shm_fd, (off_t)sz);
-			if (ret < 0)
+			if (UNLIKELY(ret < 0))
 				pr_fail("%s: ftruncate of shared memory failed\n", args->name);
 			(void)shim_fsync(shm_fd);
 
 			/* fstat shared memory */
-			ret = fstat(shm_fd, &statbuf);
-			if (ret < 0) {
+			ret = shim_fstat(shm_fd, &statbuf);
+			if (UNLIKELY(ret < 0)) {
 				pr_fail("%s: fstat failed on shared memory\n", args->name);
 			} else {
 				if (statbuf.st_size != (off_t)sz) {
@@ -263,31 +288,31 @@ static int stress_shm_posix_child(
 
 			/* Make it read only */
 			ret = fchmod(shm_fd, S_IRUSR);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				pr_fail("%s: failed to fchmod to S_IRUSR on shared memory\n", args->name);
 			}
 			ret = fchown(shm_fd, uid, gid);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				pr_fail("%s: failed to fchown on shared memory\n", args->name);
 			}
 
 			(void)close(shm_fd);
-			if (newpid > 0) {
+			if (LIKELY(newpid > 0)) {
 				int status;
 
 				(void)shim_waitpid(newpid, &status, 0);
 			}
 
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
-			if (stress_shm_posix_check(addr, sz, page_size) < 0) {
+			if (UNLIKELY(stress_shm_posix_check(addr, sz, page_size) < 0)) {
 				ok = false;
 				pr_fail("%s: memory check failed\n", args->name);
 				rc = EXIT_FAILURE;
 				goto reap;
 			}
 			id++;
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 reap:
 		for (i = 0; ok && (i < (ssize_t)shm_posix_objects); i++) {
@@ -311,7 +336,7 @@ reap:
 			msg.index = i;
 			msg.shm_name[SHM_NAME_LEN - 1] = '\0';
 			(void)shim_strlcpy(msg.shm_name, shm_name, SHM_NAME_LEN - 1);
-			if (write(fd, &msg, sizeof(msg)) < 0) {
+			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
 				pr_dbg("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
 				ok = false;
@@ -319,7 +344,7 @@ reap:
 			addrs[i] = NULL;
 			*shm_name = '\0';
 		}
-	} while (ok && keep_stressing(args));
+	} while (ok && stress_continue(args));
 
 	/* Inform parent of end of run */
 	msg.index = -1;
@@ -339,7 +364,7 @@ reap:
  *  stress_shm()
  *	stress POSIX shared memory
  */
-static int stress_shm(const stress_args_t *args)
+static int stress_shm(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	size_t orig_sz, sz;
@@ -348,10 +373,13 @@ static int stress_shm(const stress_args_t *args)
 	ssize_t i;
 	pid_t pid;
 	bool retry = true;
+	bool shm_mlock = false;
 	uint32_t restarts = 0;
 	size_t shm_posix_bytes = DEFAULT_SHM_POSIX_BYTES;
 	size_t shm_posix_objects = DEFAULT_SHM_POSIX_OBJECTS;
 
+	(void)stress_get_setting("shm-mlock", &shm_mlock);
+
 	if (!stress_get_setting("shm-bytes", &shm_posix_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			shm_posix_bytes = MAX_SHM_POSIX_BYTES;
@@ -385,7 +413,7 @@ static int stress_shm(const stress_args_t *args)
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	while (keep_stressing_flag() && retry) {
+	while (stress_continue_flag() && retry) {
 		if (pipe(pipefds) < 0) {
 			pr_fail("%s: pipe failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -394,9 +422,9 @@ static int stress_shm(const stress_args_t *args)
 again:
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args)) {
+			if (!stress_continue(args)) {
 				rc = EXIT_SUCCESS;
 				goto finish;
 			}
@@ -420,7 +448,7 @@ again:
 			}
 			(void)close(pipefds[1]);
 
-			while (keep_stressing_flag()) {
+			while (stress_continue_flag()) {
 				ssize_t n;
 				stress_shm_msg_t 	msg;
 				char *shm_name;
@@ -431,7 +459,7 @@ again:
 				 *  on child death, or child tells us
 				 *  about its demise.
 				 */
-				(void)memset(&msg, 0, sizeof(msg));
+				(void)shim_memset(&msg, 0, sizeof(msg));
 				n = read(pipefds[0], &msg, sizeof(msg));
 				if (n <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
@@ -454,11 +482,9 @@ again:
 				msg.shm_name[SHM_NAME_LEN - 1] = '\0';
 				(void)shim_strlcpy(shm_name, msg.shm_name, SHM_NAME_LEN);
 			}
-			(void)stress_killpid(pid);
-			(void)shim_waitpid(pid, &status, 0);
+			(void)stress_kill_pid_wait(pid, &status);
 			if (WIFSIGNALED(status)) {
-				if ((WTERMSIG(status) == SIGKILL) ||
-				    (WTERMSIG(status) == SIGKILL)) {
+				if (WTERMSIG(status) == SIGKILL) {
 					stress_log_system_mem_info();
 					pr_dbg("%s: assuming killed by OOM killer, "
 						"restarting again (instance %d)\n",
@@ -486,7 +512,7 @@ again:
 			stress_parent_died_alarm();
 
 			(void)close(pipefds[0]);
-			rc = stress_shm_posix_child(args, pipefds[1], sz, shm_posix_objects);
+			rc = stress_shm_posix_child(args, pipefds[1], sz, shm_posix_objects, shm_mlock);
 			(void)close(pipefds[1]);
 			_exit(rc);
 		}
@@ -517,6 +543,7 @@ stressor_info_t stress_shm_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without librt"
 };
