@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,9 +29,12 @@
  *
  */
 #include "stress-ng.h"
-#include "core-cache.h"
 #include "core-cpu.h"
+#include "core-cpu-cache.h"
 #include "core-nt-store.h"
+#include "core-numa.h"
+#include "core-pragma.h"
+#include "core-target-clones.h"
 
 #define MIN_STREAM_L3_SIZE	(4 * KB)
 #define MAX_STREAM_L3_SIZE	(MAX_MEM_LIMIT)
@@ -50,9 +53,10 @@ typedef struct {
 
 static const stress_help_t help[] = {
 	{ NULL,	"stream N",		"start N workers exercising memory bandwidth" },
-	{ NULL,	"stream-index",		"specify number of indices into the data (0..3)" },
+	{ NULL,	"stream-index N",	"specify number of indices into the data (0..3)" },
 	{ NULL,	"stream-l3-size N",	"specify the L3 cache size of the CPU" },
 	{ NULL,	"stream-madvise M",	"specify mmap'd stream buffer madvise advice" },
+	{ NULL,	"stream-mlock",		"attempt to mlock pages into memory" },
 	{ NULL,	"stream-ops N",		"stop after N bogo stream operations" },
 	{ NULL,	NULL,                   NULL }
 };
@@ -65,6 +69,9 @@ static const stress_stream_madvise_info_t stream_madvise_info[] = {
 #if defined(MADV_NOHUGEPAGE)
 	{ "nohugepage",	MADV_NOHUGEPAGE },
 #endif
+#if defined(MADV_COLLAPSE)
+	{ "collapse",	MADV_NORMAL },
+#endif
 #if defined(MADV_NORMAL)
 	{ "normal",	MADV_NORMAL },
 #endif
@@ -75,6 +82,11 @@ static const stress_stream_madvise_info_t stream_madvise_info[] = {
 	{ NULL,         0 },
 };
 
+static int stress_set_stream_mlock(const char *opt)
+{
+	return stress_set_setting_true("stream-mlock", opt);
+}
+
 static int stress_set_stream_L3_size(const char *opt)
 {
 	uint64_t stream_L3_size;
@@ -112,17 +124,34 @@ static int stress_set_stream_index(const char *opt)
 	return stress_set_setting("stream-index", TYPE_ID_UINT32, &stream_index);
 }
 
-static inline void OPTIMIZE3 stress_stream_copy_index0(
-	double *RESTRICT c,
-	const double *RESTRICT a,
+/*
+ *  stress_stream_checksum_to_hexstr()
+ *	turn a double into a hexadecimal string making zero assumptions about
+ *	the size of a double since this maybe arch specific.
+ */
+static void stress_stream_checksum_to_hexstr(char *str, const size_t len, const double checksum)
+{
+	const unsigned char *ptr = (const unsigned char *)&checksum;
+	size_t i, j;
+
+	for (i = 0, j = 0; (i < sizeof(checksum)) && (j < len); i++, j += 2) {
+		(void)snprintf(str + j, 3, "%2.2x", ptr[i]);
+	}
+	str[i] = '\0';
+}
+
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_copy_index0(
+	double *const RESTRICT c,
+	const double *const RESTRICT a,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		STORE(cv[i + 0], a[i + 0]);
 		STORE(cv[i + 1], a[i + 1]);
@@ -136,16 +165,17 @@ static inline void OPTIMIZE3 stress_stream_copy_index0(
 }
 
 #if defined(HAVE_NT_STORE_DOUBLE)
-static inline void OPTIMIZE3 stress_stream_copy_index0_nt(
-	double *RESTRICT c,
-	const double *RESTRICT a,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_copy_index0_nt(
+	double *const RESTRICT c,
+	const double *const RESTRICT a,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		NT_STORE(c[i + 0], a[i + 0]);
 		NT_STORE(c[i + 1], a[i + 1]);
@@ -160,17 +190,18 @@ static inline void OPTIMIZE3 stress_stream_copy_index0_nt(
 #endif
 
 static inline void OPTIMIZE3 stress_stream_copy_index1(
-	double *RESTRICT c,
-	const double *RESTRICT a,
-	const size_t *RESTRICT idx1,
+	double *const RESTRICT c,
+	const double *const RESTRICT a,
+	const size_t *const RESTRICT idx1,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		const size_t idx = idx1[i];
 
@@ -183,10 +214,10 @@ static inline void OPTIMIZE3 stress_stream_copy_index1(
 }
 
 static inline void OPTIMIZE3 stress_stream_copy_index2(
-	double *RESTRICT c,
-	const double *RESTRICT a,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
+	double *const RESTRICT c,
+	const double *const RESTRICT a,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
 	const uint64_t n,
 	double *rd_bytes,
 	double *wr_bytes,
@@ -195,6 +226,7 @@ static inline void OPTIMIZE3 stress_stream_copy_index2(
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(cv[idx1[i]], a[idx2[i]]);
 
@@ -204,19 +236,20 @@ static inline void OPTIMIZE3 stress_stream_copy_index2(
 }
 
 static inline void OPTIMIZE3 stress_stream_copy_index3(
-	double *RESTRICT c,
-	const double *RESTRICT a,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
-	const size_t *RESTRICT idx3,
+	double *const RESTRICT c,
+	const double *const RESTRICT a,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
+	const size_t *const RESTRICT idx3,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(cv[idx3[idx1[i]]], a[idx2[i]]);
 
@@ -225,18 +258,19 @@ static inline void OPTIMIZE3 stress_stream_copy_index3(
 	*fp_ops += 0.0;
 }
 
-static inline void OPTIMIZE3 stress_stream_scale_index0(
-	double *RESTRICT b,
-	const double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_scale_index0(
+	double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT bv = b;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		STORE(bv[i + 0], q * c[i + 0]);
 		STORE(bv[i + 1], q * c[i + 1]);
@@ -250,17 +284,18 @@ static inline void OPTIMIZE3 stress_stream_scale_index0(
 }
 
 #if defined(HAVE_NT_STORE_DOUBLE)
-static inline void OPTIMIZE3 stress_stream_scale_index0_nt(
-	double *RESTRICT b,
-	const double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_scale_index0_nt(
+	double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		NT_STORE(b[i + 0], q * c[i + 0]);
 		NT_STORE(b[i + 1], q * c[i + 1]);
@@ -274,19 +309,20 @@ static inline void OPTIMIZE3 stress_stream_scale_index0_nt(
 }
 #endif
 
-static inline void OPTIMIZE3 stress_stream_scale_index1(
-	double *RESTRICT b,
-	const double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_scale_index1(
+	double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
+	const size_t *const RESTRICT idx1,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT bv = b;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		const size_t idx = idx1[i];
 
@@ -299,19 +335,20 @@ static inline void OPTIMIZE3 stress_stream_scale_index1(
 }
 
 static inline void OPTIMIZE3 stress_stream_scale_index2(
-	double *RESTRICT b,
-	const double *RESTRICT c,
+	double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT bv = b;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(bv[idx1[i]], q * c[idx2[i]]);
 
@@ -321,20 +358,21 @@ static inline void OPTIMIZE3 stress_stream_scale_index2(
 }
 
 static inline void OPTIMIZE3 stress_stream_scale_index3(
-	double *RESTRICT b,
-	const double *RESTRICT c,
+	double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
-	const size_t *RESTRICT idx3,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
+	const size_t *const RESTRICT idx3,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT bv = b;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(bv[idx3[idx1[i]]], q * c[idx2[i]]);
 
@@ -343,18 +381,19 @@ static inline void OPTIMIZE3 stress_stream_scale_index3(
 	*fp_ops += (double)n;
 }
 
-static inline void OPTIMIZE3 stress_stream_add_index0(
-	const double *RESTRICT a,
-	const double *RESTRICT b,
-	double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_add_index0(
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	double *const RESTRICT c,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		STORE(cv[i + 0], a[i + 0] + b[i + 0]);
 		STORE(cv[i + 1], a[i + 1] + b[i + 1]);
@@ -368,17 +407,18 @@ static inline void OPTIMIZE3 stress_stream_add_index0(
 }
 
 #if defined(HAVE_NT_STORE_DOUBLE)
-static inline void OPTIMIZE3 stress_stream_add_index0_nt(
-	const double *RESTRICT a,
-	const double *RESTRICT b,
-	double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_add_index0_nt(
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	double *const RESTRICT c,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		NT_STORE(c[i + 0], a[i + 0] + b[i + 0]);
 		NT_STORE(c[i + 1], a[i + 1] + b[i + 1]);
@@ -393,18 +433,19 @@ static inline void OPTIMIZE3 stress_stream_add_index0_nt(
 #endif
 
 static inline void OPTIMIZE3 stress_stream_add_index1(
-	const double *RESTRICT a,
-	const double *RESTRICT b,
-	double *RESTRICT c,
-	const size_t *RESTRICT idx1,
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	double *const RESTRICT c,
+	const size_t *const RESTRICT idx1,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		const size_t idx = idx1[i];
 
@@ -417,19 +458,20 @@ static inline void OPTIMIZE3 stress_stream_add_index1(
 }
 
 static inline void OPTIMIZE3 stress_stream_add_index2(
-	const double *RESTRICT a,
-	const double *RESTRICT b,
-	double *RESTRICT c,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	double *const RESTRICT c,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		const size_t idx = idx1[i];
 
@@ -442,20 +484,21 @@ static inline void OPTIMIZE3 stress_stream_add_index2(
 }
 
 static inline void OPTIMIZE3 stress_stream_add_index3(
-	const double *RESTRICT a,
-	const double *RESTRICT b,
-	double *RESTRICT c,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
-	const size_t *RESTRICT idx3,
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	double *const RESTRICT c,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
+	const size_t *const RESTRICT idx3,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT cv = c;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(cv[idx1[i]], a[idx2[i]] + b[idx3[i]]);
 
@@ -464,19 +507,20 @@ static inline void OPTIMIZE3 stress_stream_add_index3(
 	*fp_ops += (double)n;
 }
 
-static inline void OPTIMIZE3 stress_stream_triad_index0(
-	double *RESTRICT a,
-	const double *RESTRICT b,
-	const double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_triad_index0(
+	double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT av = a;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		STORE(av[i + 0], b[i + 0] + (c[i + 0] * q));
 		STORE(av[i + 1], b[i + 1] + (c[i + 1] * q));
@@ -490,18 +534,19 @@ static inline void OPTIMIZE3 stress_stream_triad_index0(
 }
 
 #if defined(HAVE_NT_STORE_DOUBLE)
-static inline void OPTIMIZE3 stress_stream_triad_index0_nt(
-	double *RESTRICT a,
-	const double *RESTRICT b,
-	const double *RESTRICT c,
+static inline void OPTIMIZE3 TARGET_CLONES stress_stream_triad_index0_nt(
+	double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i += 4) {
 		NT_STORE(a[i + 0], b[i + 0] + (c[i + 0] * q));
 		NT_STORE(a[i + 1], b[i + 1] + (c[i + 1] * q));
@@ -516,19 +561,20 @@ static inline void OPTIMIZE3 stress_stream_triad_index0_nt(
 #endif
 
 static inline void OPTIMIZE3 stress_stream_triad_index1(
-	double *RESTRICT a,
-	const double *RESTRICT b,
-	const double *RESTRICT c,
+	double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
+	const size_t *const RESTRICT idx1,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT av = a;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		size_t idx = idx1[i];
 
@@ -540,20 +586,21 @@ static inline void OPTIMIZE3 stress_stream_triad_index1(
 }
 
 static inline void OPTIMIZE3 stress_stream_triad_index2(
-	double *RESTRICT a,
-	const double *RESTRICT b,
-	const double *RESTRICT c,
+	double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT av = a;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++) {
 		const size_t idx = idx1[i];
 
@@ -566,21 +613,22 @@ static inline void OPTIMIZE3 stress_stream_triad_index2(
 }
 
 static inline void OPTIMIZE3 stress_stream_triad_index3(
-	double *RESTRICT a,
-	const double *RESTRICT b,
-	const double *RESTRICT c,
+	double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
 	const double q,
-	const size_t *RESTRICT idx1,
-	const size_t *RESTRICT idx2,
-	const size_t *RESTRICT idx3,
+	const size_t *const RESTRICT idx1,
+	const size_t *const RESTRICT idx2,
+	const size_t *const RESTRICT idx3,
 	const uint64_t n,
-	double *rd_bytes,
-	double *wr_bytes,
-	double *fp_ops)
+	double *const RESTRICT rd_bytes,
+	double *const RESTRICT wr_bytes,
+	double *const RESTRICT fp_ops)
 {
 	register uint64_t i;
 	register double volatile *RESTRICT av = a;
 
+PRAGMA_UNROLL_N(8)
 	for (i = 0; i < n; i++)
 		STORE(av[idx1[i]], b[idx2[i]] + (c[idx3[i]] * q));
 
@@ -589,29 +637,71 @@ static inline void OPTIMIZE3 stress_stream_triad_index3(
 	*fp_ops += (double)n * 2.0;
 }
 
-static void stress_stream_init_data(
-	double *RESTRICT data,
+static inline TARGET_CLONES OPTIMIZE3 void stress_stream_init_data(
+	double *const RESTRICT a,
+	double *const RESTRICT b,
+	double *const RESTRICT c,
 	const uint64_t n)
 {
-	uint64_t i;
-	const double divisor = 1.0 / (double)(4294967296ULL);
-	register double volatile *RESTRICT datav = data;
+	register const double divisor = 1.0 / (double)(4294967296ULL);
+	register const double delta = (double)stress_mwc32() * divisor;
 
-	for (i = 0; i < n; i++) {
-		register const uint32_t r1 = stress_mwc32();
+	register const uint32_t r = stress_mwc32();
+	register double v = (double)r * divisor;
+	register double *ptr, *ptr_end;
 
-		STORE(datav[i], (double)r1 * divisor);
+PRAGMA_UNROLL_N(4)
+	for (ptr = a, ptr_end = a + n; ptr < ptr_end; ptr += 4) {
+		STORE(ptr[0], v);
+		STORE(ptr[1], v);
+		STORE(ptr[2], v);
+		STORE(ptr[3], v);
+		v += delta;
+	}
+
+PRAGMA_UNROLL_N(4)
+	for (ptr = b, ptr_end = b + n; ptr < ptr_end; ptr += 4) {
+		STORE(ptr[0], v);
+		STORE(ptr[1], v);
+		STORE(ptr[2], v);
+		STORE(ptr[3], v);
+		v += delta;
+	}
+
+PRAGMA_UNROLL_N(4)
+	for (ptr = c, ptr_end = c + n; ptr < ptr_end; ptr += 4) {
+		STORE(ptr[0], v);
+		STORE(ptr[1], v);
+		STORE(ptr[2], v);
+		STORE(ptr[3], v);
+		v += delta;
 	}
 }
 
-static inline void *stress_stream_mmap(const stress_args_t *args, uint64_t sz)
+static double TARGET_CLONES OPTIMIZE3 stress_stream_checksum_data(
+	const double *const RESTRICT a,
+	const double *const RESTRICT b,
+	const double *const RESTRICT c,
+	const uint64_t n)
+{
+	double checksum = 0.0;
+	register uint64_t i;
+
+PRAGMA_UNROLL_N(8)
+	for (i = 0; i < n; i++) {
+		checksum += a[i] + b[i] + c[i];
+	}
+	return checksum;
+}
+
+static inline void *stress_stream_mmap(
+	stress_args_t *args,
+	const uint64_t sz,
+	const bool stream_mlock)
 {
 	void *ptr;
 
-	ptr = mmap(NULL, (size_t)sz, PROT_READ | PROT_WRITE,
-#if defined(MAP_POPULATE)
-		MAP_POPULATE |
-#endif
+	ptr = stress_mmap_populate(NULL, (size_t)sz, PROT_READ | PROT_WRITE,
 #if defined(HAVE_MADVISE)
 		MAP_PRIVATE |
 #else
@@ -624,12 +714,14 @@ static inline void *stress_stream_mmap(const stress_args_t *args, uint64_t sz)
 			args->name, sz);
 		ptr = MAP_FAILED;
 	} else {
+		if (stream_mlock)
+			(void)shim_mlock(ptr, (size_t)sz);
 #if defined(HAVE_MADVISE)
 		int advice = MADV_NORMAL;
 
 		(void)stress_get_setting("stream-madvise", &advice);
 
-		VOID_RET(int, madvise(ptr, sz, advice));
+		VOID_RET(int, madvise(ptr, (size_t)sz, advice));
 #else
 		UNEXPECTED
 #endif
@@ -637,40 +729,41 @@ static inline void *stress_stream_mmap(const stress_args_t *args, uint64_t sz)
 	return ptr;
 }
 
-static inline uint64_t get_stream_L3_size(const stress_args_t *args)
+static inline uint64_t get_stream_L3_size(stress_args_t *args)
 {
-	uint64_t cache_size = MEM_CACHE_SIZE;
+	uint64_t cache_size = 2 * MB;
 #if defined(__linux__)
-	stress_cpus_t *cpu_caches;
+	stress_cpu_cache_cpus_t *cpu_caches;
 	stress_cpu_cache_t *cache = NULL;
 	uint16_t max_cache_level;
+	const int numa_nodes = stress_numa_nodes();
 
-	cpu_caches = stress_get_all_cpu_cache_details();
+	cpu_caches = stress_cpu_cache_get_all_details();
 	if (!cpu_caches) {
 		if (!args->instance)
 			pr_inf("%s: using built-in defaults as unable to "
 				"determine cache details\n", args->name);
-		return cache_size;
+		goto report_size;
 	}
-	max_cache_level = stress_get_max_cache_level(cpu_caches);
+	max_cache_level = stress_cpu_cache_get_max_level(cpu_caches);
 	if ((max_cache_level > 0) && (max_cache_level < 3) && (!args->instance))
 		pr_inf("%s: no L3 cache, using L%" PRIu16 " size instead\n",
 			args->name, max_cache_level);
 
-	cache = stress_get_cpu_cache(cpu_caches, max_cache_level);
+	cache = stress_cpu_cache_get(cpu_caches, max_cache_level);
 	if (!cache) {
 		if (!args->instance)
 			pr_inf("%s: using built-in defaults as no suitable "
 				"cache found\n", args->name);
 		stress_free_cpu_caches(cpu_caches);
-		return cache_size;
+		goto report_size;
 	}
 	if (!cache->size) {
 		if (!args->instance)
 			pr_inf("%s: using built-in defaults as unable to "
 				"determine cache size\n", args->name);
 		stress_free_cpu_caches(cpu_caches);
-		return cache_size;
+		goto report_size;
 	}
 	cache_size = cache->size;
 
@@ -680,6 +773,14 @@ static inline uint64_t get_stream_L3_size(const stress_args_t *args)
 		pr_inf("%s: using built-in defaults as unable to "
 			"determine cache details\n", args->name);
 #endif
+
+#if defined(__linux__)
+report_size:
+	cache_size *= numa_nodes;
+	if ((args->instance == 0) && (numa_nodes > 1))
+		pr_inf("%s: scaling L3 cache size by number of numa nodes %d to %" PRIu64 "K\n",
+			args->name, numa_nodes, cache_size / 1024);
+#endif
 	return cache_size;
 }
 
@@ -705,21 +806,29 @@ static void stress_stream_init_index(
  *  stress_stream()
  *	stress cache/memory/CPU with stream stressors
  */
-static int stress_stream(const stress_args_t *args)
+static int stress_stream(stress_args_t *args)
 {
 	int rc = EXIT_FAILURE;
-	double *a, *b, *c;
-	size_t *idx1 = NULL, *idx2 = NULL, *idx3 = NULL;
+	double *a = MAP_FAILED, *b = MAP_FAILED, *c = MAP_FAILED;
+	size_t *idx1 = MAP_FAILED, *idx2 = MAP_FAILED, *idx3 = MAP_FAILED;
 	const double q = 3.0;
+	double old_checksum = -1.0;
 	double fp_ops = 0.0, t1, t2, dt;
-	uint32_t stream_index = 0;
+	uint32_t w, z, stream_index = 0;
 	uint64_t L3, sz, n, sz_idx;
 	uint64_t stream_L3_size = DEFAULT_STREAM_L3_SIZE;
+	uint32_t init_counter, init_counter_max;
 	bool guess = false;
+	bool stream_mlock = false;
 #if defined(HAVE_NT_STORE_DOUBLE)
 	const bool has_sse2 = stress_cpu_x86_has_sse2();
 #endif
 	double rd_bytes = 0.0, wr_bytes = 0.0;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+
+	stress_catch_sigill();
+
+	(void)stress_get_setting("stream-mlock", &stream_mlock);
 
 	if (stress_get_setting("stream-L3-size", &stream_L3_size))
 		L3 = stream_L3_size;
@@ -744,7 +853,7 @@ static int stress_stream(const stress_args_t *args)
 				"defaulting to %" PRIu64 "K\n",
 				args->name, L3 / 1024);
 		} else {
-			pr_inf("%s: Using CPU cache size of %" PRIu64 "K\n",
+			pr_inf("%s: Using cache size of %" PRIu64 "K\n",
 				args->name, L3 / 1024);
 		}
 	}
@@ -760,133 +869,167 @@ static int stress_stream(const stress_args_t *args)
 	 */
 	sz = (L3 * 4);
 	n = sz / sizeof(*a);
-	sz &= ~(uint64_t)63;
+	/*
+	 *  n must be a multiple of the max unroll size (8)
+	 */
+	n = (n + 7) & ~(uint64_t)7;
+	sz = n * sizeof(*a);
 
-	a = stress_stream_mmap(args, sz);
+	a = stress_stream_mmap(args, sz, stream_mlock);
 	if (a == MAP_FAILED)
-		goto err_a;
-	b = stress_stream_mmap(args, sz);
+		goto err_unmap;
+	b = stress_stream_mmap(args, sz, stream_mlock);
 	if (b == MAP_FAILED)
-		goto err_b;
-	c = stress_stream_mmap(args, sz);
+		goto err_unmap;
+	c = stress_stream_mmap(args, sz, stream_mlock);
 	if (c == MAP_FAILED)
-		goto err_c;
+		goto err_unmap;
 
 	sz_idx = n * sizeof(size_t);
 	switch (stream_index) {
 	case 3:
-		idx3 = stress_stream_mmap(args, sz_idx);
+		idx3 = stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx3 == MAP_FAILED)
-			goto err_idx3;
+			goto err_unmap;
 		stress_stream_init_index(idx3, n);
-		CASE_FALLTHROUGH;
+		goto case_stream_index_2;
 	case 2:
-		idx2 = stress_stream_mmap(args, sz_idx);
+case_stream_index_2:
+		idx2 = stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx2 == MAP_FAILED)
-			goto err_idx2;
+			goto err_unmap;
 		stress_stream_init_index(idx2, n);
-		CASE_FALLTHROUGH;
+		goto case_stream_index_1;
 	case 1:
-		idx1 = stress_stream_mmap(args, sz_idx);
+case_stream_index_1:
+		idx1 = stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx1 == MAP_FAILED)
-			goto err_idx1;
+			goto err_unmap;
 		stress_stream_init_index(idx1, n);
-		CASE_FALLTHROUGH;
+		break;
 	case 0:
 	default:
 		break;
 	}
 
-	stress_stream_init_data(a, n);
-	stress_stream_init_data(b, n);
-	stress_stream_init_data(c, n);
+	stress_mwc_get_seed(&w, &z);
+
+	init_counter = 0;
+	init_counter_max = verify ? 1 : 64;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	t1 = stress_time_now();
+	dt = 0.0;
 	do {
+		if (init_counter == 0) {
+			stress_mwc_set_seed(w, z);
+			stress_stream_init_data(a, b, c, n);
+		}
+		init_counter++;
+		if (init_counter >= init_counter_max)
+			init_counter = 0;
+
 		switch (stream_index) {
 		case 3:
+			t1 = stress_time_now();
 			stress_stream_copy_index3(c, a, idx1, idx2, idx3, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_scale_index3(b, c, q, idx1, idx2, idx3, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_add_index3(c, b, a, idx1, idx2, idx3, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_triad_index3(a, b, c, q, idx1, idx2, idx3, n, &rd_bytes, &wr_bytes, &fp_ops);
+			t2 = stress_time_now();
 			break;
 		case 2:
+			t1 = stress_time_now();
 			stress_stream_copy_index2(c, a, idx1, idx2, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_scale_index2(b, c, q, idx1, idx2, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_add_index2(c, b, a, idx1, idx2, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_triad_index2(a, b, c, q, idx1, idx2, n, &rd_bytes, &wr_bytes, &fp_ops);
+			t2 = stress_time_now();
 			break;
 		case 1:
+			t1 = stress_time_now();
 			stress_stream_copy_index1(c, a, idx1, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_scale_index1(b, c, q, idx1, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_add_index1(c, b, a, idx1, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_triad_index1(a, b, c, q, idx1, n, &rd_bytes, &wr_bytes, &fp_ops);
+			t2 = stress_time_now();
 			break;
 		case 0:
 		default:
 #if defined(HAVE_NT_STORE_DOUBLE)
 			if (has_sse2) {
+				t1 = stress_time_now();
 				stress_stream_copy_index0_nt(c, a, n, &rd_bytes, &wr_bytes, &fp_ops);
 				stress_stream_scale_index0_nt(b, c, q, n, &rd_bytes, &wr_bytes, &fp_ops);
 				stress_stream_add_index0_nt(c, b, a, n,  &rd_bytes, &wr_bytes, &fp_ops);
 				stress_stream_triad_index0_nt(a, b, c, q, n, &rd_bytes, &wr_bytes, &fp_ops);
+				t2 = stress_time_now();
 				break;
 			}
 #endif
+			t1 = stress_time_now();
 			stress_stream_copy_index0(c, a, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_scale_index0(b, c, q, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_add_index0(c, b, a, n, &rd_bytes, &wr_bytes, &fp_ops);
 			stress_stream_triad_index0(a, b, c, q, n, &rd_bytes, &wr_bytes, &fp_ops);
+			t2 = stress_time_now();
 			break;
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
-	t2 = stress_time_now();
+		dt += (t2 - t1);
+
+		if (verify) {
+			double new_checksum;
+
+			new_checksum = stress_stream_checksum_data(a, b, c, n);
+			if ((old_checksum > 0.0) && (fabs(new_checksum - old_checksum) > 0.001)) {
+				char new_str[32], old_str[32];
+
+				stress_stream_checksum_to_hexstr(new_str, sizeof(new_str), new_checksum);
+				stress_stream_checksum_to_hexstr(old_str, sizeof(old_str), old_checksum);
+
+				pr_fail("%s: checksum failure, got 0x%s, expecting 0x%s\n",
+					args->name, new_str, old_str);
+			} else {
+				old_checksum = new_checksum;
+			}
+		}
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	dt = t2 - t1;
 	if (dt >= 4.5) {
 		const double mb_rd_rate = (rd_bytes / (double)MB) / dt;
 		const double mb_wr_rate = (wr_bytes / (double)MB) / dt;
 		const double fp_rate = (fp_ops / 1000000.0) / dt;
-		pr_inf("%s: memory rate: %.2f MB read/sec, %.2f MB write/sec, %.2f Mflop/sec"
+
+		pr_inf("%s: memory rate: %.2f MB read/sec, %.2f MB write/sec, %.2f double precision Mflop/sec"
 			" (instance %" PRIu32 ")\n",
 			args->name, mb_rd_rate, mb_wr_rate, fp_rate, args->instance);
-		stress_metrics_set(args, 0, "memory read rate (MB per sec)", mb_rd_rate);
-		stress_metrics_set(args, 1, "memory write rate (MB per sec)", mb_wr_rate);
-		stress_metrics_set(args, 2, "memory rate (Mflop per sec)", fp_rate);
+		stress_metrics_set(args, 0, "MB per sec memory read rate",
+			mb_rd_rate, STRESS_HARMONIC_MEAN);
+		stress_metrics_set(args, 1, "MB per sec memory write rate",
+			mb_wr_rate, STRESS_HARMONIC_MEAN);
+		stress_metrics_set(args, 2, "Mflop per sec (double precision) compute rate",
+			fp_rate, STRESS_HARMONIC_MEAN);
 	} else {
 		if (args->instance == 0)
 			pr_inf("%s: run duration too short to determine memory rate\n", args->name);
 	}
 
 	rc = EXIT_SUCCESS;
-
+err_unmap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-
-	if (idx3)
+	if (idx3 != MAP_FAILED)
 		(void)munmap((void *)idx3, sz_idx);
-err_idx3:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	if (idx2)
+	if (idx2 != MAP_FAILED)
 		(void)munmap((void *)idx2, sz_idx);
-err_idx2:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	if (idx1)
+	if (idx1 != MAP_FAILED)
 		(void)munmap((void *)idx1, sz_idx);
-err_idx1:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)munmap((void *)c, sz);
-err_c:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)munmap((void *)b, sz);
-err_b:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)munmap((void *)a, sz);
-err_a:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-
+	if (c != MAP_FAILED)
+		(void)munmap((void *)c, sz);
+	if (b != MAP_FAILED)
+		(void)munmap((void *)b, sz);
+	if (a != MAP_FAILED)
+		(void)munmap((void *)a, sz);
 	return rc;
 }
 
@@ -894,6 +1037,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_stream_index,	stress_set_stream_index },
 	{ OPT_stream_l3_size,	stress_set_stream_L3_size },
 	{ OPT_stream_madvise,	stress_set_stream_madvise },
+	{ OPT_stream_mlock,	stress_set_stream_mlock },
 	{ 0,			NULL }
 };
 
@@ -901,5 +1045,6 @@ stressor_info_t stress_stream_info = {
 	.stressor = stress_stream,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
