@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King
+ * Copyright (C) 2022-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,10 +18,13 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-hash.h"
+#include "core-killpid.h"
 #include "core-pthread.h"
 #include "core-put.h"
+#include "core-try-open.h"
 
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
@@ -58,6 +61,8 @@ static const stress_help_t help[] = {
 
 static sigset_t set;
 static shim_pthread_spinlock_t lock;
+static shim_pthread_spinlock_t open_lock;
+static shim_pthread_spinlock_t hash_lock;
 static volatile bool drain_kmsg = false;
 static volatile uint32_t counter = 0;
 static const char signum_path[] = "/sys/kernel/notes";
@@ -66,12 +71,29 @@ static stress_hash_table_t *sysfs_hash_table;
 static uint64_t hash_items = 0;
 
 typedef struct {
-	const stress_args_t *args;	/* stressor args */
+	stress_args_t *args;	/* stressor args */
 	int kmsgfd;			/* /dev/kmsg file descriptor */
 	bool sys_admin;			/* true if sys admin capable */
 	char sysfs_path[PATH_MAX];	/* path to exercise */
+	uint64_t sysfs_files_opened;	/* count of sysfs files opened */
 } stress_ctxt_t;
 
+typedef struct {
+	const char *path;
+	void (*const sysfs_func)(const char *path);
+} stress_sysfs_wr_func_t;
+
+static void stress_sysfs_sys_power_disk(const char *path)
+{
+	(void)stress_system_write(path, "test_resume", 11);
+}
+
+static stress_sysfs_wr_func_t stress_sysfs_wr_funcs[] = {
+#if defined(__linux__)
+	{ "/sys/power/disk",	stress_sysfs_sys_power_disk },
+#endif
+};
+
 static inline uint32_t path_sum(const char *path)
 {
 	return stress_hash_x17(path);
@@ -117,19 +139,34 @@ static bool stress_kmsg_drain(const int fd)
  *  stress_sys_add_bad()
  *	add a path onto the bad (omit) hash table
  */
-static void stress_sys_add_bad(const char *path)
+static inline void stress_sys_add_bad(const char *path)
 {
-	if (shim_pthread_spin_lock(&lock))
+	if (shim_pthread_spin_lock(&hash_lock))
 		return;	/* Can't lock! */
-
 	if (!stress_hash_add(sysfs_hash_table, path))
 		hash_items++;
-	(void)shim_pthread_spin_unlock(&lock);
+	(void)shim_pthread_spin_unlock(&hash_lock);
+}
+
+/*
+ *  stress_sys_bad()
+ *	find str in hash table, if non-null it's bad
+ */
+static inline stress_hash_t *stress_sys_bad(stress_hash_table_t *hash_table, const char *str)
+{
+	stress_hash_t *hash;
+
+	if (shim_pthread_spin_lock(&hash_lock))
+		return NULL;	/* Can't lock! */
+	hash = stress_hash_get(hash_table, str);
+	(void)shim_pthread_spin_unlock(&hash_lock);
+
+	return hash;
 }
 
 /*
  *  stress_sys_rw()
- *	read a proc file
+ *	read a sys file
  */
 static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 {
@@ -137,11 +174,11 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 	ssize_t i = 0, ret;
 	char buffer[SYS_BUF_SZ];
 	char path[PATH_MAX];
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
 	const double threshold = 0.2;
 	size_t page_size = ctxt->args->page_size;
 
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		double t_start;
 		uint8_t *ptr;
 		fd_set rfds;
@@ -157,7 +194,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 		if (counter > OPS_PER_SYSFS_FILE)
 			shim_sched_yield();
 
-		if (!*path || !keep_stressing_flag())
+		if (!*path || !stress_continue_flag())
 			break;
 
 		t_start = stress_time_now();
@@ -170,6 +207,14 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 			stress_sys_add_bad(path);
 			goto next;
 		}
+		ret = shim_pthread_spin_lock(&open_lock);
+		if (ret) {
+			(void)close(fd);
+			return false;
+		}
+		ctxt->sysfs_files_opened++;
+		(void)shim_pthread_spin_unlock(&open_lock);
+
 		if (stress_time_now() - t_start > threshold) {
 			(void)close(fd);
 			goto next;
@@ -181,7 +226,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 		while (i < (4096 * SYS_BUF_SZ)) {
 			const ssize_t sz = 1 + stress_mwc32modn(sizeof(buffer) - 1);
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			rret = read(fd, buffer, (size_t)sz);
 			if (rret < 0)
@@ -205,7 +250,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			struct stat statbuf;
 
-			if (fstat(fd, &statbuf) < 0)
+			if (shim_fstat(fd, &statbuf) < 0)
 				pr_fail("%s: stat failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
@@ -241,7 +286,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 			goto next;
 
 		/*
-		 *  select on proc file
+		 *  select on sys file
 		 */
 		tv.tv_sec = 0;
 		tv.tv_usec = 0;
@@ -345,6 +390,17 @@ err:
 			if (stress_time_now() - t_start > threshold)
 				goto next;
 		} else {
+			/*
+			 * Special case for some sysfs entries
+			 */
+			size_t j;
+
+			for (j = 0; j < SIZEOF_ARRAY(stress_sysfs_wr_funcs); j++) {
+				if (strcmp(stress_sysfs_wr_funcs[j].path, path) == 0) {
+					stress_sysfs_wr_funcs[j].sysfs_func(path);
+				}
+			}
+
 #if 0
 			/*
 			 * Special case where we are root and file
@@ -352,7 +408,7 @@ err:
 			 */
 			const char *rom = strstr(path, "rom");
 
-			if (rom && rom[3] == '\0') {
+			if (rom && (rom[3] == '\0')) {
 				if ((fd = open(path, O_RDWR | O_NONBLOCK)) < 0)
 					goto next;
 				/* Enable ROM read */
@@ -399,7 +455,7 @@ static void *stress_sys_rw_thread(void *ctxt_ptr)
 {
 	static void *nowt = NULL;
 	stress_ctxt_t *ctxt = (stress_ctxt_t *)ctxt_ptr;
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
 
 	/*
 	 *  Block all signals, let controlling thread
@@ -407,7 +463,7 @@ static void *stress_sys_rw_thread(void *ctxt_ptr)
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (keep_stressing(args))
+	while (stress_continue(args))
 		stress_sys_rw(ctxt);
 
 	return &nowt;
@@ -467,11 +523,11 @@ static void stress_sys_dir(
 	const int depth)
 {
 	struct dirent **dlist = NULL;
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
 	mode_t flags = S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
 	int i, n;
 
-	if (!keep_stressing_flag())
+	if (!stress_continue_flag())
 		return;
 
 	/* Don't want to go too deep */
@@ -492,11 +548,11 @@ static void stress_sys_dir(
 		flags |= S_IRUSR | S_IWUSR;
 
 	/* Non-directories first */
-	for (i = 0; (i < n) && keep_stressing(args); i++) {
+	for (i = 0; (i < n) && stress_continue(args); i++) {
 		int ret;
 		struct stat buf;
 		char tmp[PATH_MAX];
-		struct dirent *d = dlist[i];
+		const struct dirent *d = dlist[i];
 		double time_start, time_end, time_out;
 
 		if (stress_is_dot_filename(d->d_name))
@@ -504,7 +560,7 @@ static void stress_sys_dir(
 
 		(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
 		/* Is it in the hash of bad paths? */
-		if (stress_hash_get(sysfs_hash_table, tmp))
+		if (stress_sys_bad(sysfs_hash_table, tmp))
 			goto dt_reg_free;
 
 		if (stress_sys_skip(tmp))
@@ -513,7 +569,7 @@ static void stress_sys_dir(
 		if (d->d_type != DT_REG)
 			continue;
 
-		ret = stat(tmp, &buf);
+		ret = shim_stat(tmp, &buf);
 		if (ret < 0)
 			goto dt_reg_free;
 
@@ -545,9 +601,9 @@ static void stress_sys_dir(
 			/* Cater for slower delays */
 			if ((counter > 0) && (stress_time_now() > time_end))
 				break;
-		} while ((counter < OPS_PER_SYSFS_FILE) && keep_stressing(args));
+		} while ((counter < OPS_PER_SYSFS_FILE) && stress_continue(args));
 
-		inc_counter(args);
+		stress_bogo_inc(args);
 dt_reg_free:
 		free(dlist[i]);
 		dlist[i] = NULL;
@@ -559,8 +615,8 @@ dt_reg_free:
 	}
 
 	/* Now directories.. */
-	for (i = 0; (i < n) && keep_stressing(args); i++) {
-		struct dirent *d = dlist[i];
+	for (i = 0; (i < n) && stress_continue(args); i++) {
+		const struct dirent *d = dlist[i];
 		struct stat buf;
 		int ret;
 		char tmp[PATH_MAX];
@@ -571,13 +627,13 @@ dt_reg_free:
 			goto dt_dir_free;
 
 		(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
-		ret = stat(tmp, &buf);
+		ret = shim_stat(tmp, &buf);
 		if (ret < 0)
 			goto dt_dir_free;
 		if ((buf.st_mode & flags) == 0)
 			goto dt_dir_free;
 
-		inc_counter(args);
+		stress_bogo_inc(args);
 		stress_sys_dir(ctxt, tmp, recurse, depth + 1);
 dt_dir_free:
 		free(dlist[i]);
@@ -626,15 +682,16 @@ static bool stress_sysfs_bad_signal(const int status)
  *  stress_sysfs
  *	stress reading all of /sys
  */
-static int stress_sysfs(const stress_args_t *args)
+static int stress_sysfs(stress_args_t *args)
 {
 	int i, n, rc = EXIT_SUCCESS;
 	pthread_t pthreads[MAX_SYSFS_THREADS];
 	int ret, pthreads_ret[MAX_SYSFS_THREADS];
 	stress_ctxt_t *ctxt;
 	struct dirent **dlist = NULL;
+	double t, duration, rate;
 
-	ctxt = (stress_ctxt_t *)mmap(NULL, sizeof(*ctxt),
+	ctxt = (stress_ctxt_t *)stress_mmap_populate(NULL, sizeof(*ctxt),
 				     PROT_READ | PROT_WRITE,
 				     MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (ctxt == MAP_FAILED) {
@@ -659,7 +716,7 @@ static int stress_sysfs(const stress_args_t *args)
 		if (ret == 0) {
 			uint16_t major, minor;
 
-			if (sscanf(utsbuf.release, "%5" SCNd16 ".%5" SCNd16, &major, &minor) == 2)
+			if (sscanf(utsbuf.release, "%5" SCNu16 ".%5" SCNu16, &major, &minor) == 2)
 				os_release = (major * 100) + minor;
 		}
 	}
@@ -670,13 +727,12 @@ static int stress_sysfs(const stress_args_t *args)
 	if (!sysfs_hash_table) {
 		pr_err("%s: cannot create sysfs hash table: %d (%s))\n",
 			args->name, errno, strerror(errno));
-		stress_dirent_list_free(dlist, n);
-		(void)munmap((void *)ctxt, sizeof(*ctxt));
-		return EXIT_NO_RESOURCE;
+		rc = EXIT_NO_RESOURCE;
+		goto exit_free;
 	}
 
-	(void)memset(ctxt, 0, sizeof(*ctxt));
-	shim_strlcpy(ctxt->sysfs_path, signum_path, sizeof(ctxt->sysfs_path));
+	(void)shim_memset(ctxt, 0, sizeof(*ctxt));
+	(void)shim_strlcpy(ctxt->sysfs_path, signum_path, sizeof(ctxt->sysfs_path));
 
 	ctxt->args = args;
 	ctxt->kmsgfd = open("/dev/kmsg", O_RDONLY | O_NONBLOCK);
@@ -685,17 +741,27 @@ static int stress_sysfs(const stress_args_t *args)
 
 	ret = shim_pthread_spin_init(&lock, PTHREAD_PROCESS_PRIVATE);
 	if (ret) {
-		pr_inf("%s: pthread_spin_init failed, errno=%d (%s)\n",
+		pr_inf("%s: pthread_spin_init on lock failed, errno=%d (%s)\n",
 			args->name, ret, strerror(ret));
-		if (ctxt->kmsgfd != -1)
-			(void)close(ctxt->kmsgfd);
-		stress_hash_delete(sysfs_hash_table);
-		stress_dirent_list_free(dlist, n);
-		(void)munmap((void *)ctxt, sizeof(*ctxt));
-		return EXIT_NO_RESOURCE;
+		rc = EXIT_NO_RESOURCE;
+		goto exit_delete_hash;
+	}
+	ret = shim_pthread_spin_init(&open_lock, PTHREAD_PROCESS_PRIVATE);
+	if (ret) {
+		pr_inf("%s: pthread_spin_init on open_lock failed, errno=%d (%s)\n",
+			args->name, ret, strerror(ret));
+		rc = EXIT_NO_RESOURCE;
+		goto exit_destroy_lock;
+	}
+	ret = shim_pthread_spin_init(&hash_lock, PTHREAD_PROCESS_PRIVATE);
+	if (ret) {
+		pr_inf("%s: pthread_spin_init on hash_lock failed, errno=%d (%s)\n",
+			args->name, ret, strerror(ret));
+		rc = EXIT_NO_RESOURCE;
+		goto exit_destroy_open_lock;
 	}
 
-	(void)memset(pthreads_ret, 0, sizeof(pthreads_ret));
+	(void)shim_memset(pthreads_ret, 0, sizeof(pthreads_ret));
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -711,15 +777,16 @@ static int stress_sysfs(const stress_args_t *args)
 	 *  if the child was terminated prematurely by an unexpected
 	 *  signal from the kernel.
 	 */
+	t = stress_time_now();
 	do {
 		pid_t pid;
 
 again:
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args)) {
+			if (!stress_continue(args)) {
 				rc = EXIT_SUCCESS;
 				goto finish;
 			}
@@ -733,8 +800,7 @@ again:
 					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				/* Ring ring, time to die */
-				(void)kill(pid, SIGALRM);
-				VOID_RET(int, shim_waitpid(pid, &status, 0));
+				stress_kill_and_wait(args, pid, SIGALRM, true);
 			} else {
 				if (stress_sysfs_bad_signal(status)) {
 					pr_inf("%s: killed by %s exercising '%s'\n",
@@ -753,6 +819,8 @@ again:
 		} else {
 			/* Child, spawn threads for sysfs stressing */
 
+			stress_parent_died_alarm();
+
 			for (i = 0; i < MAX_SYSFS_THREADS; i++) {
 				pthreads_ret[i] = pthread_create(&pthreads[i], NULL,
 								stress_sys_rw_thread, ctxt);
@@ -760,12 +828,12 @@ again:
 
 			do {
 				int j = (int)args->instance % n;
-				const int inc = (int)stress_get_prime64((((uint64_t)(args->instance + 1) * 50U) + 1200));
+				const int inc = (int)stress_get_next_prime64((((uint64_t)(args->instance + 1) * 50U) + 1200));
 
 				for (i = 0; i < n; i++) {
 					char sysfspath[PATH_MAX];
 
-					if (!keep_stressing(args))
+					if (!stress_continue(args))
 						break;
 
 					if (stress_is_dot_filename(dlist[j]->d_name))
@@ -777,20 +845,22 @@ again:
 					stress_sys_dir(ctxt, sysfspath, true, 0);
 					j = (j + inc) % n;
 				}
-			} while (keep_stressing(args));
+			} while (stress_continue(args));
 
 			ret = shim_pthread_spin_lock(&lock);
 			if (ret) {
 				pr_dbg("%s: failed to lock spin lock for sysfs_path\n", args->name);
 			} else {
-				shim_strlcpy(ctxt->sysfs_path, "", sizeof(ctxt->sysfs_path));
+				(void)shim_strlcpy(ctxt->sysfs_path, "", sizeof(ctxt->sysfs_path));
 				VOID_RET(int, shim_pthread_spin_unlock(&lock));
 			}
 
 			/* Forcefully kill threads */
 			for (i = 0; i < MAX_SYSFS_THREADS; i++) {
-				if (pthreads_ret[i] == 0)
+				if (pthreads_ret[i] == 0) {
+					stress_force_killed_bogo(args);
 					(void)pthread_kill(pthreads[i], SIGKILL);
+				}
 			}
 
 			for (i = 0; i < MAX_SYSFS_THREADS; i++) {
@@ -800,17 +870,28 @@ again:
 
 			_exit(EXIT_SUCCESS);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
+	duration = stress_time_now() - t;
 
 	pr_dbg("%s: skipped %" PRIu64 " out of %" PRIu64 " sysfs files accessed\n",
-		args->name, hash_items, get_counter(args));
+		args->name, hash_items, stress_bogo_get(args));
+
+	rate = (duration > 0.0) ? (double)ctxt->sysfs_files_opened / duration : 0.0;
+	stress_metrics_set(args, 0, "sysfs files exercised per sec",
+		rate, STRESS_HARMONIC_MEAN);
+
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	(void)shim_pthread_spin_destroy(&hash_lock);
+exit_destroy_open_lock:
+	(void)shim_pthread_spin_destroy(&open_lock);
+exit_destroy_lock:
+	(void)shim_pthread_spin_destroy(&lock);
+exit_delete_hash:
 	stress_hash_delete(sysfs_hash_table);
 	if (ctxt->kmsgfd != -1)
 		(void)close(ctxt->kmsgfd);
-	(void)shim_pthread_spin_destroy(&lock);
 
 exit_free:
 	stress_dirent_list_free(dlist, n);
@@ -835,6 +916,7 @@ stressor_info_t stress_sysfs_info = {
 stressor_info_t stress_sysfs_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "not Linux or built without pthread support"
 };
