@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,9 +18,23 @@
  *
  */
 #include "stress-ng.h"
+#include "core-killpid.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
 
 #define WASTE_SIZE	(64 * MB)
 
+typedef struct {
+	volatile uint64_t invoked;	/* count of vfork processes that started */
+	uint64_t	waited;		/* count of vfork processes waited for */
+	volatile bool 	terminate;	/* true to indicate it's time to stop */
+	double		t1;		/* time before vfork */
+	double		t2;		/* time once vfork process started */
+	double		duration;	/* total duration of vfork invocations */
+	uint64_t	counter;	/* number duration measurements made */
+} vforkmany_shared_t;
+
 static const stress_help_t help[] = {
 	{ NULL,	"vforkmany N",     "start N workers spawning many vfork children" },
 	{ NULL,	"vforkmany-ops N", "stop after spawning N vfork children" },
@@ -30,28 +44,26 @@ static const stress_help_t help[] = {
 
 static int stress_set_vforkmany_vm(const char *opt)
 {
-	bool vm = true;
-
-	(void)opt;
-
-	return stress_set_setting("vforkmany-vm", TYPE_ID_BOOL, &vm);
+	return stress_set_setting_true("vforkmany-vm", opt);
 }
 
 /*
  *  vforkmany_wait()
  *	wait and then kill
  */
-static void vforkmany_wait(const pid_t pid)
+static void vforkmany_wait(vforkmany_shared_t *vforkmany_shared, const pid_t pid)
 {
 	for (;;) {
 		int ret, status;
 
 		errno = 0;
 		ret = waitpid(pid, &status, 0);
-		if ((ret >= 0) || (errno != EINTR))
+		if ((ret >= 0) || (errno != EINTR)) {
+			vforkmany_shared->waited++;
 			break;
+		}
 
-		(void)kill(pid, SIGALRM);
+		(void)shim_kill(pid, SIGALRM);
 	}
 }
 
@@ -64,18 +76,22 @@ static void vforkmany_wait(const pid_t pid)
  *	careful not to overwrite shared variables across
  *	all the processes.
  */
-static int stress_vforkmany(const stress_args_t *args)
+static int stress_vforkmany(stress_args_t *args)
 {
+	/* avoid variables on stack since we're using vfork */
 	static pid_t chpid;
 	static uint8_t *stack_sig;
-	static volatile bool *terminate;
-	static bool *terminate_mmap;
 	static bool vm = false;
+	static vforkmany_shared_t *vforkmany_shared;
+	static int rc = EXIT_SUCCESS;
 
 	(void)stress_get_setting("vforkmany-vm", &vm);
 
+	stress_ksm_memory_merge(1);
+
 	/* We should use an alternative signal stack */
-	stack_sig = (uint8_t *)mmap(NULL, STRESS_SIGSTKSZ, PROT_READ | PROT_WRITE,
+	stack_sig = (uint8_t *)stress_mmap_populate(NULL, STRESS_SIGSTKSZ,
+			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (stack_sig == MAP_FAILED) {
 		pr_inf_skip("%s: skipping stressor, cannot allocate signal stack,"
@@ -86,40 +102,48 @@ static int stress_vforkmany(const stress_args_t *args)
 	if (stress_sigaltstack(stack_sig, STRESS_SIGSTKSZ) < 0)
 		return EXIT_FAILURE;
 
-	terminate = terminate_mmap =
-		(bool *)mmap(NULL, args->page_size,
-				PROT_READ | PROT_WRITE,
-				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
-	if (terminate_mmap == MAP_FAILED) {
+	vforkmany_shared = (vforkmany_shared_t *)
+		stress_mmap_populate(NULL, sizeof(*vforkmany_shared),
+			PROT_READ | PROT_WRITE,
+			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+	if (vforkmany_shared == MAP_FAILED) {
 		pr_inf("%s: mmap failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
+		VOID_RET(int, stress_sigaltstack(NULL, 0));
+		(void)munmap((void *)stack_sig, STRESS_SIGSTKSZ);
 		return EXIT_NO_RESOURCE;
 	}
-	*terminate = false;
+	vforkmany_shared->terminate = false;
+	vforkmany_shared->invoked = false;
+	vforkmany_shared->waited = false;
+	vforkmany_shared->t1 = 0.0;
+	vforkmany_shared->t2 = 0.0;
+	vforkmany_shared->duration = 0.0;
+	vforkmany_shared->counter = 0;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 fork_again:
 	chpid = fork();
 	if (chpid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto fork_again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_err("%s: fork failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
-		(void)munmap((void *)terminate_mmap, args->page_size);
+		(void)munmap((void *)vforkmany_shared, sizeof(*vforkmany_shared));
+		VOID_RET(int, stress_sigaltstack(NULL, 0));
+		(void)munmap((void *)stack_sig, STRESS_SIGSTKSZ);
 		return EXIT_FAILURE;
 	} else if (chpid == 0) {
 		static uint8_t *waste;
 		static size_t waste_size = WASTE_SIZE;
 
-		(void)setpgid(0, g_pgrp);
-
 		/*
 		 *  We want the children to be OOM'd if we
 		 *  eat up too much memory
 		 */
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 		stress_parent_died_alarm();
 
 		/*
@@ -134,7 +158,7 @@ fork_again:
 			if (waste != MAP_FAILED)
 				break;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				_exit(0);
 
 			waste_size >>= 1;
@@ -142,7 +166,7 @@ fork_again:
 
 		if (waste != MAP_FAILED)
 			(void)stress_mincore_touch_pages_interruptible(waste, WASTE_SIZE);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			_exit(0);
 
 		if (vm) {
@@ -179,8 +203,8 @@ vfork_again:
 			 * instead poll the run time and break out
 			 * of the loop if we've run out of run time
 			 */
-			if (*terminate) {
-				keep_stressing_set_flag(false);
+			if (vforkmany_shared->terminate) {
+				stress_continue_set_flag(false);
 				break;
 			}
 			if (start_pid == -1) {
@@ -188,14 +212,25 @@ vfork_again:
 				if (pid >= 0)
 					start_pid = getpid();
 			} else {
+				vforkmany_shared->t1 = stress_time_now();
 				pid = shim_vfork();
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 			if (pid < 0) {
 				/* failed */
 				shim_sched_yield();
 				_exit(0);
 			} else if (pid == 0) {
+				vforkmany_shared->invoked++;
+				if (vforkmany_shared->t1 > 0.0) {
+					vforkmany_shared->t2 = stress_time_now();
+					if (vforkmany_shared->t2 > vforkmany_shared->t1) {
+						vforkmany_shared->counter++;
+						vforkmany_shared->duration +=
+							vforkmany_shared->t2 - vforkmany_shared->t1;
+					}
+				}
+
 				if (vm) {
 					int flags = 0;
 
@@ -218,18 +253,18 @@ vfork_again:
 					(void)stress_mincore_touch_pages_interruptible(waste, WASTE_SIZE);
 
 				/* child, parent is blocked, spawn new child */
-				if (!args->max_ops || (get_counter(args) < args->max_ops))
+				if (!args->max_ops || (stress_bogo_get(args) < args->max_ops))
 					goto vfork_again;
 				_exit(0);
 			} else {
 				/* parent, wait for child, and exit if not first parent */
 				if (pid >= 1)
-					(void)vforkmany_wait(pid);
+					(void)vforkmany_wait(vforkmany_shared, pid);
 				shim_sched_yield();
 				if (getpid() != start_pid)
 					_exit(0);
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
 		if (waste != MAP_FAILED)
 			(void)munmap((void *)waste, WASTE_SIZE);
@@ -242,36 +277,35 @@ vfork_again:
 		 * see and will then exit.  We wait for the first
 		 * one spawned to unblock and exit
 		 */
-		(void)setpgid(chpid, g_pgrp);
 		g_opt_flags &= ~OPT_FLAGS_OOMABLE;
-		stress_set_oom_adjustment(args->name, false);
+		stress_set_oom_adjustment(args, false);
 
 		(void)sleep((unsigned int)g_opt_timeout);
-		*terminate = true;
+		vforkmany_shared->terminate = true;
 
-		for (;;) {
-			int ret, chstatus;
-
-			(void)kill(chpid, SIGALRM);
-			errno = 0;
-			ret = waitpid(chpid, &chstatus, 0);
-
-			/* Reaped? - all done */
-			if (ret >= 0)
-				break;
-			/* Interrupted? - retry */
-			if (errno == EINTR)
-				continue;
-			/* Something went wrong, kill */
-			(void)stress_killpid(chpid);
-		}
+		stress_kill_and_wait(args, chpid, SIGALRM, false);
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)munmap((void *)terminate_mmap, args->page_size);
+
+	if (vforkmany_shared->counter) {
+		double rate = vforkmany_shared->duration / (double)vforkmany_shared->counter;
+
+		stress_metrics_set(args, 0, "nanosecs to start vfork'd a process",
+			rate * 1000000000.0, STRESS_HARMONIC_MEAN);
+	}
+	if ((vforkmany_shared->waited > 0) && (vforkmany_shared->invoked == 0)) {
+		pr_fail("%s: no vfork'd processes got fully invoked correctly "
+			"before they terminated\n", args->name);
+		rc = EXIT_FAILURE;
+	}
+
+	VOID_RET(int, stress_sigaltstack(NULL, 0));
 	(void)munmap((void *)stack_sig, STRESS_SIGSTKSZ);
-	return EXIT_SUCCESS;
+	(void)munmap((void *)vforkmany_shared, sizeof(*vforkmany_shared));
+
+	return rc;
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -283,5 +317,6 @@ stressor_info_t stress_vforkmany_info = {
 	.stressor = stress_vforkmany,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
